@@ -1,18 +1,18 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Author:
- *   Chris Toshok (toshok@helixcode.com)
+ *   Chris Toshok (toshok@ximian.com)
  *
- * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  */
 
 #define DEBUG
 
 #include "config.h"  
-#include <gtk/gtksignal.h>
 #include <fcntl.h>
 #include <time.h>
 #include <lber.h>
+#include <gtk/gtksignal.h>
 
 #ifdef DEBUG
 #define LDAP_DEBUG
@@ -29,22 +29,17 @@
 #define OPENLDAP1
 #endif
 
-#if 0
-#ifdef OPENLDAP1
-#define LDAP_NAME_ERROR(x) NAME_ERROR(x)
-#endif
-#endif
-
 #ifdef OPENLDAP2
 #include "ldap_schema.h"
 #endif
 
+#include <e-util/e-sexp.h>
+#include <ebook/e-card-simple.h>
+
 #include "pas-backend-ldap.h"
 #include "pas-book.h"
 #include "pas-card-cursor.h"
 
-#include <e-util/e-sexp.h>
-#include <ebook/e-card-simple.h>
 
 #define LDAP_MAX_SEARCH_RESPONSES 100
 
@@ -248,7 +243,7 @@ view_destroy(GtkObject *object, gpointer data)
 				   ldap connection.  remove the idle
 				   handler and anbandon the msg id */
 				g_source_remove(view->search_idle);
-				pas_book_view_notify_status_message (view->book_view, "Abandoning pending search");
+				pas_book_view_notify_status_message (view->book_view, "Abandoning pending search.");
 				if (view->search_msgid != -1)
 					ldap_abandon (bl->priv->ldap, view->search_msgid);
 
@@ -401,6 +396,11 @@ pas_backend_ldap_connect (PASBackendLDAP *bl)
 		ldap_simple_bind_s(blpriv->ldap,
 				   NULL /*binddn*/, NULL /*passwd*/);
 		blpriv->connected = TRUE;
+
+		/* check to see if evolutionPerson is supported, if we can (me
+		   might not be able to if we can't authenticate.  if we
+		   can't, try again in auth_user.) */
+		check_schema_support (bl);
 	}
 	else {
 		g_warning ("pas_backend_ldap_connect failed for "
@@ -411,10 +411,6 @@ pas_backend_ldap_connect (PASBackendLDAP *bl)
 		blpriv->connected = FALSE;
 	}
 
-	/* check to see if evolutionPerson is supported, if we can (me
-           might not be able to if we can't authenticate.  if we
-           can't, try again in auth_user.) */
-	check_schema_support (bl);
 }
 
 static void
@@ -437,7 +433,7 @@ ldap_op_process_current (PASBackend *backend)
 
 	if (!bl->priv->connected) {
 		if (op->view)
-			pas_book_view_notify_status_message (op->view, "Connecting to LDAP server");
+			pas_book_view_notify_status_message (op->view, "Connecting to LDAP server...");
 		pas_backend_ldap_connect(bl);
 	}
 
@@ -446,8 +442,10 @@ ldap_op_process_current (PASBackend *backend)
 			ldap_op_finished (op);
 	}
 	else {
-		if (op->view)
-			pas_book_view_notify_status_message (op->view, "Unable to connect to LDAP server");
+		if (op->view) {
+			pas_book_view_notify_complete (op->view);
+			pas_book_view_notify_status_message (op->view, "Unable to connect to LDAP server.");
+		}
 
 		ldap_op_finished (op);
 	}
@@ -526,6 +524,8 @@ ldap_error_to_response (int ldap_error)
 		return GNOME_Evolution_Addressbook_BookListener_PermissionDenied;
 	else if (ldap_error == LDAP_SERVER_DOWN)
 		return GNOME_Evolution_Addressbook_BookListener_RepositoryOffline;
+	else if (ldap_error == LDAP_ALREADY_EXISTS)
+		return GNOME_Evolution_Addressbook_BookListener_CardIdAlreadyExists;
 	else
 		return GNOME_Evolution_Addressbook_BookListener_OtherError;
 }
@@ -827,7 +827,7 @@ create_card_handler (PASBackend *backend, LDAPOp *op)
 	ldap_mods = (LDAPMod**)mod_array->pdata;
 
 	if (op->view)
-		pas_book_view_notify_status_message (op->view, "Adding card to LDAP server");
+		pas_book_view_notify_status_message (op->view, "Adding card to LDAP server...");
 
 	/* actually perform the ldap add */
 	ldap_error = ldap_add_s (ldap, dn, ldap_mods);
@@ -954,29 +954,14 @@ typedef struct {
 static ECardSimple *
 search_for_dn (PASBackendLDAP *bl, const char *dn)
 {
-#if 0
-	char **attrs;
-#endif
-	char *query;
 	LDAP *ldap = bl->priv->ldap;
 	LDAPMessage    *res, *e;
 	ECardSimple *result = NULL;
 
-#if 0
-	/* this is broken because if we (say) modify the cn and can't
-           modify the dn (because it overlaps), we lose the ability to
-           search by that component of the dn. */
-	attrs = ldap_explode_dn (dn, 0);
-	query = g_strdup_printf ("(%s)", attrs[0]);
-	printf ("searching for %s\n", query);
-#else
-	query = g_strdup ("(objectclass=*)");
-#endif
-
 	if (ldap_search_s (ldap,
-			   bl->priv->ldap_rootdn,
-			   bl->priv->ldap_scope,
-			   query,
+			   dn,
+			   LDAP_SCOPE_BASE,
+			   "(objectclass=*)",
 			   NULL, 0, &res) != -1) {
 		e = ldap_first_entry (ldap, res);
 		while (NULL != e) {
@@ -991,7 +976,6 @@ search_for_dn (PASBackendLDAP *bl, const char *dn)
 		ldap_msgfree(res);
 	}
 
-	g_free (query);
 	return result;
 }
 
@@ -1001,7 +985,7 @@ modify_card_handler (PASBackend *backend, LDAPOp *op)
 	LDAPModifyOp *modify_op = (LDAPModifyOp*)op;
 	PASBackendLDAP *bl = PAS_BACKEND_LDAP (backend);
 	ECard *new_ecard;
-	char *id;
+	const char *id;
 	int response;
 	int            ldap_error = LDAP_SUCCESS;
 	GPtrArray *mod_array;
@@ -1568,7 +1552,7 @@ func_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data
 			match_str = g_strdup_printf("=*%s%s)",
 						    str, one_star ? "" : "*");
 
-			query_length = strlen (header);
+			query_length = strlen (header) + strlen (footer);
 
 			for (i = 0; i < num_prop_infos; i ++) {
 				query_length += 1 + strlen(prop_info[i].ldap_attr) + strlen (match_str);
@@ -1624,6 +1608,12 @@ func_is(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 			*list = g_list_prepend(*list,
 					       g_strdup_printf("(%s=%s)",
 							       ldap_attr, str));
+		else {
+			g_warning ("unknown query property\n");
+			/* we want something that'll always be false */
+			*list = g_list_prepend(*list,
+					       g_strdup("objectClass=MyBarnIsBiggerThanYourBarn"));
+		}
 	}
 
 	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
@@ -1737,13 +1727,19 @@ pas_backend_ldap_build_query (gchar *query)
 	e_sexp_result_free(sexp, r);
 	e_sexp_unref (sexp);
 
-	if (list->next) {
-		g_warning ("conversion to ldap query string failed");
-		retval = NULL;
-		g_list_foreach (list, (GFunc)g_free, NULL);
+	if (list) {
+		if (list->next) {
+			g_warning ("conversion to ldap query string failed");
+			retval = NULL;
+			g_list_foreach (list, (GFunc)g_free, NULL);
+		}
+		else {
+			retval = list->data;
+		}
 	}
 	else {
-		retval = list->data;
+		g_warning ("conversion to ldap query string failed");
+		retval = NULL;
 	}
 
 	g_list_free (list);
@@ -1837,12 +1833,12 @@ poll_ldap (LDAPSearchOp *op)
 	GList   *cards = NULL;
 	static int received = 0;
 
-	pas_book_view_notify_status_message (view->book_view, "Polling for LDAP search result");
+	pas_book_view_notify_status_message (view->book_view, "Receiving LDAP search results...");
 
 	rc = ldap_result (ldap, view->search_msgid, 0, NULL, &res);
 	
 	if (rc == -1 && received == 0) {
-		pas_book_view_notify_status_message (view->book_view, "Restarting search");
+		pas_book_view_notify_status_message (view->book_view, "Restarting search.");
 		/* connection went down and we never got any. */
 		bl->priv->connected = FALSE;
 
@@ -1856,7 +1852,7 @@ poll_ldap (LDAPSearchOp *op)
 		pas_book_view_notify_complete (view->book_view);
 		ldap_op_finished ((LDAPOp*)op);
 		received = 0;
-		pas_book_view_notify_status_message (view->book_view, "Search complete");
+		pas_book_view_notify_status_message (view->book_view, "Search complete.");
 		return FALSE;
 	}
 
