@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-ldap-storage.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,24 +50,28 @@
 #include <config.h>
 #endif
 
-#include <bonobo.h>
-
-#include "evolution-shell-component.h"
-#include "evolution-storage.h"
-
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
-
-#include "addressbook-storage.h"
-
-#include <gal/util/e-util.h>
-#include <gal/util/e-xml-utils.h>
-
 #include <sys/types.h>
 #include <sys/fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
 
+#include <glib.h>
+#include <gnome-xml/parser.h>
+#include <gnome-xml/xmlmemory.h>
+
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-object.h>
+
+#include <gal/util/e-util.h>
+#include <gal/util/e-xml-utils.h>
+#include <libgnome/gnome-i18n.h>
+
+#include "evolution-shell-component.h"
+#include "evolution-storage.h"
+
+#include "addressbook-storage.h"
+
 #define ADDRESSBOOK_SOURCES_XML "addressbook-sources.xml"
 
 static gboolean load_source_data (EvolutionStorage *storage, const char *file_path);
@@ -139,8 +143,6 @@ ldap_unparse_auth (AddressbookLDAPAuthType auth_type)
 		return "none";
 	case ADDRESSBOOK_LDAP_AUTH_SIMPLE:
 		return "simple";
-	case ADDRESSBOOK_LDAP_AUTH_SASL:
-		return "sasl";
 	default:
 		g_assert(0);
 		return "none";
@@ -155,8 +157,6 @@ ldap_parse_auth (const char *auth)
 
 	if (!strcmp (auth, "simple"))
 		return ADDRESSBOOK_LDAP_AUTH_SIMPLE;
-	else if (!strcmp (auth, "sasl"))
-		return ADDRESSBOOK_LDAP_AUTH_SASL;
 	else
 		return ADDRESSBOOK_LDAP_AUTH_NONE;
 }
@@ -413,8 +413,6 @@ addressbook_storage_add_source (AddressbookSource *source)
 				      source->uri, source->description, FALSE);
 
 	g_free (path);
-
-	save_source_data (storage_path);
 }
 
 void
@@ -446,8 +444,6 @@ addressbook_storage_remove_source (const char *name)
 	evolution_storage_removed_folder (storage, path);
 
 	g_free (path);
-
-	save_source_data (storage_path);
 }
 
 GList *
@@ -508,6 +504,12 @@ addressbook_storage_clear_sources ()
 	sources = NULL;
 }
 
+void
+addressbook_storage_write_sources ()
+{
+	save_source_data (storage_path);
+}
+
 AddressbookSource *
 addressbook_source_copy (const AddressbookSource *source)
 {
@@ -525,7 +527,7 @@ addressbook_source_copy (const AddressbookSource *source)
 		copy->ldap.rootdn = g_strdup (source->ldap.rootdn);
 		copy->ldap.scope = source->ldap.scope;
 		copy->ldap.auth = source->ldap.auth;
-		copy->ldap.binddn = source->ldap.binddn;
+		copy->ldap.binddn = g_strdup (source->ldap.binddn);
 		copy->ldap.remember_passwd = source->ldap.remember_passwd;
 	}
 	else {
