@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-select-names-bonobo.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,20 +25,21 @@
 #include <config.h>
 #endif
 
-#include <gnome.h>
+#include "e-select-names-bonobo.h"
 
 #include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-control.h>
 
-#include "Evolution-Addressbook-SelectNames.h"
-
 #include <gal/util/e-util.h>
 #include <gal/e-text/e-entry.h>
+
+#include "Evolution-Addressbook-SelectNames.h"
+
 #include "e-select-names-manager.h"
+
 #include "e-select-names-model.h"
 #include "e-select-names-text-model.h"
 
-#include "e-select-names-bonobo.h"
 
 
 #define PARENT_TYPE bonobo_object_get_type ()
@@ -65,7 +66,7 @@ entry_get_property_fn (BonoboPropertyBag *bag,
 		       void *user_data)
 {
 	GtkWidget *w;
-	const char *text;
+	char *text;
 
 	w = GTK_WIDGET (user_data);
 
@@ -81,8 +82,9 @@ entry_get_property_fn (BonoboPropertyBag *bag,
 			gtk_object_get(GTK_OBJECT(text_model),
 				       "source", &model,
 				       NULL);
-			text = e_select_names_model_get_address_text (model);
+			text = e_select_names_model_export_destinationv (model);
 			BONOBO_ARG_SET_STRING (arg, text);
+			g_free (text);
 		}
 		break;
 	default:
