@@ -26,9 +26,22 @@
  */
 
 #include <config.h>
+
 #include <stdio.h>
-#include <gnome.h>
-#include <addressbook/contact-editor/e-contact-quick-add.h>
+#include <string.h>
+
+#include <glib.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-app-helper.h>
+#include <libgnomeui/gnome-popup-menu.h>
+
+#include <addressbook/backend/ebook/e-book-util.h>
+#include <addressbook/gui/contact-editor/e-contact-editor.h>
+#include <addressbook/gui/contact-editor/e-contact-quick-add.h>
+#include "e-addressbook-util.h"
 #include "e-select-names-popup.h"
 
 typedef struct _PopupInfo PopupInfo;
@@ -78,10 +91,32 @@ popup_info_cleanup (GtkWidget *w, gpointer info)
 	popup_info_free ((PopupInfo *) info);
 }
 
-static void
-do_nothing (gpointer foo)
-{
+/* You are in a maze of twisty little callbacks, all alike... */
 
+static void
+make_contact_editor_cb (EBook *book, gpointer user_data)
+{
+	if (book) {
+		EDestination *dest = E_DESTINATION (user_data);
+		EContactEditor *ce;
+		ECard *card;
+
+		card = (ECard *) e_destination_get_card (dest);
+		ce = e_addressbook_show_contact_editor (book, card, FALSE, TRUE);
+		e_contact_editor_raise (ce);
+		gtk_object_unref (GTK_OBJECT (dest));
+	}
+}
+
+static void
+edit_contact_info_cb (GtkWidget *w, gpointer user_data)
+{
+	PopupInfo *info = (PopupInfo *) user_data;
+	if (info == NULL)
+		return;
+
+	gtk_object_ref (GTK_OBJECT (info->dest));
+	e_book_use_local_address_book (make_contact_editor_cb, (gpointer) info->dest);
 }
 
 static void
@@ -137,6 +172,39 @@ add_remove_all_recipients (GnomeUIInfo *uiinfo, PopupInfo *info)
 	uiinfo->moreinfo = remove_all_recipients_cb;
 }
 
+static void
+toggle_html_mail_cb (GtkWidget *w, gpointer user_data)
+{
+	PopupInfo *info = (PopupInfo *) user_data;
+	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM (w);
+	const EDestination *dest;
+
+	if (info == NULL)
+		return;
+
+	dest = info->dest;
+
+	item = GTK_CHECK_MENU_ITEM (item);
+	e_destination_set_html_mail_pref ((EDestination *) dest, item->active);
+}
+
+static void
+add_html_mail (GnomeUIInfo *uiinfo, PopupInfo *info)
+{
+	uiinfo->type = GNOME_APP_UI_TOGGLEITEM;
+	uiinfo->label = _("Send HTML Mail?");
+	uiinfo->moreinfo = toggle_html_mail_cb;
+}
+
+static void
+init_html_mail (GnomeUIInfo *uiinfo, PopupInfo *info)
+{
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (uiinfo->widget),
+					e_destination_get_html_mail_pref (info->dest));
+	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (uiinfo->widget), TRUE);
+
+}
+
 #define ARBITRARY_UIINFO_LIMIT 64
 static GtkWidget *
 popup_menu_card (PopupInfo *info)
@@ -148,7 +216,7 @@ popup_menu_card (PopupInfo *info)
 	gint i=0;
 	GtkWidget *pop;
 	EIterator *iterator;
-	gchar *name_str;
+	gint html_toggle;
 
 	/*
 	 * Build up our GnomeUIInfo array.
@@ -158,10 +226,9 @@ popup_menu_card (PopupInfo *info)
 	memset (radioinfo, 0, sizeof (radioinfo));
 
 	card = e_destination_get_card (info->dest);
-	name_str = e_card_name_to_string (card->name);
 
 	uiinfo[i].type = GNOME_APP_UI_ITEM;
-	uiinfo[i].label = name_str;
+	uiinfo[i].label = (gchar *) e_destination_get_name (info->dest); 
 	++i;
 
 	uiinfo[i].type = GNOME_APP_UI_SEPARATOR;
@@ -174,11 +241,14 @@ popup_menu_card (PopupInfo *info)
 
 		iterator = e_list_get_iterator (card->email);
 		for (e_iterator_reset (iterator); e_iterator_is_valid (iterator); e_iterator_next (iterator)) {
+			gchar *label = (gchar *)e_iterator_get (iterator);
 			
-			radioinfo[j].type = GNOME_APP_UI_ITEM;
-			radioinfo[j].label = (gchar *)e_iterator_get (iterator);
-			radioinfo[j].moreinfo = change_email_num_cb;
-			++j;
+			if (label && *label) {
+				radioinfo[j].type = GNOME_APP_UI_ITEM;
+				radioinfo[j].label = label;
+				radioinfo[j].moreinfo = change_email_num_cb;
+				++j;
+			}
 		}
 
 		radioinfo[j].type = GNOME_APP_UI_ENDOFINFO;
@@ -196,9 +266,13 @@ popup_menu_card (PopupInfo *info)
 	uiinfo[i].type = GNOME_APP_UI_SEPARATOR;
 	++i;
 
+	add_html_mail (&(uiinfo[i]), info);
+	html_toggle = i;
+	++i;
+
 	uiinfo[i].type = GNOME_APP_UI_ITEM;
 	uiinfo[i].label = N_("Edit Contact Info");
-	uiinfo[i].moreinfo = do_nothing;
+	uiinfo[i].moreinfo = edit_contact_info_cb;
 	++i;
 
 	add_remove_recipient (&(uiinfo[i]), info);
@@ -214,7 +288,6 @@ popup_menu_card (PopupInfo *info)
 	 */
 
 	pop = gnome_popup_menu_new (uiinfo);
-	g_free (name_str);
 
 	if (using_radio) {
 		gint n = e_destination_get_email_num (info->dest);
@@ -226,6 +299,8 @@ popup_menu_card (PopupInfo *info)
 		}
 	}
 
+	init_html_mail (&(uiinfo[html_toggle]), info);
+
 	return pop;
 }
 
@@ -233,7 +308,7 @@ static void
 quick_add_cb (GtkWidget *w, gpointer user_data)
 {
 	PopupInfo *info = (PopupInfo *) user_data;
-	e_contact_quick_add_free_form (e_destination_get_string (info->dest), NULL, NULL);
+	e_contact_quick_add_free_form (e_destination_get_address (info->dest), NULL, NULL);
 }
 
 static GtkWidget *
@@ -243,10 +318,11 @@ popup_menu_nocard (PopupInfo *info)
 	gint i=0;
 	GtkWidget *pop;
 	const gchar *str;
+	gint html_toggle;
 
 	memset (uiinfo, 0, sizeof (uiinfo));
 
-	str = e_destination_get_string (info->dest);
+	str = e_destination_get_name (info->dest);
 
 	uiinfo[i].type = GNOME_APP_UI_ITEM;
 	uiinfo[i].label = (gchar *) str;
@@ -255,6 +331,10 @@ popup_menu_nocard (PopupInfo *info)
 	uiinfo[i].type = GNOME_APP_UI_SEPARATOR;
 	++i;
 
+	add_html_mail (&(uiinfo[i]), info);
+	html_toggle = i;
+	++i;
+	
 	uiinfo[i].type = GNOME_APP_UI_ITEM;
 	uiinfo[i].label = _("Add to Contacts");
 	uiinfo[i].moreinfo = quick_add_cb;
@@ -269,11 +349,12 @@ popup_menu_nocard (PopupInfo *info)
 	uiinfo[i].type = GNOME_APP_UI_ENDOFINFO;
 
 	pop = gnome_popup_menu_new (uiinfo);
+
+	init_html_mail (&(uiinfo[html_toggle]), info);
 	
 	return pop;
 }
 
-
 void
 e_select_names_popup (ESelectNamesModel *model, GdkEventButton *ev, gint pos)
 {
