@@ -2,10 +2,10 @@
 /* Evolution calendar - Calendar Conduit
  *
  * Copyright (C) 1998 Free Software Foundation
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
  * Authors: Eskil Heyn Olsen <deity@eskil.dk> 
- *          JP Rosevear <jpr@helixcode.com>
+ *          JP Rosevear <jpr@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -536,7 +536,6 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 	int pos, i;
 	CalComponentText summary = {NULL, NULL};
 	CalComponentDateTime dt = {NULL, NULL};
-	GList *alist, *l;
 	char *txt;
 	
 	g_return_val_if_fail (remote != NULL, NULL);
@@ -700,7 +699,7 @@ check_for_slow_setting (GnomePilotConduit *c, ECalConduitContext *ctxt)
 		GnomePilotConduitStandard *conduit;
 		LOG ("    doing slow sync\n");
 		conduit = GNOME_PILOT_CONDUIT_STANDARD (c);
-		gnome_pilot_conduit_standard_set_slow (conduit);
+		gnome_pilot_conduit_standard_set_slow (conduit, TRUE);
 	} else {
 		LOG ("    doing fast sync\n");
 	}
@@ -1156,9 +1155,6 @@ conduit_get_gpilot_conduit (guint32 pilot_id)
 	retval = gnome_pilot_conduit_sync_abs_new ("DatebookDB", 0x64617465);
 	g_assert (retval != NULL);
 
-	gnome_pilot_conduit_construct (GNOME_PILOT_CONDUIT (retval),
-				       "e_calendar_conduit");
-
 	ctxt = e_calendar_context_new (pilot_id);
 	gtk_object_set_data (GTK_OBJECT (retval), "calconduit_context", ctxt);
 
