@@ -2,7 +2,7 @@
 /* Evolution calendar - Commands for the calendar GUI control
  *
  * Copyright (C) 1998 The Free Software Foundation
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  * Copyright (C) 2000 Ximian, Inc.
  *
  * Authors: Miguel de Icaza <miguel@ximian.com>
@@ -47,12 +47,15 @@
 #include <libgnome/gnome-i18n.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <cal-util/timeutil.h>
+#include "shell/Evolution.h"
 #include "calendar-commands.h"
+#include "calendar-config.h"
 #include "gnome-cal.h"
 #include "goto.h"
 #include "print.h"
 #include "dialogs/cal-prefs-dialog.h"
-#include "e-util/e-gui-utils.h"
+#include "itip-utils.h"
+#include "evolution-shell-component-utils.h"
 
 /* A list of all of the calendars started */
 static GList *all_calendars = NULL;
@@ -75,7 +78,7 @@ new_event_cb (BonoboUIComponent *uic, gpointer data, const char *path)
 {
 	GnomeCalendar *gcal;
 	time_t dtstart, dtend;
-	
+
 	gcal = GNOME_CALENDAR (data);
 	gnome_calendar_get_current_time_range (gcal, &dtstart, &dtend);
 	gnome_calendar_new_appointment_for (gcal, dtstart, dtend, TRUE);
@@ -86,21 +89,29 @@ static void
 print (GnomeCalendar *gcal, gboolean preview)
 {
 	time_t start;
-	const char *view;
+	GnomeCalendarViewType view_type;
 	PrintView print_view;
 
 	gnome_calendar_get_current_time_range (gcal, &start, NULL);
-	view = gnome_calendar_get_current_view_name (gcal);
+	view_type = gnome_calendar_get_view (gcal);
 
-	if (strcmp (view, "dayview") == 0)
+	switch (view_type) {
+	case GNOME_CAL_DAY_VIEW:
 		print_view = PRINT_VIEW_DAY;
-	else if (strcmp (view, "workweekview") == 0 || strcmp (view, "weekview") == 0)
+		break;
+
+	case GNOME_CAL_WORK_WEEK_VIEW:
+	case GNOME_CAL_WEEK_VIEW:
 		print_view = PRINT_VIEW_WEEK;
-	else if (strcmp (view, "monthview") == 0)
+		break;
+
+	case GNOME_CAL_MONTH_VIEW:
 		print_view = PRINT_VIEW_MONTH;
-	else {
+		break;
+
+	default:
 		g_assert_not_reached ();
-		print_view = PRINT_VIEW_DAY;
+		return;
 	}
 
 	print_calendar (gcal, preview, start, print_view);
@@ -216,7 +227,7 @@ show_day_view_clicked (BonoboUIComponent *uic, gpointer data, const char *path)
 
 	gcal = GNOME_CALENDAR (data);
 
-	gnome_calendar_set_view (gcal, "dayview", FALSE, TRUE);
+	gnome_calendar_set_view (gcal, GNOME_CAL_DAY_VIEW, FALSE, TRUE);
 }
 
 static void
@@ -226,7 +237,7 @@ show_work_week_view_clicked (BonoboUIComponent *uic, gpointer data, const char *
 
 	gcal = GNOME_CALENDAR (data);
 
-	gnome_calendar_set_view (gcal, "workweekview", FALSE, TRUE);
+	gnome_calendar_set_view (gcal, GNOME_CAL_WORK_WEEK_VIEW, FALSE, TRUE);
 }
 
 static void
@@ -236,7 +247,7 @@ show_week_view_clicked (BonoboUIComponent *uic, gpointer data, const char *path)
 
 	gcal = GNOME_CALENDAR (data);
 
-	gnome_calendar_set_view (gcal, "weekview", FALSE, TRUE);
+	gnome_calendar_set_view (gcal, GNOME_CAL_WEEK_VIEW, FALSE, TRUE);
 }
 
 static void
@@ -246,10 +257,11 @@ show_month_view_clicked (BonoboUIComponent *uic, gpointer data, const char *path
 
 	gcal = GNOME_CALENDAR (data);
 
-	gnome_calendar_set_view (gcal, "monthview", FALSE, TRUE);
+	gnome_calendar_set_view (gcal, GNOME_CAL_MONTH_VIEW, FALSE, TRUE);
 }
 
 
+
 static void
 new_calendar_cmd (BonoboUIComponent *uic, gpointer data, const char *path)
 {
@@ -348,7 +360,7 @@ save_as_calendar_cmd (BonoboUIComponent *uic, gpointer data, const char *path)
 }
 
 static void
-properties_cmd (BonoboUIComponent *uic, gpointer data, const char *path)
+preferences_cmd (BonoboUIComponent *uic, gpointer data, const char *path)
 {
 	if (!preferences_dialog)
 		preferences_dialog = cal_prefs_dialog_new ();
@@ -356,6 +368,107 @@ properties_cmd (BonoboUIComponent *uic, gpointer data, const char *path)
 		cal_prefs_dialog_show (preferences_dialog);
 }
 
+static void
+cut_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+{
+	GnomeCalendar *gcal;
+
+	gcal = GNOME_CALENDAR (data);
+	set_clock_cursor (gcal);
+	gnome_calendar_cut_clipboard (gcal);
+	set_normal_cursor (gcal);
+}
+
+static void
+copy_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+{
+	GnomeCalendar *gcal;
+
+	gcal = GNOME_CALENDAR (data);
+
+	set_clock_cursor (gcal);
+	gnome_calendar_copy_clipboard (gcal);
+	set_normal_cursor (gcal);
+}
+
+static void
+paste_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+{
+	GnomeCalendar *gcal;
+
+	gcal = GNOME_CALENDAR (data);
+
+	set_clock_cursor (gcal);
+	gnome_calendar_paste_clipboard (gcal);
+	set_normal_cursor (gcal);
+}
+
+static void
+publish_freebusy_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+{
+	GnomeCalendar *gcal;
+	CalClient *client;
+	CalClientGetStatus status;
+	CalComponent *comp;
+	time_t start, end;
+	
+	gcal = GNOME_CALENDAR (data);
+	gnome_calendar_get_current_time_range (gcal, &start, &end);
+
+	client = gnome_calendar_get_cal_client (gcal);
+	status = cal_client_get_free_busy (client, start, end, &comp);
+	if (status == CAL_CLIENT_GET_SUCCESS)
+		itip_send_comp (CAL_COMPONENT_METHOD_PUBLISH, comp);
+}
+
+/* Does a queryInterface on the control's parent control frame for the ShellView interface */
+static GNOME_Evolution_ShellView
+get_shell_view_interface (BonoboControl *control)
+{
+	Bonobo_ControlFrame control_frame;
+	GNOME_Evolution_ShellView shell_view;
+	CORBA_Environment ev;
+
+	control_frame = bonobo_control_get_control_frame (control);
+
+	g_assert (control_frame != CORBA_OBJECT_NIL);
+
+	CORBA_exception_init (&ev);
+	shell_view = Bonobo_Unknown_queryInterface (control_frame,
+						    "IDL:GNOME/Evolution/ShellView:1.0",
+						    &ev);
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_message ("get_shell_view_interface(): "
+			   "Could not queryInterface() on the control frame");
+		shell_view = CORBA_OBJECT_NIL;
+		goto out;
+	}
+
+	CORBA_exception_free (&ev);
+
+ out:
+
+	return shell_view;
+}
+
+/* Clears the folder bar label on the shell view */
+static void
+clear_folder_bar_label (BonoboControl *control)
+{
+	GNOME_Evolution_ShellView shell_view;
+	CORBA_Environment ev;
+
+	shell_view = get_shell_view_interface (control);
+	if (shell_view == CORBA_OBJECT_NIL)
+		return;
+
+	CORBA_exception_init (&ev);
+	GNOME_Evolution_ShellView_setFolderBarLabel (shell_view, "", &ev);
+	if (ev._major != CORBA_NO_EXCEPTION)
+		g_message ("clear_folder_bar_label(): Could not set the folder bar label");
+
+	CORBA_exception_free (&ev);
+}
 
 static BonoboUIVerb verbs [] = {
 	BONOBO_UI_VERB ("CalendarNew", new_calendar_cmd),
@@ -365,13 +478,19 @@ static BonoboUIVerb verbs [] = {
 	BONOBO_UI_VERB ("CalendarPrintPreview", file_print_preview_cb),
 	BONOBO_UI_VERB ("EditNewAppointment", new_appointment_cb),
 	BONOBO_UI_VERB ("EditNewEvent", new_event_cb),
-	BONOBO_UI_VERB ("CalendarPreferences", properties_cmd),
-		  
+	BONOBO_UI_VERB ("CalendarPreferences", preferences_cmd),
+
+	BONOBO_UI_VERB ("CutEvent", cut_event_cmd),
+	BONOBO_UI_VERB ("CopyEvent", copy_event_cmd),
+	BONOBO_UI_VERB ("PasteEvent", paste_event_cmd),
+
+	BONOBO_UI_VERB ("PublishFreeBusy", publish_freebusy_cmd),
+
 	BONOBO_UI_VERB ("CalendarPrev", previous_clicked),
 	BONOBO_UI_VERB ("CalendarToday", today_clicked),
 	BONOBO_UI_VERB ("CalendarNext", next_clicked),
 	BONOBO_UI_VERB ("CalendarGoto", goto_clicked),
-		  
+
 	BONOBO_UI_VERB ("ShowDayView", show_day_view_clicked),
 	BONOBO_UI_VERB ("ShowWorkWeekView", show_work_week_view_clicked),
 	BONOBO_UI_VERB ("ShowWeekView", show_week_view_clicked),
@@ -399,7 +518,7 @@ static EPixmap pixmaps [] =
 
 void
 calendar_control_activate (BonoboControl *control,
-			   GnomeCalendar *cal)
+			   GnomeCalendar *gcal)
 {
 	Bonobo_UIContainer remote_uih;
 	BonoboUIComponent *uic;
@@ -426,9 +545,8 @@ calendar_control_activate (BonoboControl *control,
 	/* This makes the appropriate radio button in the toolbar active. */
 	gnome_calendar_update_view_buttons (cal);
 #endif
-	
-	bonobo_ui_component_add_verb_list_with_data (
-		uic, verbs, cal);
+
+	bonobo_ui_component_add_verb_list_with_data (uic, verbs, gcal);
 
 	bonobo_ui_component_freeze (uic, NULL);
 
@@ -438,15 +556,25 @@ calendar_control_activate (BonoboControl *control,
 
 	e_pixmaps_update (uic, pixmaps);
 
+	gnome_calendar_setup_view_menus (gcal, uic);
+
 	bonobo_ui_component_thaw (uic, NULL);
+
+	/* Show the dialog for setting the timezone if the user hasn't chosen
+	   a default timezone already. */
+	calendar_config_check_timezone_set ();
+
+	clear_folder_bar_label (control);
 }
 
 void
-calendar_control_deactivate (BonoboControl *control)
+calendar_control_deactivate (BonoboControl *control, GnomeCalendar *gcal)
 {
 	BonoboUIComponent *uic = bonobo_control_get_ui_component (control);
 	g_assert (uic != NULL);
 
+	gnome_calendar_discard_view_menus (gcal);
+
 	bonobo_ui_component_rm (uic, "/", NULL);
  	bonobo_ui_component_unset_container (uic);
 }
