@@ -4,7 +4,7 @@
  * Author :
  *  Damon Chaplin <damon@ximian.com>
  *
- * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
@@ -61,6 +61,10 @@ void	  calendar_config_write_on_exit		(void);
  * Calendar Settings.
  */
 
+/* The current timezone, e.g. "Europe/London". */
+gchar*	  calendar_config_get_timezone		(void);
+void	  calendar_config_set_timezone		(gchar	     *timezone);
+
 /* The working days of the week, a bit-wise combination of flags. */
 CalWeekdays calendar_config_get_working_days	(void);
 void	  calendar_config_set_working_days	(CalWeekdays  days);
@@ -127,4 +131,15 @@ void	  calendar_config_configure_e_date_edit		(EDateEdit	*dedit);
 void	  calendar_config_configure_e_calendar_table	(ECalendarTable	*cal_table);
 void	  calendar_config_configure_e_cell_date_edit	(ECellDateEdit	*ecde);
 
+/* Shows the timezone dialog if the user hasn't set a default timezone. */
+void	  calendar_config_check_timezone_set	(void);
+
+/* Colors for the task list */
+
+const char *calendar_config_get_tasks_due_today_color (void);
+void calendar_config_set_tasks_due_today_color (const char *color);
+
+const char *calendar_config_get_tasks_overdue_color (void);
+void calendar_config_set_tasks_overdue_color (const char *color);
+
 #endif /* _CALENDAR_CONFIG_H_ */
