@@ -1,6 +1,6 @@
 /* Evolution calendar - Utilities for manipulating CalComponent objects
  *
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  * Copyright (C) 2000 Ximian, Inc.
  *
  * Author: Federico Mena-Quintero <federico@ximian.com>
@@ -37,7 +37,7 @@
  * component object.
  **/
 void
-cal_comp_util_add_exdate (CalComponent *comp, struct icaltimetype itt)
+cal_comp_util_add_exdate (CalComponent *comp, time_t t, icaltimezone *zone)
 {
 	GSList *list;
 	CalComponentDateTime *cdt;
@@ -49,8 +49,8 @@ cal_comp_util_add_exdate (CalComponent *comp, struct icaltimetype itt)
 
 	cdt = g_new (CalComponentDateTime, 1);
 	cdt->value = g_new (struct icaltimetype, 1);
-	*cdt->value = itt;
-	cdt->tzid = NULL;
+	*cdt->value = icaltime_from_timet_with_zone (t, TRUE, zone);
+	cdt->tzid = icaltimezone_get_tzid (zone);
 
 	list = g_slist_append (list, cdt);
 	cal_component_set_exdate_list (comp, list);
