@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* control-factory.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000  Ximian, Inc.
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
@@ -27,6 +27,7 @@
 #include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-generic-factory.h>
 #include <bonobo/bonobo-persist-file.h>
+#include <bonobo/bonobo-context.h>
 #include <glade/glade.h>
 
 #include <liboaf/liboaf.h>
@@ -49,16 +50,17 @@ CORBA_ORB orb;
 
 
 static void
-control_activate_cb (BonoboControl *control,
-		     gboolean activate,
-		     gpointer user_data)
+control_activate_cb (BonoboControl *control, gboolean activate, gpointer data)
 {
-	if (activate)
-		calendar_control_activate (control, user_data);
-	else
-		calendar_control_deactivate (control);
-}
+	GnomeCalendar *gcal;
 
+	gcal = GNOME_CALENDAR (data);
+
+	if (activate)
+		calendar_control_activate (control, gcal);
+	else
+		calendar_control_deactivate (control, gcal);
+}
 
 static void
 get_prop (BonoboPropertyBag *bag,
@@ -127,6 +129,7 @@ calendar_properties_init (GnomeCalendar *gcal, BonoboControl *control)
 				 0);
 
 	bonobo_control_set_properties (control, pbag);
+	bonobo_object_unref (BONOBO_OBJECT (pbag));
 }
 
 /* Callback factory function for calendar controls */
@@ -153,7 +156,8 @@ control_factory_init (void)
 		return;
 
 	factory = bonobo_generic_factory_new (CONTROL_FACTORY_ID, control_factory_fn, NULL);
-
+	bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
+       
 	if (factory == NULL)
 		g_error ("I could not register a Calendar control factory.");
 }
