@@ -1,7 +1,7 @@
 /* Evolution calendar - Main calendar view widget
  *
  * Copyright (C) 1998 The Free Software Foundation
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  * Copyright (C) 2001 Ximian, Inc.
  *
  * Authors: Miguel de Icaza <miguel@ximian.com>
@@ -29,6 +29,7 @@
 #include <time.h>
 #include <libgnome/gnome-defs.h>
 #include <gtk/gtkvbox.h>
+#include <bonobo/bonobo-ui-component.h>
 #include <widgets/misc/e-calendar.h>
 #include <cal-client/cal-client.h>
 
@@ -47,6 +48,14 @@ typedef struct _GnomeCalendar GnomeCalendar;
 typedef struct _GnomeCalendarClass GnomeCalendarClass;
 typedef struct _GnomeCalendarPrivate GnomeCalendarPrivate;
 
+/* View types */
+typedef enum {
+	GNOME_CAL_DAY_VIEW,
+	GNOME_CAL_WORK_WEEK_VIEW,
+	GNOME_CAL_WEEK_VIEW,
+	GNOME_CAL_MONTH_VIEW
+} GnomeCalendarViewType;
+
 struct _GnomeCalendar {
 	GtkVBox vbox;
 
@@ -65,12 +74,12 @@ GtkWidget *gnome_calendar_construct		(GnomeCalendar *gcal);
 GtkWidget *gnome_calendar_new			(void);
 
 CalClient *gnome_calendar_get_cal_client	(GnomeCalendar *gcal);
+CalClient *gnome_calendar_get_task_pad_cal_client(GnomeCalendar *gcal);
 
 gboolean   gnome_calendar_open                  (GnomeCalendar *gcal, const char *str_uri);
-/*
-int	   gnome_calendar_create		(GnomeCalendar *gcal,
-						 char *file);
-*/
+
+void gnome_calendar_set_query (GnomeCalendar *gcal, char *sexp);
+
 void       gnome_calendar_next             	(GnomeCalendar *gcal);
 void       gnome_calendar_previous         	(GnomeCalendar *gcal);
 void       gnome_calendar_goto             	(GnomeCalendar *gcal,
@@ -79,11 +88,13 @@ void       gnome_calendar_dayjump          	(GnomeCalendar *gcal,
 						 time_t time);
 /* Jumps to the current day */
 void       gnome_calendar_goto_today            (GnomeCalendar *gcal);
-char      *gnome_calendar_get_current_view_name (GnomeCalendar *gcal);
-void	   gnome_calendar_set_view		(GnomeCalendar	*gcal,
-						 char		*page_name,
-						 gboolean	 reset_range_shown,
-						 gboolean	 focus);
+
+GnomeCalendarViewType gnome_calendar_get_view (GnomeCalendar *gcal);
+void gnome_calendar_set_view (GnomeCalendar *gcal, GnomeCalendarViewType view_type,
+			      gboolean range_selected, gboolean grab_focus);
+
+void gnome_calendar_setup_view_menus (GnomeCalendar *gcal, BonoboUIComponent *uic);
+void gnome_calendar_discard_view_menus (GnomeCalendar *gcal);
 
 void	   gnome_calendar_set_selected_time_range (GnomeCalendar *gcal,
 						   time_t	  start_time,
@@ -123,6 +134,15 @@ void	   gnome_calendar_set_view_buttons	(GnomeCalendar	*gcal,
    just return without doing anything. */
 void	   gnome_calendar_update_view_buttons	(GnomeCalendar	*gcal);
 
+/* Get the current timezone. */
+icaltimezone *gnome_calendar_get_timezone	(GnomeCalendar	*gcal);
+
+
+/* Clipboard operations */
+void       gnome_calendar_cut_clipboard         (GnomeCalendar  *gcal);
+void       gnome_calendar_copy_clipboard        (GnomeCalendar  *gcal);
+void       gnome_calendar_paste_clipboard       (GnomeCalendar  *gcal);
+
 
 
 END_GNOME_DECLS
