@@ -1,9 +1,10 @@
 /* Evolution calendar client interface object
  *
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  * Copyright (C) 2000 Ximian, Inc.
  *
- * Author: Federico Mena-Quintero <federico@ximian.com>
+ * Authors: Federico Mena-Quintero <federico@ximian.com>
+ *          Rodrigo Moya <rodrigo@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +22,14 @@
  */
 
 #include <config.h>
+#include <ical.h>
 #include "cal.h"
-#include "cal-backend.h"
+#include "query.h"
+#include "wombat.h"
 
-
+#define PARENT_TYPE         BONOBO_X_OBJECT_TYPE
+
+static BonoboXObjectClass *parent_class;
 
 /* Private part of the Cal structure */
 struct _CalPrivate {
@@ -33,123 +38,15 @@ struct _CalPrivate {
 
 	/* Listener on the client we notify */
 	GNOME_Evolution_Calendar_Listener listener;
+
+	/* and a reference to the WombatClient interface */
+	GNOME_Evolution_WombatClient wombat_client;
 };
 
-
-
-static void cal_class_init (CalClass *class);
-static void cal_init (Cal *cal);
-static void cal_destroy (GtkObject *object);
-
-static POA_GNOME_Evolution_Calendar_Cal__vepv cal_vepv;
-
-static BonoboObjectClass *parent_class;
-
-
-
-/**
- * cal_get_type:
- * @void:
- *
- * Registers the #Cal class if necessary, and returns the type ID associated to
- * it.
- *
- * Return value: The type ID of the #Cal class.
- **/
-GtkType
-cal_get_type (void)
-{
-	static GtkType cal_type = 0;
-
-	if (!cal_type) {
-		static const GtkTypeInfo cal_info = {
-			"Cal",
-			sizeof (Cal),
-			sizeof (CalClass),
-			(GtkClassInitFunc) cal_class_init,
-			(GtkObjectInitFunc) cal_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		cal_type = gtk_type_unique (BONOBO_OBJECT_TYPE, &cal_info);
-	}
-
-	return cal_type;
-}
-
-/* CORBA class initialzation function for the calendar */
-static void
-init_cal_corba_class (void)
-{
-	cal_vepv.Bonobo_Unknown_epv = bonobo_object_get_epv ();
-	cal_vepv.GNOME_Evolution_Calendar_Cal_epv = cal_get_epv ();
-}
-
-/* Class initialization function for the calendar */
-static void
-cal_class_init (CalClass *class)
-{
-	GtkObjectClass *object_class;
-
-	object_class = (GtkObjectClass *) class;
-
-	parent_class = gtk_type_class (BONOBO_OBJECT_TYPE);
-
-	object_class->destroy = cal_destroy;
-
-	init_cal_corba_class ();
-}
-
-/* Object initialization function for the calendar */
-static void
-cal_init (Cal *cal)
-{
-	CalPrivate *priv;
-
-	priv = g_new0 (CalPrivate, 1);
-	cal->priv = priv;
-
-	priv->listener = CORBA_OBJECT_NIL;
-}
-
-/* Destroy handler for the calendar */
-static void
-cal_destroy (GtkObject *object)
-{
-	Cal *cal;
-	CalPrivate *priv;
-	CORBA_Environment ev;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_CAL (object));
-
-	cal = CAL (object);
-	priv = cal->priv;
-
-	priv->backend = NULL;
-
-	CORBA_exception_init (&ev);
-	CORBA_Object_release (priv->listener, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION)
-		g_message ("cal_destroy(): could not release the listener");
-
-	CORBA_exception_free (&ev);
-
-	g_free (priv);
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
-
-
-/* CORBA servant implementation */
 
 /* Cal::get_uri method */
 static CORBA_char *
-Cal_get_uri (PortableServer_Servant servant,
+impl_Cal_get_uri (PortableServer_Servant servant,
 	     CORBA_Environment *ev)
 {
 	Cal *cal;
@@ -182,9 +79,9 @@ uncorba_obj_type (GNOME_Evolution_Calendar_CalObjType type)
 
 /* Cal::get_n_objects method */
 static CORBA_long
-Cal_get_n_objects (PortableServer_Servant servant,
-		   GNOME_Evolution_Calendar_CalObjType type,
-		   CORBA_Environment *ev)
+impl_Cal_get_n_objects (PortableServer_Servant servant,
+			GNOME_Evolution_Calendar_CalObjType type,
+			CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -201,9 +98,9 @@ Cal_get_n_objects (PortableServer_Servant servant,
 
 /* Cal::get_object method */
 static GNOME_Evolution_Calendar_CalObj
-Cal_get_object (PortableServer_Servant servant,
-		const GNOME_Evolution_Calendar_CalObjUID uid,
-		CORBA_Environment *ev)
+impl_Cal_get_object (PortableServer_Servant servant,
+		     const GNOME_Evolution_Calendar_CalObjUID uid,
+		     CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -256,9 +153,9 @@ build_uid_seq (GList *uids)
 
 /* Cal::get_uids method */
 static GNOME_Evolution_Calendar_CalObjUIDSeq *
-Cal_get_uids (PortableServer_Servant servant,
-	      GNOME_Evolution_Calendar_CalObjType type,
-	      CORBA_Environment *ev)
+impl_Cal_get_uids (PortableServer_Servant servant,
+		   GNOME_Evolution_Calendar_CalObjType type,
+		   CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -281,10 +178,10 @@ Cal_get_uids (PortableServer_Servant servant,
 
 /* Cal::get_changes method */
 static GNOME_Evolution_Calendar_CalObjChangeSeq *
-Cal_get_changes (PortableServer_Servant servant,
-		 GNOME_Evolution_Calendar_CalObjType type,
-		 const CORBA_char *change_id,
-		 CORBA_Environment *ev)
+impl_Cal_get_changes (PortableServer_Servant servant,
+		      GNOME_Evolution_Calendar_CalObjType type,
+		      const CORBA_char *change_id,
+		      CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -300,11 +197,11 @@ Cal_get_changes (PortableServer_Servant servant,
 
 /* Cal::get_objects_in_range method */
 static GNOME_Evolution_Calendar_CalObjUIDSeq *
-Cal_get_objects_in_range (PortableServer_Servant servant,
-			  GNOME_Evolution_Calendar_CalObjType type,
-			  GNOME_Evolution_Calendar_Time_t start,
-			  GNOME_Evolution_Calendar_Time_t end,
-			  CORBA_Environment *ev)
+impl_Cal_get_objects_in_range (PortableServer_Servant servant,
+			       GNOME_Evolution_Calendar_CalObjType type,
+			       GNOME_Evolution_Calendar_Time_t start,
+			       GNOME_Evolution_Calendar_Time_t end,
+			       CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -335,12 +232,53 @@ Cal_get_objects_in_range (PortableServer_Servant servant,
 	return seq;
 }
 
+/* Cal::get_free_busy method */
+static GNOME_Evolution_Calendar_CalObj
+impl_Cal_get_free_busy (PortableServer_Servant servant,
+			GNOME_Evolution_Calendar_Time_t start,
+			GNOME_Evolution_Calendar_Time_t end,
+			CORBA_Environment *ev)
+{
+	Cal *cal;
+	CalPrivate *priv;
+	time_t t_start, t_end;
+	char *calobj;
+
+	cal = CAL (bonobo_object_from_servant (servant));
+	priv = cal->priv;
+
+	t_start = (time_t) start;
+	t_end = (time_t) end;
+
+	if (t_start > t_end || t_start == -1 || t_end == -1) {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_GNOME_Evolution_Calendar_Cal_InvalidRange,
+				     NULL);
+		return NULL;
+	}
+
+	calobj = cal_backend_get_free_busy (priv->backend, t_start, t_end);
+        if (calobj) {
+		CORBA_char *calobj_copy;
+
+		calobj_copy = CORBA_string_dup (calobj);
+		g_free (calobj);
+		return calobj_copy;
+	}
+
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			     ex_GNOME_Evolution_Calendar_Cal_NotFound,
+			     NULL);
+
+	return NULL;
+}
+
 /* Cal::get_alarms_in_range method */
 static GNOME_Evolution_Calendar_CalComponentAlarmsSeq *
-Cal_get_alarms_in_range (PortableServer_Servant servant,
-			 GNOME_Evolution_Calendar_Time_t start,
-			 GNOME_Evolution_Calendar_Time_t end,
-			 CORBA_Environment *ev)
+impl_Cal_get_alarms_in_range (PortableServer_Servant servant,
+			      GNOME_Evolution_Calendar_Time_t start,
+			      GNOME_Evolution_Calendar_Time_t end,
+			      CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -367,11 +305,11 @@ Cal_get_alarms_in_range (PortableServer_Servant servant,
 
 /* Cal::get_alarms_for_object method */
 static GNOME_Evolution_Calendar_CalComponentAlarms *
-Cal_get_alarms_for_object (PortableServer_Servant servant,
-			   const GNOME_Evolution_Calendar_CalObjUID uid,
-			   GNOME_Evolution_Calendar_Time_t start,
-			   GNOME_Evolution_Calendar_Time_t end,
-			   CORBA_Environment * ev)
+impl_Cal_get_alarms_for_object (PortableServer_Servant servant,
+				const GNOME_Evolution_Calendar_CalObjUID uid,
+				GNOME_Evolution_Calendar_Time_t start,
+				GNOME_Evolution_Calendar_Time_t end,
+				CORBA_Environment * ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -409,12 +347,11 @@ Cal_get_alarms_for_object (PortableServer_Servant servant,
 	}
 }
 
-/* Cal::update_object method */
+/* Cal::update_objects method */
 static void
-Cal_update_object (PortableServer_Servant servant,
-		   const GNOME_Evolution_Calendar_CalObjUID uid,
-		   const GNOME_Evolution_Calendar_CalObj calobj,
-		   CORBA_Environment *ev)
+impl_Cal_update_objects (PortableServer_Servant servant,
+			 const GNOME_Evolution_Calendar_CalObj calobj,
+			 CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -422,7 +359,7 @@ Cal_update_object (PortableServer_Servant servant,
 	cal = CAL (bonobo_object_from_servant (servant));
 	priv = cal->priv;
 
-	if (!cal_backend_update_object (priv->backend, uid, calobj))
+	if (!cal_backend_update_objects (priv->backend, calobj))
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_GNOME_Evolution_Calendar_Cal_InvalidObject,
 				     NULL);
@@ -430,9 +367,9 @@ Cal_update_object (PortableServer_Servant servant,
 
 /* Cal::remove_object method */
 static void
-Cal_remove_object (PortableServer_Servant servant,
-		   const GNOME_Evolution_Calendar_CalObjUID uid,
-		   CORBA_Environment *ev)
+impl_Cal_remove_object (PortableServer_Servant servant,
+			const GNOME_Evolution_Calendar_CalObjUID uid,
+			CORBA_Environment *ev)
 {
 	Cal *cal;
 	CalPrivate *priv;
@@ -446,35 +383,74 @@ Cal_remove_object (PortableServer_Servant servant,
 				     NULL);
 }
 
-/**
- * cal_get_epv:
- * @void:
- *
- * Creates an EPV for the Cal CORBA class.
- *
- * Return value: A newly-allocated EPV.
- **/
-POA_GNOME_Evolution_Calendar_Cal__epv *
-cal_get_epv (void)
+/* Cal::getQuery implementation */
+static GNOME_Evolution_Calendar_Query
+impl_Cal_get_query (PortableServer_Servant servant,
+		    const CORBA_char *sexp,
+		    GNOME_Evolution_Calendar_QueryListener ql,
+		    CORBA_Environment *ev)
 {
-	POA_GNOME_Evolution_Calendar_Cal__epv *epv;
+	Cal *cal;
+	CalPrivate *priv;
+	Query *query;
+	CORBA_Environment ev2;
+	GNOME_Evolution_Calendar_Query query_copy;
 
-	epv = g_new0 (POA_GNOME_Evolution_Calendar_Cal__epv, 1);
-	epv->_get_uri     = Cal_get_uri;
-	epv->countObjects = Cal_get_n_objects;
-	epv->getObject    = Cal_get_object;
-	epv->getUIds      = Cal_get_uids;
-	epv->getChanges   = Cal_get_changes;
-	epv->getObjectsInRange  = Cal_get_objects_in_range;
-	epv->getAlarmsInRange   = Cal_get_alarms_in_range;
-	epv->getAlarmsForObject = Cal_get_alarms_for_object;
-	epv->updateObject = Cal_update_object;
-	epv->removeObject = Cal_remove_object;
+	cal = CAL (bonobo_object_from_servant (servant));
+	priv = cal->priv;
 
-	return epv;
+	query = query_new (priv->backend, ql, sexp);
+	if (!query) {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_GNOME_Evolution_Calendar_Cal_CouldNotCreate,
+				     NULL);
+		return CORBA_OBJECT_NIL;
+	}
+
+	CORBA_exception_init (&ev2);
+	query_copy = CORBA_Object_duplicate (BONOBO_OBJREF (query), &ev2);
+	if (ev2._major != CORBA_NO_EXCEPTION) {
+		CORBA_exception_free (&ev2);
+		g_message ("Cal_get_query(): Could not duplicate the query reference");
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_GNOME_Evolution_Calendar_Cal_CouldNotCreate,
+				     NULL);
+		return CORBA_OBJECT_NIL;
+	}
+
+	CORBA_exception_free (&ev2);
+
+	return query_copy;
 }
 
-
+/* Cal::get_timezone_object method */
+static GNOME_Evolution_Calendar_CalObj
+impl_Cal_get_timezone_object (PortableServer_Servant servant,
+			      const GNOME_Evolution_Calendar_CalTimezoneObjUID tzid,
+			      CORBA_Environment *ev)
+{
+	Cal *cal;
+	CalPrivate *priv;
+	char *calobj;
+
+	cal = CAL (bonobo_object_from_servant (servant));
+	priv = cal->priv;
+
+	calobj = cal_backend_get_timezone_object (priv->backend, tzid);
+
+	if (calobj) {
+		CORBA_char *calobj_copy;
+
+		calobj_copy = CORBA_string_dup (calobj);
+		g_free (calobj);
+		return calobj_copy;
+	} else {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_GNOME_Evolution_Calendar_Cal_NotFound,
+				     NULL);
+		return NULL;
+	}
+}
 
 /**
  * cal_construct:
@@ -491,7 +467,6 @@ cal_get_epv (void)
  **/
 Cal *
 cal_construct (Cal *cal,
-	       GNOME_Evolution_Calendar_Cal corba_cal,
 	       CalBackend *backend,
 	       GNOME_Evolution_Calendar_Listener listener)
 {
@@ -514,49 +489,23 @@ cal_construct (Cal *cal,
 		return NULL;
 	}
 
+	/* obtain the WombatClient interface */
+	priv->wombat_client = Bonobo_Unknown_queryInterface (
+		priv->listener,
+		"IDL:GNOME/Evolution/WombatClient:1.0",
+		&ev);
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_message ("cal_construct: could not get the WombatClient interface");
+		priv->wombat_client = CORBA_OBJECT_NIL;
+	}
+
 	CORBA_exception_free (&ev);
 
 	priv->backend = backend;
 
-	bonobo_object_construct (BONOBO_OBJECT (cal), corba_cal);
 	return cal;
 }
 
-/**
- * cal_corba_object_create:
- * @object: #BonoboObject that will wrap the CORBA object.
- *
- * Creates and activates the CORBA object that is wrapped by the specified
- * calendar client interface @object.
- *
- * Return value: An activated object reference or #CORBA_OBJECT_NIL in case of
- * failure.
- **/
-GNOME_Evolution_Calendar_Cal
-cal_corba_object_create (BonoboObject *object)
-{
-	POA_GNOME_Evolution_Calendar_Cal *servant;
-	CORBA_Environment ev;
-
-	g_return_val_if_fail (object != NULL, CORBA_OBJECT_NIL);
-	g_return_val_if_fail (IS_CAL (object), CORBA_OBJECT_NIL);
-
-	servant = (POA_GNOME_Evolution_Calendar_Cal *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &cal_vepv;
-
-	CORBA_exception_init (&ev);
-	POA_GNOME_Evolution_Calendar_Cal__init ((PortableServer_Servant) servant, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_message ("cal_corba_object_create(): could not init the servant");
-		g_free (servant);
-		CORBA_exception_free (&ev);
-		return CORBA_OBJECT_NIL;
-	}
-
-	CORBA_exception_free (&ev);
-	return (GNOME_Evolution_Calendar_Cal) bonobo_object_activate_servant (object, servant);
-}
-
 /**
  * cal_new:
  * @backend: A calendar backend.
@@ -572,28 +521,13 @@ Cal *
 cal_new (CalBackend *backend, GNOME_Evolution_Calendar_Listener listener)
 {
 	Cal *cal, *retval;
-	GNOME_Evolution_Calendar_Cal corba_cal;
-	CORBA_Environment ev;
-	gboolean ret;
 
 	g_return_val_if_fail (backend != NULL, NULL);
 	g_return_val_if_fail (IS_CAL_BACKEND (backend), NULL);
 
 	cal = CAL (gtk_type_new (CAL_TYPE));
-	corba_cal = cal_corba_object_create (BONOBO_OBJECT (cal));
 
-	CORBA_exception_init (&ev);
-	ret = CORBA_Object_is_nil ((CORBA_Object) corba_cal, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION || ret) {
-		g_message ("cal_new(): could not create the CORBA object");
-		bonobo_object_unref (BONOBO_OBJECT (cal));
-		CORBA_exception_free (&ev);
-		return NULL;
-	}
-
-	CORBA_exception_free (&ev);
-
-	retval = cal_construct (cal, corba_cal, backend, listener);
+	retval = cal_construct (cal, backend, listener);
 	if (!retval) {
 		g_message ("cal_new(): could not construct the calendar client interface");
 		bonobo_object_unref (BONOBO_OBJECT (cal));
@@ -603,6 +537,80 @@ cal_new (CalBackend *backend, GNOME_Evolution_Calendar_Listener listener)
 	return retval;
 }
 
+/* Destroy handler for the calendar */
+static void
+cal_destroy (GtkObject *object)
+{
+	Cal *cal;
+	CalPrivate *priv;
+	CORBA_Environment ev;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (IS_CAL (object));
+
+	cal = CAL (object);
+	priv = cal->priv;
+
+	priv->backend = NULL;
+
+	CORBA_exception_init (&ev);
+	CORBA_Object_release (priv->listener, &ev);
+	if (ev._major != CORBA_NO_EXCEPTION)
+		g_message ("cal_destroy(): could not release the listener");
+
+	CORBA_exception_free (&ev);
+
+	g_free (priv);
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+}
+
+
+
+/* Class initialization function for the calendar */
+static void
+cal_class_init (CalClass *klass)
+{
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	POA_GNOME_Evolution_Calendar_Cal__epv *epv = &klass->epv;
+
+	parent_class = gtk_type_class (BONOBO_OBJECT_TYPE);
+
+	/* Class method overrides */
+	object_class->destroy = cal_destroy;
+
+	/* Epv methods */
+	epv->_get_uri = impl_Cal_get_uri;
+	epv->countObjects = impl_Cal_get_n_objects;
+	epv->getObject = impl_Cal_get_object;
+	epv->getTimezoneObject = impl_Cal_get_timezone_object;
+	epv->getUIDs = impl_Cal_get_uids;
+	epv->getChanges = impl_Cal_get_changes;
+	epv->getObjectsInRange = impl_Cal_get_objects_in_range;
+	epv->getFreeBusy = impl_Cal_get_free_busy;
+	epv->getAlarmsInRange = impl_Cal_get_alarms_in_range;
+	epv->getAlarmsForObject = impl_Cal_get_alarms_for_object;
+	epv->updateObjects = impl_Cal_update_objects;
+	epv->removeObject = impl_Cal_remove_object;
+	epv->getQuery = impl_Cal_get_query;
+}
+
+
+/* Object initialization function for the calendar */
+static void
+cal_init (Cal *cal)
+{
+	CalPrivate *priv;
+
+	priv = g_new0 (CalPrivate, 1);
+	cal->priv = priv;
+
+	priv->listener = CORBA_OBJECT_NIL;
+}
+
+BONOBO_X_TYPE_FUNC_FULL (Cal, GNOME_Evolution_Calendar_Cal, PARENT_TYPE, cal);
+
 /**
  * cal_notify_update:
  * @cal: A calendar client interface.
@@ -664,3 +672,81 @@ cal_notify_remove (Cal *cal, const char *uid)
 
 	CORBA_exception_free (&ev);
 }
+
+/**
+ * cal_get_password:
+ * @cal: A calendar client interface.
+ * @prompt: The message to show to the user when asking for the password.
+ * @key: A key associated with the password being asked.
+ *
+ * Gets a password from the calendar client this Cal knows about. It does
+ * so by using the WombatClient interface being used by the corresponding
+ * CalClient.
+ *
+ * Returns: a password entered by the user.
+ */
+char *
+cal_get_password (Cal *cal, const char *prompt, const char *key)
+{
+	CalPrivate *priv;
+	CORBA_Environment ev;
+	CORBA_char *pwd;
+
+	g_return_val_if_fail (cal != NULL, NULL);
+	g_return_val_if_fail (IS_CAL (cal), NULL);
+
+	priv = cal->priv;
+	g_return_val_if_fail (priv->wombat_client != CORBA_OBJECT_NIL, NULL);
+
+	CORBA_exception_init (&ev);
+	pwd = GNOME_Evolution_WombatClient_getPassword (
+		priv->wombat_client,
+		(const CORBA_char *) prompt,
+		(const CORBA_char *) key,
+		&ev);
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_message ("cal_get_password: could not get password from associated WombatClient");
+		CORBA_exception_free (&ev);
+		return NULL;
+	}
+
+	return pwd;
+}
+
+/**
+ * cal_forget_password:
+ * @cal: A calendar client interface.
+ * @key: A key associated with the password to be forgotten.
+ *
+ * Notifies the associated calendar client that it should forget
+ * about the password identified by @key, so that next time the backend
+ * asks the client about it, the client would ask again the user for it.
+ * This is done in cases where the password supplied the first time
+ * was not a valid password and the backend needs the user to enter
+ * a new one.
+ */
+void
+cal_forget_password (Cal *cal, const char *key)
+{
+	CalPrivate *priv;
+	CORBA_Environment ev;
+
+	g_return_if_fail (cal != NULL);
+	g_return_if_fail (IS_CAL (cal));
+
+	priv = cal->priv;
+	g_return_if_fail (priv->wombat_client != CORBA_OBJECT_NIL);
+
+	CORBA_exception_init (&ev);
+	GNOME_Evolution_WombatClient_forgetPassword (
+		priv->wombat_client,
+		(const CORBA_char *) key,
+		&ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_message ("cal_forget_password: could not notify WombatClient about "
+			   "password to be forgotten");
+	}
+
+	CORBA_exception_free (&ev);
+}
