@@ -1,7 +1,7 @@
 /*
- *  Copyright (C) 2000 Helix Code Inc.
+ *  Copyright (C) 2000 Ximian Inc.
  *
- *  Authors: Michael Zucchi <notzed@helixcode.com>
+ *  Authors: Michael Zucchi <notzed@ximian.com>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -35,4 +35,6 @@ const char *camel_charset_best_name(CamelCharset *);
 /* helper function */
 const char *camel_charset_best(const char *in, int len);
 
+char *camel_charset_locale_name (void);
+
 #endif /* ! _CAMEL_CHARSET_MAP_H */
