@@ -1,8 +1,8 @@
 /*
- *  Copyright (C) 2000 Helix Code Inc.
+ *  Copyright (C) 2000 Ximian Inc.
  *
- *  Authors: Michael Zucchi <notzed@helixcode.com>
- *           Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Michael Zucchi <notzed@ximian.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -22,8 +22,8 @@
 #ifndef _CAMEL_MIME_UTILS_H
 #define _CAMEL_MIME_UTILS_H
 
-#include <glib.h>
 #include <time.h>
+#include <glib.h>
 
 /* maximum size of a line from header_fold() */
 #define CAMEL_FOLD_SIZE (77)
@@ -145,6 +145,7 @@ void header_raw_clear(struct _header_raw **list);
 char *header_raw_check_mailing_list(struct _header_raw **list);
 
 /* fold a header */
+char *header_address_fold(const char *in, int headerlen);
 char *header_fold(const char *in, int headerlen);
 char *header_unfold(const char *in);
 
@@ -175,6 +176,9 @@ void header_references_list_append_asis(struct _header_references **list, char *
 int header_references_list_size(struct _header_references **list);
 struct _header_references *header_references_dup(const struct _header_references *list);
 
+/* decode content-location */
+char *header_location_decode(const char *in);
+
 /* decode the mime-type header */
 void header_mime_decode(const char *in, int *maj, int *min);
 
