@@ -4,9 +4,9 @@
 /* 
  * Authors:
  *  Bertrand Guiheneuf <bertrand@helixcode.com>
- *  Dan Winship <danw@helixcode.com>
+ *  Dan Winship <danw@ximian.com>
  *
- * Copyright 1999, 2000 Helix Code, Inc. (http://www.helixcode.com)
+ * Copyright 1999, 2000 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,16 +24,17 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
 
 #include "camel-session.h"
-
 #include "camel-store.h"
 #include "camel-folder.h"
 #include "camel-vtrash-folder.h"
 #include "camel-exception.h"
-
 #include "camel-private.h"
 
 static CamelServiceClass *parent_class = NULL;
@@ -59,9 +60,7 @@ static void rename_folder (CamelStore *store, const char *old_name,
 
 static void store_sync (CamelStore *store, CamelException *ex);
 static CamelFolderInfo *get_folder_info (CamelStore *store, const char *top,
-					 gboolean fast, gboolean recursive,
-					 gboolean subscribed_only,
-					 CamelException *ex);
+					 guint32 flags, CamelException *ex);
 static void free_folder_info (CamelStore *store, CamelFolderInfo *tree);
 
 static gboolean folder_subscribed (CamelStore *store, const char *folder_name);
@@ -376,7 +375,7 @@ trash_finalize (CamelObject *trash, gpointer event_data, gpointer user_data)
 static void
 init_trash (CamelStore *store)
 {
-	store->vtrash = camel_vtrash_folder_new (store, _("Trash"));
+	store->vtrash = camel_vtrash_folder_new (store, CAMEL_VTRASH_NAME);
 	
 	if (store->vtrash) {
 		/* attach to the finalise event of the vtrash */
@@ -445,9 +444,13 @@ static void
 store_sync (CamelStore *store, CamelException *ex)
 {
 	if (store->folders) {
+		CamelException internal_ex;
+	
+		camel_exception_init (&internal_ex);
 		CAMEL_STORE_LOCK(store, cache_lock);
-		g_hash_table_foreach (store->folders, sync_folder, ex);
+		g_hash_table_foreach (store->folders, sync_folder, &internal_ex);
 		CAMEL_STORE_UNLOCK(store, cache_lock);
+		camel_exception_xfer (ex, &internal_ex);
 	}
 }
 
@@ -470,9 +473,7 @@ camel_store_sync (CamelStore *store, CamelException *ex)
 
 static CamelFolderInfo *
 get_folder_info (CamelStore *store, const char *top,
-		 gboolean fast, gboolean recursive,
-		 gboolean subscribed_only,
-		 CamelException *ex)
+		 guint32 flags, CamelException *ex)
 {
 	g_warning ("CamelStore::get_folder_info not implemented for `%s'",
 		   camel_type_to_name (CAMEL_OBJECT_GET_TYPE (store)));
@@ -483,36 +484,37 @@ get_folder_info (CamelStore *store, const char *top,
  * camel_store_get_folder_info:
  * @store: a CamelStore
  * @top: the name of the folder to start from
- * @fast: whether or not to do a "fast" scan.
- * @recursive: whether to include information for subfolders
+ * @flags: various CAMEL_STORE_FOLDER_INFO_* flags to control behavior
  * @ex: a CamelException
  *
  * This fetches information about the folder structure of @store,
  * starting with @top, and returns a tree of CamelFolderInfo
- * structures. If @fast is %TRUE, the message_count or
- * unread_message_count fields of some or all of the structures may be
- * set to -1, if the store cannot determine that information quickly.
- * If @recursive is %TRUE, the returned tree will include all levels of
- * hierarchy below @top. If it is %FALSE, it will only include the
- * immediate subfolders of @top.
- *
+ * structures. If @flags includes %CAMEL_STORE_FOLDER_INFO_SUBSCRIBED,
+ * only subscribed folders will be listed. (This flag can only be used
+ * for stores that support subscriptions.) If @flags includes
+ * %CAMEL_STORE_FOLDER_INFO_RECURSIVE, the returned tree will include
+ * all levels of hierarchy below @top. If not, it will only include
+ * the immediate subfolders of @top. If @flags includes
+ * %CAMEL_STORE_FOLDER_INFO_FAST, the unread_message_count fields of
+ * some or all of the structures may be set to -1, if the store cannot
+ * determine that information quickly.
+ * 
  * Return value: a CamelFolderInfo tree, which must be freed with
  * camel_store_free_folder_info.
  **/
 CamelFolderInfo *
 camel_store_get_folder_info (CamelStore *store, const char *top,
-			     gboolean fast, gboolean recursive,
-			     gboolean subscribed_only,
-			     CamelException *ex)
+			     guint32 flags, CamelException *ex)
 {
 	CamelFolderInfo *ret;
 
 	g_return_val_if_fail (CAMEL_IS_STORE (store), NULL);
+	g_return_val_if_fail ((store->flags & CAMEL_STORE_SUBSCRIPTIONS) ||
+			      !(flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED),
+			      NULL);
 
 	CAMEL_STORE_LOCK(store, folder_lock);
-
-	ret = CS_CLASS (store)->get_folder_info (store, top, fast, recursive, subscribed_only, ex);
-
+	ret = CS_CLASS (store)->get_folder_info (store, top, flags, ex);
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 
 	return ret;
@@ -776,3 +778,28 @@ camel_store_unsubscribe_folder (CamelStore *store,
 
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 }
+
+
+int
+camel_mkdir_hier (const char *path, mode_t mode)
+{
+	char *copy, *p;
+	
+	p = copy = g_strdup (path);
+	do {
+		p = strchr (p + 1, '/');
+		if (p)
+			*p = '\0';
+		if (access (copy, F_OK) == -1) {
+			if (mkdir (copy, mode) == -1) {
+				g_free (copy);
+				return -1;
+			}
+		}
+		if (p)
+			*p = '/';
+	} while (p);
+	
+	g_free (copy);
+	return 0;
+}
