@@ -5,7 +5,7 @@
  * Author : 
  *  Bertrand Guiheneuf <bertrand@helixcode.com>
  *
- * Copyright 1999, 2000 Helix Code, Inc. (http://www.helixcode.com)
+ * Copyright 1999, 2000 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,9 +23,10 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
-#include "camel.h"
-#include <unicode.h>
+#endif
+
 #ifdef HAVE_NSS
 #include <nspr.h>
 #include <prthread.h>
@@ -33,10 +34,22 @@
 #include <ssl.h>
 #endif /* HAVE_NSS */
 
+#include "camel.h"
+
 gboolean camel_verbose_debug = FALSE;
 
+#ifdef HAVE_NSS
+static void
+camel_shutdown (void)
+{
+	NSS_Shutdown ();
+	
+	PR_Cleanup ();
+}
+#endif /* HAVE_NSS */
+
 gint
-camel_init (const char *certdb_dir, gboolean nss_init)
+camel_init (const char *configdir, gboolean nss_init)
 {
 #ifdef ENABLE_THREADS
 #ifdef G_THREADS_ENABLED	
@@ -49,18 +62,21 @@ camel_init (const char *certdb_dir, gboolean nss_init)
 	if (getenv ("CAMEL_VERBOSE_DEBUG"))
 		camel_verbose_debug = TRUE;
 	
-	unicode_init ();
-	
 #ifdef HAVE_NSS
 	if (nss_init) {
 		PR_Init (PR_SYSTEM_THREAD, PR_PRIORITY_NORMAL, 10);
 		
-		if (NSS_Init (certdb_dir) == SECFailure) {
-			g_warning ("Failed to initialize NSS");
-			return -1;
+		if (NSS_InitReadWrite (configdir) == SECFailure) {
+			/* fall back on using volatile dbs? */
+			if (NSS_NoDB_Init (configdir) == SECFailure) {
+				g_warning ("Failed to initialize NSS");
+				return -1;
+			}
 		}
 		
 		NSS_SetDomesticPolicy ();
+		
+		g_atexit (camel_shutdown);
 	}
 	
 	SSL_OptionSetDefault (SSL_ENABLE_SSL2, PR_TRUE);
@@ -71,13 +87,3 @@ camel_init (const char *certdb_dir, gboolean nss_init)
 	
 	return 0;
 }
-
-void
-camel_shutdown (void)
-{
-#ifdef HAVE_NSS
-	NSS_Shutdown ();
-	
-	PR_Cleanup ();
-#endif /* HAVE_NSS */
-}
