@@ -2,9 +2,9 @@
 /* camel-smtp-transport.c : class for a smtp transport */
 
 /* 
- * Authors: Jeffrey Stedfast <fejj@helixcode.com>
+ * Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- * Copyright (C) 2000 Helix Code, Inc. (www.helixcode.com)
+ * Copyright (C) 2000 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,7 +22,9 @@
  * USA
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/param.h>
 #include <sys/types.h>
@@ -38,6 +40,7 @@
 #undef MIN
 #undef MAX
 #include "camel-mime-filter-crlf.h"
+#include "camel-mime-filter-stripheader.h"
 #include "camel-mime-filter-linewrap.h"
 #include "camel-stream-filter.h"
 #include "camel-smtp-transport.h"
@@ -51,10 +54,13 @@
 #include "camel-tcp-stream-ssl.h"
 #include <prnetdb.h>
 #endif
+#ifdef HAVE_OPENSSL
+#include "camel-tcp-stream-openssl.h"
+#endif
 #include "camel-session.h"
 #include "camel-exception.h"
 #include "camel-sasl.h"
-#include <gal/util/e-util.h>
+#include "string-utils.h"
 
 #define d(x) x
 
@@ -218,7 +224,7 @@ get_smtp_error_string (int error)
 }
 
 static gboolean
-smtp_connect (CamelService *service, CamelException *ex)
+connect_to_server (CamelService *service, CamelException *ex)
 {
 	CamelSmtpTransport *transport = CAMEL_SMTP_TRANSPORT (service);
 	CamelStream *tcp_stream;
@@ -241,18 +247,24 @@ smtp_connect (CamelService *service, CamelException *ex)
 	
 	port = service->url->port ? service->url->port : SMTP_PORT;
 	
-#ifdef HAVE_NSS
+#if defined(HAVE_NSS) || defined(HAVE_OPENSSL)
 	if (transport->use_ssl) {
 		port = service->url->port ? service->url->port : 465;
+#ifdef HAVE_NSS
+		/* use the preferred implementation - NSS */
 		tcp_stream = camel_tcp_stream_ssl_new (service, service->url->host);
+#else
+		tcp_stream = camel_tcp_stream_openssl_new (service, service->url->host);
+#endif /* HAVE_NSS */
 	} else {
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
 #else
 	tcp_stream = camel_tcp_stream_raw_new ();
-#endif /* HAVE_NSS */
+#endif /* HAVE_NSS || HAVE_OPENSSL */
 	
 	ret = camel_tcp_stream_connect (CAMEL_TCP_STREAM (tcp_stream), h, port);
+	camel_free_host(h);
 	if (ret == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				      _("Could not connect to %s (port %d): %s"),
@@ -322,6 +334,17 @@ smtp_connect (CamelService *service, CamelException *ex)
 		smtp_helo (transport, ex);
 	}
 	
+	return TRUE;
+}
+
+static gboolean
+smtp_connect (CamelService *service, CamelException *ex)
+{
+	CamelSmtpTransport *transport = CAMEL_SMTP_TRANSPORT (service);
+
+	if (!connect_to_server (service, ex))
+		return FALSE;
+
 	/* check to see if AUTH is required, if so...then AUTH ourselves */
 	if (service->url->authmech) {
 		CamelSession *session = camel_service_get_session (service);
@@ -331,8 +354,8 @@ smtp_connect (CamelService *service, CamelException *ex)
 		
 		if (!transport->is_esmtp || !g_hash_table_lookup (transport->authtypes, service->url->authmech)) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      "SMTP server %s does not support requested "
-					      "authentication type %s", service->url->host,
+					      _("SMTP server %s does not support requested "
+					      "authentication type %s"), service->url->host,
 					      service->url->authmech);
 			camel_service_disconnect (service, TRUE, NULL);
 			return FALSE;
@@ -341,7 +364,7 @@ smtp_connect (CamelService *service, CamelException *ex)
 		authtype = camel_sasl_authtype (service->url->authmech);
 		if (!authtype) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      "No support for authentication type %s",
+					      _("No support for authentication type %s"),
 					      service->url->authmech);
 			camel_service_disconnect (service, TRUE, NULL);
 			return FALSE;
@@ -361,9 +384,8 @@ smtp_connect (CamelService *service, CamelException *ex)
 		while (!authenticated) {
 			if (errbuf) {
 				/* We need to un-cache the password before prompting again */
-				camel_session_query_authenticator (
-					session, CAMEL_AUTHENTICATOR_TELL, NULL,
-					TRUE, service, "password", ex);
+				camel_session_forget_password (
+					session, service, "password", ex);
 				g_free (service->url->passwd);
 				service->url->passwd = NULL;
 			}
@@ -376,9 +398,9 @@ smtp_connect (CamelService *service, CamelException *ex)
 							  service->url->host);
 				
 				service->url->passwd =
-					camel_session_query_authenticator (
-						session, CAMEL_AUTHENTICATOR_ASK,
-						prompt, TRUE, service, "password", ex);
+					camel_session_get_password (
+						session, prompt, TRUE,
+						service, "password", ex);
 				
 				g_free (prompt);
 				g_free (errbuf);
@@ -395,14 +417,17 @@ smtp_connect (CamelService *service, CamelException *ex)
 			authenticated = smtp_auth (transport, authtype->authproto, ex);
 			if (!authenticated) {
 				errbuf = g_strdup_printf (_("Unable to authenticate "
-							    "to IMAP server.\n%s\n\n"),
+							    "to SMTP server.\n%s\n\n"),
 							  camel_exception_get_description (ex));
 				camel_exception_clear (ex);
 			}
 		}
 		
-		/* we have to re-EHLO */
-		smtp_helo (transport, ex);
+		/* The spec says we have to re-EHLO, but some servers
+		 * we won't bother to name don't want you to... so ignore
+		 * errors.
+		 */
+		smtp_helo (transport, NULL);
 	}
 	
 	return TRUE;
@@ -485,10 +510,10 @@ query_auth_types (CamelService *service, CamelException *ex)
 	CamelServiceAuthType *authtype;
 	GList *types, *t, *next;
 	
-	if (!smtp_connect (service, ex))
+	if (!connect_to_server (service, ex))
 		return NULL;
 	
-	types = camel_sasl_authtype_list (TRUE);
+	types = g_list_copy (service->provider->authtypes);
 	for (t = types; t; t = next) {
 		authtype = t->data;
 		next = t->next;
@@ -499,6 +524,7 @@ query_auth_types (CamelService *service, CamelException *ex)
 		}
 	}
 	
+	smtp_disconnect (service, TRUE, NULL);
 	return types;
 }
 
@@ -592,7 +618,7 @@ smtp_send (CamelTransport *transport, CamelMedium *message, CamelException *ex)
 	bcc = camel_mime_message_get_recipients (CAMEL_MIME_MESSAGE (message), CAMEL_RECIPIENT_TYPE_BCC);
 	
 	/* get all of the To addresses into our recipient list */
-	len = CAMEL_ADDRESS (to)->addresses->len;
+	len = camel_address_length (CAMEL_ADDRESS (to));
 	for (index = 0; index < len; index++) {
 		const char *addr;
 		
@@ -601,7 +627,7 @@ smtp_send (CamelTransport *transport, CamelMedium *message, CamelException *ex)
 	}
 	
 	/* get all of the Cc addresses into our recipient list */
-	len = CAMEL_ADDRESS (cc)->addresses->len;
+	len = camel_address_length (CAMEL_ADDRESS (cc));
 	for (index = 0; index < len; index++) {
 		const char *addr;
 		
@@ -610,7 +636,7 @@ smtp_send (CamelTransport *transport, CamelMedium *message, CamelException *ex)
 	}
 	
 	/* get all of the Bcc addresses into our recipient list */
-	len = CAMEL_ADDRESS (bcc)->addresses->len;
+	len = camel_address_length (CAMEL_ADDRESS (bcc));
 	for (index = 0; index < len; index++) {
 		const char *addr;
 		
@@ -663,7 +689,7 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
 			int error;
-
+			
 			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -672,7 +698,7 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 			return FALSE;
 		}
 		
-		if (e_strstrcase (respbuf, "8BITMIME")) {
+		if (strstrcase (respbuf, "8BITMIME")) {
 			d(fprintf (stderr, "This server supports 8bit MIME\n"));
 			CAMEL_TRANSPORT (transport)->supports_8bit = TRUE;
 		}
@@ -693,43 +719,52 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 static gboolean
 smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 {
-	gchar *cmdbuf, *respbuf = NULL;
+	gchar *cmdbuf, *respbuf = NULL, *challenge;
 	CamelSasl *sasl;
 	
-	/* tell the server we want to authenticate... */
-	cmdbuf = g_strdup_printf ("AUTH %s\r\n", mech);
+	sasl = camel_sasl_new ("smtp", mech, CAMEL_SERVICE (transport));
+	if (!sasl) {
+		g_free (respbuf);
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Error creating SASL authentication object."));
+		return FALSE;
+	}
+	
+	challenge = camel_sasl_challenge_base64 (sasl, NULL, ex);
+	if (challenge) {
+		cmdbuf = g_strdup_printf ("AUTH %s %s\r\n", mech, challenge);
+		g_free (challenge);
+	} else
+		cmdbuf = g_strdup_printf ("AUTH %s\r\n", mech);
+	
 	d(fprintf (stderr, "sending : %s", cmdbuf));
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("AUTH request timed out: %s"),
 				      g_strerror (errno));
-		return FALSE;
+		goto lose;
 	}
 	g_free (cmdbuf);
 	
-	/* get the base64 encoded server challenge which should follow a 334 code */
 	respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
 	d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
-	if (!respbuf || strncmp (respbuf, "334", 3)) {
-		g_free (respbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("AUTH request timed out: %s"),
-				      g_strerror (errno));
-		return FALSE;
-	}
-	
-	sasl = camel_sasl_new ("smtp", mech, CAMEL_SERVICE (transport));
-	if (!sasl) {
-		g_free (respbuf);
-		goto break_and_lose;
-	}
 	
 	while (!camel_sasl_authenticated (sasl)) {
-		char *challenge;
-		
-		if (!respbuf)
+		if (!respbuf) {
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("AUTH request timed out: %s"),
+					      g_strerror (errno));
 			goto lose;
+		}
+		
+		/* the server challenge/response should follow a 334 code */
+		if (strncmp (respbuf, "334", 3)) {
+			g_free (respbuf);
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("AUTH request failed."));
+			goto lose;
+		}
 		
 		/* eat whtspc */
 		for (challenge = respbuf + 4; isspace (*challenge); challenge++);
@@ -775,7 +810,8 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 				     _("Bad authentication response from server.\n"));
 	}
 	
-	camel_object_unref (CAMEL_OBJECT (sasl));
+	if (sasl)
+		camel_object_unref (CAMEL_OBJECT (sasl));
 	
 	return FALSE;
 }
@@ -876,7 +912,7 @@ smtp_data (CamelSmtpTransport *transport, CamelMedium *message, gboolean has_8bi
 	/* now we can actually send what's important :p */
 	gchar *cmdbuf, *respbuf = NULL;
 	CamelStreamFilter *filtered_stream;
-	CamelMimeFilter *crlffilter;
+	CamelMimeFilter *crlffilter, *bccfilter;
 	
 	/* if the message contains 8bit mime parts and the server
            doesn't support it, encode 8bit parts to the best
@@ -914,13 +950,15 @@ smtp_data (CamelSmtpTransport *transport, CamelMedium *message, gboolean has_8bi
 				      get_smtp_error_string (error));
 		return FALSE;
 	}
-
+	
 	g_free (respbuf);
 	respbuf = NULL;
 	
 	/* setup stream filtering */
 	crlffilter = camel_mime_filter_crlf_new (CAMEL_MIME_FILTER_CRLF_ENCODE, CAMEL_MIME_FILTER_CRLF_MODE_CRLF_DOTS);
+	bccfilter = camel_mime_filter_stripheader_new ("Bcc");
 	filtered_stream = camel_stream_filter_new_with_stream (transport->ostream);
+	camel_stream_filter_add (filtered_stream, CAMEL_MIME_FILTER (bccfilter));
 	camel_stream_filter_add (filtered_stream, CAMEL_MIME_FILTER (crlffilter));
 	
 	if (camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message), CAMEL_STREAM (filtered_stream)) == -1) {
