@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* msg-composer-hdrs.c
  *
- * Copyright (C) 1999 Helix Code, Inc.
+ * Copyright (C) 1999 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,20 +25,29 @@
 #include <config.h>
 #endif
 
-#include <gnome.h>
-#include <camel/camel.h>
-
-#include <bonobo.h>
+#include <bonobo/bonobo-control.h>
+#include <bonobo/bonobo-widget.h>
 
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtktooltips.h>
+#include <libgnomeui/gnome-uidefs.h>
 #include <liboaf/liboaf.h>
 
 #include "Composer.h"
 
-#include "e-msg-composer-hdrs.h"
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtktooltips.h>
+
 #include <gal/e-text/e-entry.h>
 #include <gal/widgets/e-unicode.h>
 
+#include <camel/camel.h>
+#include <e-destination.h>
+#include "e-msg-composer-hdrs.h"
 #include "mail/mail-config.h"
+#include "addressbook/backend/ebook/e-book-util.h"
 
 
 #define SELECT_NAMES_OAFID "OAFIID:GNOME_Evolution_Addressbook_SelectNames"
@@ -76,6 +85,7 @@ enum {
 	SHOW_ADDRESS_DIALOG,
 	SUBJECT_CHANGED,
 	HDRS_CHANGED,
+	FROM_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -163,6 +173,7 @@ from_changed (GtkWidget *item, gpointer data)
 	EMsgComposerHdrs *hdrs = E_MSG_COMPOSER_HDRS (data);
 	
 	hdrs->account = gtk_object_get_data (GTK_OBJECT (item), "account");
+	gtk_signal_emit (GTK_OBJECT (hdrs), signals [FROM_CHANGED]);
 }
 
 static GtkWidget *
@@ -550,6 +561,15 @@ class_init (EMsgComposerHdrsClass *class)
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
+	signals[FROM_CHANGED] =
+		gtk_signal_new ("from_changed",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EMsgComposerHdrsClass,
+						   from_changed),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
 
@@ -612,20 +632,43 @@ e_msg_composer_hdrs_new (gint visible_flags)
 static void
 set_recipients (CamelMimeMessage *msg, GtkWidget *entry_widget, const gchar *type)
 {
+	EDestination **destv;
 	CamelInternetAddress *addr;
-	char *string;
+	char *string = NULL, *dest_str = NULL;
+	int i;
 	
 	bonobo_widget_get_property (BONOBO_WIDGET (entry_widget), "text", &string, NULL);
+	destv = e_destination_importv (string);
 	
-	addr = camel_internet_address_new ();
-	camel_address_unformat (CAMEL_ADDRESS (addr), string);
+	g_message ("importv: [%s]", string);
 	
-	/* TODO: In here, we could cross-reference the names with an alias book
-	   or address book, it should be sufficient for unformat to do the parsing too */
+	if (destv) {
+		dest_str = e_destination_get_address_textv (destv);
+		g_message ("destination is: %s", dest_str);
+		
+		/* dest_str has been utf8 encoded 2x by this point...not good */
+		
+		if (dest_str && *dest_str) {
+			addr = camel_internet_address_new ();
+			camel_address_unformat (CAMEL_ADDRESS (addr), dest_str);
+			
+			/* TODO: In here, we could cross-reference the names with an alias book
+			   or address book, it should be sufficient for unformat to do the parsing too */
+			
+			camel_mime_message_set_recipients (msg, type, addr);
+			
+			camel_object_unref (CAMEL_OBJECT (addr));
+			
+			g_free (dest_str);
+		}
+		
+		for (i = 0; destv[i]; i++) {
+			e_destination_touch (destv[i]);
+			gtk_object_unref (GTK_OBJECT (destv[i]));
+		}
+		g_free (destv);
+	}
 	
-	camel_mime_message_set_recipients (msg, type, addr);
-	
-	camel_object_unref (CAMEL_OBJECT (addr));
 	g_free (string);
 }
 
@@ -693,9 +736,9 @@ e_msg_composer_hdrs_set_from_account (EMsgComposerHdrs *hdrs,
 	
 	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-
+	
 	omenu = GTK_OPTION_MENU (hdrs->priv->from.entry);
-
+	
 	/* find the item that represents the account and activate it */
 	l = hdrs->priv->from_options;
 	while (l) {
@@ -728,13 +771,14 @@ e_msg_composer_hdrs_set_reply_to (EMsgComposerHdrs *hdrs,
 				    "text", reply_to, NULL);
 }
 
+/* FIXME: these shouldn't take GLists, they should take CamelInternetAddress's */
 void
 e_msg_composer_hdrs_set_to (EMsgComposerHdrs *hdrs,
 			    const GList *to_list)
 {
 	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-
+	
 	set_entry (BONOBO_WIDGET (hdrs->priv->to.entry), to_list);
 }
 
@@ -744,7 +788,7 @@ e_msg_composer_hdrs_set_cc (EMsgComposerHdrs *hdrs,
 {
 	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-
+	
 	set_entry (BONOBO_WIDGET (hdrs->priv->cc.entry), cc_list);
 }
 
@@ -754,7 +798,7 @@ e_msg_composer_hdrs_set_bcc (EMsgComposerHdrs *hdrs,
 {
 	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-
+	
 	set_entry (BONOBO_WIDGET (hdrs->priv->bcc.entry), bcc_list);
 }
 
@@ -765,7 +809,7 @@ e_msg_composer_hdrs_set_subject (EMsgComposerHdrs *hdrs,
 	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	g_return_if_fail (subject != NULL);
-
+	
 	gtk_object_set (GTK_OBJECT (hdrs->priv->subject.entry),
 			"text", subject,
 			NULL);
