@@ -1,8 +1,8 @@
 /*
- *  Copyright (C) 2000 Helix Code Inc.
+ *  Copyright (C) 2000 Ximian Inc.
  *
  *  Authors: Not Zed <notzed@lostzed.mmc.com.au>
- *           Jeffrey Stedfast <fejj@helixcode.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -20,7 +20,12 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
+#include <glib.h>
+#include <gtk/gtkframe.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-stock.h>
 #include <glade/glade.h>
 
 #include <gal/widgets/e-unicode.h>
@@ -237,6 +242,7 @@ rule_add (GtkWidget *widget, RuleEditor *re)
 				   GNOME_STOCK_BUTTON_CANCEL,
 				   NULL);
 	
+	gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 400);
 	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), rules, TRUE, TRUE, 0);
@@ -295,6 +301,7 @@ rule_edit (GtkWidget *widget, RuleEditor *re)
 				   GNOME_STOCK_BUTTON_CANCEL,
 				   NULL);
 	
+	gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 400);
 	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), rules, TRUE, TRUE, 0);
@@ -313,15 +320,22 @@ rule_delete (GtkWidget *widget, RuleEditor *re)
 	d(printf ("delete rule\n"));
 	pos = rule_context_get_rank_rule (re->context, re->current, re->source);
 	if (pos != -1) {
+		gint len;
+		
 		rule_context_remove_rule (re->context, re->current);
 		
-		item = g_list_nth_data (((GtkList *)re->list)->children, pos);
+		item = g_list_nth_data (GTK_LIST (re->list)->children, pos);
 		l = g_list_append (NULL, item);
 		gtk_list_remove_items (re->list, l);
 		g_list_free (l);
 		
 		gtk_object_unref (GTK_OBJECT (re->current));
 		re->current = NULL;
+		
+		/* now select the next rule */
+		len = g_list_length (GTK_LIST (re->list)->children);
+		pos = pos >= len ? len - 1 : pos;
+		gtk_list_select_item (GTK_LIST (re->list), pos);
 	}
 	
 	rule_editor_set_sensitive (re);
@@ -451,23 +465,26 @@ rule_editor_construct(RuleEditor *re, RuleContext *context, GladeXML *gui, const
 {
 	GtkWidget *w;
 	int i;
-
-	re->context = context;
-	gtk_object_ref((GtkObject *)context);
-
-        w = glade_xml_get_widget(gui, "rule_editor");
-	gtk_box_pack_start((GtkBox *)GNOME_DIALOG(re)->vbox, w, TRUE, TRUE, 0);
 	
-	for (i=0; i<BUTTON_LAST; i++) {
-		re->priv->buttons[i] = (GtkButton *)w = glade_xml_get_widget(gui, edit_buttons[i].name);
-		gtk_signal_connect(GTK_OBJECT(w), "clicked", edit_buttons[i].func, re);
+	re->context = context;
+	gtk_object_ref (GTK_OBJECT (context));
+	
+	gtk_window_set_policy (GTK_WINDOW (re), FALSE, TRUE, FALSE);
+	
+        w = glade_xml_get_widget (gui, "rule_editor");
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (re)->vbox), w, TRUE, TRUE, 0);
+	
+	for (i = 0; i < BUTTON_LAST; i++) {
+		re->priv->buttons[i] = (GtkButton *)w = glade_xml_get_widget (gui, edit_buttons[i].name);
+		gtk_signal_connect (GTK_OBJECT (w), "clicked", edit_buttons[i].func, re);
 	}
-
+	
         re->list = (GtkList *) w = glade_xml_get_widget(gui, "rule_list");
-	gtk_signal_connect(GTK_OBJECT(w), "select_child", select_rule, re);
-	gtk_signal_connect(GTK_OBJECT(w), "button_press_event", double_click, re);
-
-	rule_editor_set_source(re, source);
-
-	gnome_dialog_append_buttons((GnomeDialog *)re, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, 0);
+	gtk_signal_connect (GTK_OBJECT (w), "select_child", select_rule, re);
+	gtk_signal_connect (GTK_OBJECT (w), "button_press_event", double_click, re);
+	
+	rule_editor_set_source (re, source);
+	
+	gnome_dialog_append_buttons (GNOME_DIALOG (re), GNOME_STOCK_BUTTON_OK,
+				     GNOME_STOCK_BUTTON_CANCEL, NULL);
 }
