@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright 2001 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,8 @@ typedef struct {
 	gchar *address;
 	gchar *organization;
 	gchar *signature;
+	gchar *html_signature;
+	gboolean has_html_signature;
 } MailConfigIdentity;
 
 typedef struct {
@@ -54,11 +56,33 @@ typedef struct {
 	MailConfigIdentity *id;
 	MailConfigService *source;
 	MailConfigService *transport;
-
+	
 	gchar *drafts_folder_name, *drafts_folder_uri;
 	gchar *sent_folder_name, *sent_folder_uri;
+	
+	gchar *pgp_key;
+	gboolean pgp_encrypt_to_self;
+	
+	gchar *smime_key;
+	gboolean smime_encrypt_to_self;
 } MailConfigAccount;
 
+typedef enum {
+	MAIL_CONFIG_HTTP_NEVER, MAIL_CONFIG_HTTP_SOMETIMES,
+	MAIL_CONFIG_HTTP_ALWAYS
+} MailConfigHTTPMode;
+
+typedef enum {
+	MAIL_CONFIG_FORWARD_ATTACHED, MAIL_CONFIG_FORWARD_INLINE,
+	MAIL_CONFIG_FORWARD_QUOTED
+} MailConfigForwardStyle;
+
+typedef enum {
+	MAIL_CONFIG_DISPLAY_NORMAL, MAIL_CONFIG_DISPLAY_FULL_HEADERS,
+	MAIL_CONFIG_DISPLAY_SOURCE,
+	MAIL_CONFIG_DISPLAY_MAX
+} MailConfigDisplayStyle;
+
 /* Identities */
 MailConfigIdentity *identity_copy (const MailConfigIdentity *id);
 void                identity_destroy (MailConfigIdentity *id);
@@ -82,14 +106,17 @@ void mail_config_write_on_exit (void);
 /* General Accessor functions */
 gboolean mail_config_is_configured            (void);
 
-gboolean mail_config_get_thread_list          (void);
-void     mail_config_set_thread_list          (gboolean value);
+gboolean mail_config_get_empty_trash_on_exit  (void);
+void     mail_config_set_empty_trash_on_exit  (gboolean value);
 
-gboolean mail_config_get_view_source          (void);
-void     mail_config_set_view_source          (gboolean value);
+gboolean mail_config_get_thread_list          (const char *uri);
+void     mail_config_set_thread_list          (const char *uri, gboolean value);
 
-gboolean mail_config_get_hide_deleted          (void);
-void     mail_config_set_hide_deleted          (gboolean value);
+gboolean mail_config_get_show_preview         (const char *uri);
+void     mail_config_set_show_preview         (const char *uri, gboolean value);
+
+gboolean mail_config_get_hide_deleted         (void);
+void     mail_config_set_hide_deleted         (gboolean value);
 
 gint     mail_config_get_paned_size           (void);
 void     mail_config_set_paned_size           (gint size);
@@ -103,24 +130,46 @@ void     mail_config_set_citation_highlight   (gboolean);
 guint32  mail_config_get_citation_color       (void);
 void     mail_config_set_citation_color       (guint32);
 
+gint     mail_config_get_do_seen_timeout      (void);
+void     mail_config_set_do_seen_timeout      (gboolean do_seen_timeout);
+
 gint     mail_config_get_mark_as_seen_timeout (void);
 void     mail_config_set_mark_as_seen_timeout (gint timeout);
 
 gboolean mail_config_get_prompt_empty_subject (void);
 void     mail_config_set_prompt_empty_subject (gboolean value);
 
-gint     mail_config_get_pgp_type (void);
-void     mail_config_set_pgp_type (gint pgp_type);
+gboolean mail_config_get_prompt_only_bcc (void);
+void     mail_config_set_prompt_only_bcc (gboolean value);
+
+CamelPgpType mail_config_get_pgp_type (void);
+void         mail_config_set_pgp_type (CamelPgpType pgp_type);
 
 const char *mail_config_get_pgp_path (void);
 void        mail_config_set_pgp_path (const char *pgp_path);
 
-const MailConfigAccount  *mail_config_get_default_account (void);
-const MailConfigAccount  *mail_config_get_account_by_name (const char *account_name);
-const GSList             *mail_config_get_accounts        (void);
-void                      mail_config_add_account         (MailConfigAccount *account);
-const GSList             *mail_config_remove_account      (MailConfigAccount *account);
-void                      mail_config_set_default_account (const MailConfigAccount *account);
+gboolean    mail_config_get_remember_pgp_passphrase (void);
+void        mail_config_set_remember_pgp_passphrase (gboolean value);
+
+MailConfigHTTPMode mail_config_get_http_mode (void);
+void               mail_config_set_http_mode (MailConfigHTTPMode);
+
+MailConfigForwardStyle mail_config_get_default_forward_style (void);
+void                   mail_config_set_default_forward_style (MailConfigForwardStyle style);
+
+MailConfigDisplayStyle mail_config_get_message_display_style (void);
+void                   mail_config_set_message_display_style (MailConfigDisplayStyle style);
+
+const char *mail_config_get_default_charset (void);
+void        mail_config_set_default_charset (const char *charset);
+
+const MailConfigAccount  *mail_config_get_default_account       (void);
+const MailConfigAccount  *mail_config_get_account_by_name       (const char *account_name);
+const MailConfigAccount  *mail_config_get_account_by_source_url (const char *url);
+const GSList             *mail_config_get_accounts              (void);
+void                      mail_config_add_account               (MailConfigAccount *account);
+const GSList             *mail_config_remove_account            (MailConfigAccount *account);
+void                      mail_config_set_default_account       (const MailConfigAccount *account);
 
 const MailConfigIdentity *mail_config_get_default_identity (void);
 const MailConfigService  *mail_config_get_default_transport (void);
@@ -129,6 +178,7 @@ const MailConfigService  *mail_config_get_default_news    (void);
 const GSList             *mail_config_get_news            (void);
 void                      mail_config_add_news            (MailConfigService *news);
 const GSList             *mail_config_remove_news         (MailConfigService *news);
+GtkType evolution_mail_config_get_type (void);
 
 /* convenience functions to help ease the transition over to the new codebase */
 GSList *mail_config_get_sources (void);
@@ -138,6 +188,10 @@ char *mail_config_folder_to_cachename (CamelFolder *folder, const char *prefix);
 
 gboolean  mail_config_check_service (const char *url, CamelProviderType type, GList **authtypes);
 
+
+
+void evolution_mail_config_factory_init (void);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
