@@ -3,12 +3,12 @@
 
 /* 
  * Authors: 
- *  Michael Zucchi <NotZed@helixcode.com>
- *  Peter Williams <peterw@helixcode.com>
- *  Ettore Perazzoli <ettore@helixcode.com>
- *  Dan Winship <danw@helixcode.com>
+ *  Michael Zucchi <NotZed@ximian.com>
+ *  Peter Williams <peterw@ximian.com>
+ *  Ettore Perazzoli <ettore@ximian.com>
+ *  Dan Winship <danw@ximian.com>
  *
- * Copyright 2000 Helix Code, Inc. (http://www.helixcode.com)
+ * Copyright 2000 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,20 +34,23 @@
   the maintainer of this information.
 
 */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <bonobo.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <glade/glade.h>
 #include <gnome-xml/xmlmemory.h>
+#include <libgnomeui/gnome-dialog.h>
+#include <libgnomeui/gnome-dialog-util.h>
+#include <glade/glade.h>
 
 #include "Evolution.h"
 #include "evolution-storage.h"
 #include "evolution-shell-component.h"
 #include "evolution-storage-listener.h"
 
+#include "e-util/e-path.h"
+
 #include "camel/camel.h"
 #include "camel/camel-vee-store.h"
 #include "camel/camel-vee-folder.h"
@@ -62,7 +65,7 @@
 #include "mail-tools.h"
 #include "folder-browser.h"
 #include "mail-mt.h"
-
+#include "mail-folder-cache.h"
 #include "mail-vfolder.h"
 
 #define d(x)
@@ -128,8 +131,7 @@ load_metainfo(const char *path)
 	meta->format = g_strdup("mbox"); /* defaults */
 	meta->name = g_strdup("mbox");
 	meta->indexed = TRUE;
-	if (doc)
-		xmlFreeDoc(doc);
+	xmlFreeDoc(doc);
 	return meta;
 }
 
@@ -181,7 +183,6 @@ typedef struct {
 	char *local_path;
 	int local_pathlen;
 	GHashTable *folders; /* points to MailLocalFolder */
-	GHashTable *unread;
 } MailLocalStore;
 
 typedef struct {
@@ -192,22 +193,18 @@ typedef struct {
 	CamelFolder *folder;
 	MailLocalStore *local_store;
 	char *path, *name, *uri;
-	int last_unread;
 } MailLocalFolder;
 
 static MailLocalStore *local_store;
 
 CamelType mail_local_store_get_type (void);
 
-static void local_folder_changed_proxy (CamelObject *folder, gpointer event_data, gpointer user_data);
-
 static char *get_name (CamelService *service, gboolean brief);
 
 static CamelFolder *get_folder (CamelStore *store, const char *folder_name,
 				guint32 flags, CamelException *ex);
 static CamelFolderInfo *get_folder_info (CamelStore *store, const char *top,
-					 gboolean fast, gboolean recursive,
-					 gboolean subscribed_only, CamelException *ex);
+					 guint32 flags, CamelException *ex);
 static void delete_folder (CamelStore *store, const char *folder_name,
 			   CamelException *ex);
 static void rename_folder (CamelStore *store, const char *old_name,
@@ -251,17 +248,11 @@ mail_local_store_init (gpointer object, gpointer klass)
 }
 
 static void
-free_local_folder(MailLocalFolder *lf)
+free_local_folder (MailLocalFolder *lf)
 {
-	if (lf->folder) {
-		camel_object_unhook_event((CamelObject *)lf->folder,
-					  "folder_changed", local_folder_changed_proxy,
-					  lf);
-		camel_object_unhook_event((CamelObject *)lf->folder,
-					  "message_changed", local_folder_changed_proxy,
-					  lf);
+	if (lf->folder)
 		camel_object_unref((CamelObject *)lf->folder);
-	}
+
 	g_free(lf->path);
 	g_free(lf->name);
 	g_free(lf->uri);
@@ -284,10 +275,8 @@ mail_local_store_finalize (gpointer object)
 	CORBA_Environment ev;
 
 	CORBA_exception_init (&ev);
-	if (!CORBA_Object_is_nil (local_store->corba_local_storage, &ev)) {
-		Bonobo_Unknown_unref (local_store->corba_local_storage, &ev);
-		CORBA_Object_release (local_store->corba_local_storage, &ev);
-	}
+	if (!CORBA_Object_is_nil (local_store->corba_local_storage, &ev))
+		bonobo_object_release_unref (local_store->corba_local_storage, &ev);
 	CORBA_exception_free (&ev);
 
 	if (local_store->local_storage_listener)
@@ -361,13 +350,8 @@ static void
 init_trash (CamelStore *store)
 {
 	MailLocalStore *local_store = MAIL_LOCAL_STORE (store);
-	char *name;
 	
-	name = g_strdup_printf ("%s?(match-all (system-flag \"Deleted\"))", "vTrash");
-	
-	store->vtrash = camel_vtrash_folder_new (store, name);
-	
-	g_free (name);
+	store->vtrash = camel_vtrash_folder_new (store, CAMEL_VTRASH_NAME);
 	
 	if (store->vtrash) {
 		/* attach to the finalize event of the vtrash */
@@ -403,8 +387,7 @@ populate_folders (gpointer key, gpointer data, gpointer user_data)
 
 static CamelFolderInfo *
 get_folder_info (CamelStore *store, const char *top,
-		 gboolean fast, gboolean recursive,
-		 gboolean subscribed_only, CamelException *ex)
+		 guint32 flags, CamelException *ex)
 {
 	MailLocalStore *local_store = MAIL_LOCAL_STORE (store);
 	CamelFolderInfo *fi = NULL;
@@ -449,47 +432,6 @@ local_storage_destroyed_cb (EvolutionStorageListener *storage_listener,
 	g_warning ("%s -- The LocalStorage has gone?!", __FILE__);
 }
 
-
-static void
-local_folder_changed (CamelObject *object, gpointer event_data,
-		      gpointer user_data)
-{
-	MailLocalFolder *local_folder = user_data;
-	int unread = GPOINTER_TO_INT (event_data);
-	char *display;
-
-	if (unread != local_folder->last_unread) {
-		CORBA_Environment ev;
-
-		CORBA_exception_init (&ev);
-		if (unread > 0) {
-			display = g_strdup_printf ("%s (%d)", local_folder->name, unread);
-			GNOME_Evolution_LocalStorage_updateFolder (
-				local_folder->local_store->corba_local_storage,
-				local_folder->path, display, TRUE, &ev);
-			g_free (display);
-		} else {
-			GNOME_Evolution_LocalStorage_updateFolder (
-				local_folder->local_store->corba_local_storage,
-				local_folder->path, local_folder->name,
-				FALSE, &ev);
-		}
-		CORBA_exception_free (&ev);
-
-		local_folder->last_unread = unread;
-	}
-}
-
-static void
-local_folder_changed_proxy (CamelObject *folder, gpointer event_data, gpointer user_data)
-{
-	int unread;
-
-	unread = camel_folder_get_unread_message_count (CAMEL_FOLDER (folder));
-	mail_proxy_event (local_folder_changed, folder,
-			  GINT_TO_POINTER (unread), user_data);
-}
-
 /* ********************************************************************** */
 /* Register folder */
 
@@ -499,7 +441,8 @@ struct _register_msg {
 	MailLocalFolder *local_folder;
 };
 
-static char *register_folder_desc(struct _mail_msg *mm, int done)
+static char *
+register_folder_desc(struct _mail_msg *mm, int done)
 {
 	struct _register_msg *m = (struct _register_msg *)mm;
 
@@ -522,14 +465,14 @@ register_folder_register(struct _mail_msg *mm)
 	meta = load_metainfo (name);
 	g_free (name);
 
-	camel_operation_register(mm->cancel);
-
+	camel_operation_register (mm->cancel);
 	name = g_strdup_printf ("%s:%s", meta->format, path);
 	store = camel_session_get_store (session, name, &mm->ex);
 	g_free (name);
+
 	if (!store) {
 		free_metainfo (meta);
-		camel_operation_unregister(mm->cancel);
+		camel_operation_unregister (mm->cancel);
 		return;
 	}
 
@@ -537,15 +480,6 @@ register_folder_register(struct _mail_msg *mm)
 	if (meta->indexed)
 		flags |= CAMEL_STORE_FOLDER_BODY_INDEX;
 	local_folder->folder = camel_store_get_folder (store, meta->name, flags, &mm->ex);
-	if (local_folder->folder) {
-		camel_object_hook_event (CAMEL_OBJECT (local_folder->folder),
-					 "folder_changed", local_folder_changed_proxy,
-					 local_folder);
-		camel_object_hook_event (CAMEL_OBJECT (local_folder->folder),
-					 "message_changed", local_folder_changed_proxy,
-					 local_folder);
-		local_folder->last_unread = camel_folder_get_unread_message_count(local_folder->folder);
-	}
 
 	camel_object_unref (CAMEL_OBJECT (store));
 	free_metainfo (meta);
@@ -558,19 +492,31 @@ register_folder_registered(struct _mail_msg *mm)
 {
 	struct _register_msg *m = (struct _register_msg *)mm;
 	MailLocalFolder *local_folder = m->local_folder;
-	int unread;
 	
 	if (local_folder->folder) {
+		gchar *name;
+
 		g_hash_table_insert (local_folder->local_store->folders, local_folder->uri + 8,
 				     local_folder);
+		/* Remove the circular ref once the local store knows aboutthe folder */
+		camel_object_unref ((CamelObject *)local_folder->local_store);
 		
 		/* add the folder to the vfolder lists FIXME: merge stuff above with this */
 		vfolder_register_source(local_folder->folder);
-		
-		unread = local_folder->last_unread;
-		local_folder->last_unread = 0;
-		local_folder_changed (CAMEL_OBJECT (local_folder->folder), GINT_TO_POINTER (unread),
-				      local_folder);
+
+		mail_folder_cache_set_update_lstorage (local_folder->uri, 
+						       local_folder->local_store->corba_local_storage,
+						       local_folder->path);
+
+		name = strrchr (local_folder->path, '/');
+		if (name) /* should always be true... */ {
+			name += 1; /* skip the slash */
+			mail_folder_cache_note_name (local_folder->uri, name);
+		}
+
+		/* Do this after specifying the name so it isn't 'mbox' */
+		mail_folder_cache_note_folder (local_folder->uri, local_folder->folder);
+
 		m->local_folder = NULL;
 	}
 }
@@ -632,17 +578,33 @@ local_storage_removed_folder_cb (EvolutionStorageListener *storage_listener,
 {
 	MailLocalStore *local_store = data;
 	MailLocalFolder *local_folder;
+	char *physical_path;
+	char *tmpname;
 
-	if (strncmp (path, "file://", 7) != 0 ||
-	    strncmp (path + 7, local_store->local_path,
+	physical_path = e_path_to_physical (local_store->local_path, path);
+
+	if (strncmp (physical_path, local_store->local_path,
 		     local_store->local_pathlen) != 0)
 		return;
 
-	local_folder = g_hash_table_lookup (local_store->folders, path + 8);
-	if (local_folder) {
-		g_hash_table_remove (local_store->folders, path);
-		free_local_folder(local_folder);
+	tmpname = strchr (physical_path, '/');
+	if (tmpname) {
+		while (*tmpname == '/')
+			tmpname++;
+		local_folder = g_hash_table_lookup (local_store->folders, tmpname);
+		camel_object_ref ((CamelObject *)local_store); /* When we go to free_local_folder() the
+								  local_store will be unref'd */
 	}
+	else
+		local_folder = NULL;
+
+	if (local_folder) {
+		g_hash_table_remove (local_store->folders, tmpname);
+
+		free_local_folder (local_folder);
+	}
+
+	g_free (physical_path);
 }
 
 static CamelProvider local_provider = {
@@ -721,7 +683,7 @@ mail_local_storage_startup (EvolutionShellClient *shellclient,
 
 	CORBA_exception_init (&ev);
 	GNOME_Evolution_Storage_addListener (local_store->corba_local_storage,
-					corba_local_storage_listener, &ev);
+					     corba_local_storage_listener, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("Cannot add a listener to the Local Storage.");
 		camel_object_unref (CAMEL_OBJECT (local_store));
@@ -732,7 +694,9 @@ mail_local_storage_startup (EvolutionShellClient *shellclient,
 }
 
 
-/* Local folder reconfiguration stuff */
+/*----------------------------------------------------------------------
+ * Local folder reconfiguration stuff
+ *----------------------------------------------------------------------*/
 
 /*
    open new
@@ -802,7 +766,6 @@ reconfigure_folder_reconfigure(struct _mail_msg *mm)
 	char *fromurl = NULL, *tourl = NULL;
 	CamelFolder *fromfolder = NULL, *tofolder = NULL;
 	GPtrArray *uids;
-	int i;
 	char *metapath;
 	char *tmpname;
 	CamelURL *url = NULL;
@@ -811,7 +774,7 @@ reconfigure_folder_reconfigure(struct _mail_msg *mm)
 
 	d(printf("reconfiguring folder: %s to type %s\n", m->fb->uri, m->newtype));
 
-	mail_status_start(_("Reconfiguring folder"));
+	mail_status (_("Reconfiguring folder"));
 
 	/* NOTE: This var is cleared by the folder_browser via the set_uri method */
 	m->fb->reconfigure = TRUE;
@@ -842,12 +805,7 @@ reconfigure_folder_reconfigure(struct _mail_msg *mm)
 	/* first, 'close' the old folder */
 	update_progress(_("Closing current folder"), 0.0);
 	camel_folder_sync(local_folder->folder, FALSE, &mm->ex);
-	camel_object_unhook_event(CAMEL_OBJECT (local_folder->folder),
-				  "folder_changed", local_folder_changed_proxy,
-				  local_folder);
-	camel_object_unhook_event(CAMEL_OBJECT (local_folder->folder),
-				  "message_changed", local_folder_changed_proxy,
-				  local_folder);
+
 	/* Once for the FolderBrowser, once for the local store */
 	camel_object_unref(CAMEL_OBJECT(local_folder->folder));
 	camel_object_unref(CAMEL_OBJECT(local_folder->folder));
@@ -987,67 +945,96 @@ static struct _mail_msg_op reconfigure_folder_op = {
 	reconfigure_folder_free,
 };
 
+/* hash table of folders that the user has a reconfig-folder dialog for */
+static GHashTable *reconfigure_folder_hash = NULL;
+
 static void
-reconfigure_clicked(GnomeDialog *d, int button, struct _reconfigure_msg *m)
+reconfigure_clicked (GnomeDialog *dialog, int button, struct _reconfigure_msg *m)
 {
 	if (button == 0) {
-		GtkMenu *menu;
+		GtkWidget *menu;
 		int type;
 		char *types[] = { "mbox", "maildir", "mh" };
-
+		
 		/* hack to clear the message list during update */
-		message_list_set_folder(m->fb->message_list, NULL);
-
-		menu = (GtkMenu *)gtk_option_menu_get_menu(m->optionlist);
-		type = g_list_index(GTK_MENU_SHELL(menu)->children, gtk_menu_get_active(menu));
+		message_list_set_folder (m->fb->message_list, NULL, FALSE);
+		
+		menu = gtk_option_menu_get_menu (m->optionlist);
+		type = g_list_index (GTK_MENU_SHELL (menu)->children,
+				     gtk_menu_get_active (GTK_MENU (menu)));
 		if (type < 0 || type > 2)
 			type = 0;
-
-		gtk_widget_set_sensitive(m->frame, FALSE);
-		gtk_widget_set_sensitive(m->apply, FALSE);
-		gtk_widget_set_sensitive(m->cancel, FALSE);
-
+		
+		gtk_widget_set_sensitive (m->frame, FALSE);
+		gtk_widget_set_sensitive (m->apply, FALSE);
+		gtk_widget_set_sensitive (m->cancel, FALSE);
+		
 		m->newtype = g_strdup (types[type]);
-		e_thread_put(mail_thread_queued, (EMsg *)m);
+		e_thread_put (mail_thread_queued, (EMsg *)m);
 	} else
-		mail_msg_free((struct _mail_msg *)m);
-
+		mail_msg_free ((struct _mail_msg *)m);
+	
 	if (button != -1)
-		gnome_dialog_close(d);
+		gnome_dialog_close (dialog);
 }
 
 void
-mail_local_reconfigure_folder(FolderBrowser *fb)
+mail_local_reconfigure_folder (FolderBrowser *fb)
 {
 	CamelStore *store;
 	GladeXML *gui;
 	GnomeDialog *gd;
 	struct _reconfigure_msg *m;
-
+	char *name, *title;
+	
 	if (fb->folder == NULL) {
-		g_warning("Trying to reconfigure nonexistant folder");
+		g_warning ("Trying to reconfigure nonexistant folder");
 		return;
 	}
-
-	m = mail_msg_new(&reconfigure_folder_op, NULL, sizeof(*m));
-	store = camel_folder_get_parent_store(fb->folder);
-
-	gui = glade_xml_new(EVOLUTION_GLADEDIR "/local-config.glade", "dialog_format");
+	
+	if ((gd = g_hash_table_lookup (reconfigure_folder_hash, fb->folder))) {
+		/* FIXME: raise this dialog?? */
+		return;
+	}
+	
+	m = mail_msg_new (&reconfigure_folder_op, NULL, sizeof (*m));
+	store = camel_folder_get_parent_store (fb->folder);
+	
+	gui = glade_xml_new (EVOLUTION_GLADEDIR "/local-config.glade", "dialog_format");
 	gd = (GnomeDialog *)glade_xml_get_widget (gui, "dialog_format");
-
+	
+	name = mail_tool_get_folder_name (fb->folder);
+	title = g_strdup_printf (_("Reconfigure %s"), name);
+	gtk_window_set_title (GTK_WINDOW (gd), title);
+	g_free (title);
+	g_free (name);
+	
 	m->frame = glade_xml_get_widget (gui, "frame_format");
 	m->apply = glade_xml_get_widget (gui, "apply_format");
 	m->cancel = glade_xml_get_widget (gui, "cancel_format");
 	m->optionlist = (GtkOptionMenu *)glade_xml_get_widget (gui, "option_format");
 	m->newtype = NULL;
 	m->fb = fb;
-	gtk_object_ref((GtkObject *)fb);
-
-	gtk_label_set_text((GtkLabel *)glade_xml_get_widget (gui, "label_format"),
-			   ((CamelService *)store)->url->protocol);
-
-	gtk_signal_connect((GtkObject *)gd, "clicked", reconfigure_clicked, m);
-	gtk_object_unref((GtkObject *)gui);
-
+	gtk_object_ref (GTK_OBJECT (fb));
+	
+	gtk_label_set_text ((GtkLabel *)glade_xml_get_widget (gui, "label_format"),
+			    ((CamelService *)store)->url->protocol);
+	
+	gtk_signal_connect (GTK_OBJECT (gd), "clicked", reconfigure_clicked, m);
+	gtk_object_unref (GTK_OBJECT (gui));
+	
+	if (!reconfigure_folder_hash)
+		reconfigure_folder_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	
+	g_hash_table_insert (reconfigure_folder_hash, (gpointer) fb->folder, (gpointer) gd);
+	
 	gnome_dialog_run_and_close (GNOME_DIALOG (gd));
+	
+	/* remove this folder from our hash since we are done with it */
+	g_hash_table_remove (reconfigure_folder_hash, fb->folder);
+	if (g_hash_table_size (reconfigure_folder_hash) == 0) {
+		/* additional cleanup */
+		g_hash_table_destroy (reconfigure_folder_hash);
+		reconfigure_folder_hash = NULL;
+	}
 }
