@@ -2,12 +2,12 @@
 /* mail-ops.c: callbacks for the mail toolbar/menus */
 
 /* 
- * Authors: Dan Winship <danw@helixcode.com>
- *  	    Jeffrey Stedfast <fejj@helixcode.com>
- *          Peter Williams <peterw@helixcode.com>
+ * Authors: Dan Winship <danw@ximian.com>
+ *  	    Jeffrey Stedfast <fejj@ximian.com>
+ *          Peter Williams <peterw@ximian.com>
  *          Michael Zucchi <notzed@ximian.com>
  *
- * Copyright 2000,2001 Ximian, Inc. (http://www.ximian.com)
+ * Copyright 2000,2001 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,13 +25,17 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
+/* #include <ctype.h> */
 #include <errno.h>
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-unicode.h>
 #include <camel/camel-mime-filter-from.h>
 #include <camel/camel-operation.h>
+#include <camel/camel-vtrash-folder.h>
 #include "mail.h"
 #include "mail-tools.h"
 #include "mail-ops.h"
@@ -43,72 +47,10 @@
 #include "filter/filter-filter.h"
 
 #include "mail-mt.h"
+#include "mail-folder-cache.h"
 
 #define d(x) x
 
-FilterContext *
-mail_load_filter_context(void)
-{
-	char *userrules;
-	char *systemrules;
-	FilterContext *fc;
-	
-	userrules = g_strdup_printf ("%s/filters.xml", evolution_dir);
-	systemrules = g_strdup_printf ("%s/evolution/filtertypes.xml", EVOLUTION_DATADIR);
-	fc = filter_context_new ();
-	rule_context_load ((RuleContext *)fc, systemrules, userrules);
-	g_free (userrules);
-	g_free (systemrules);
-	
-	return fc;
-}
-
-static void
-setup_filter_driver(CamelFilterDriver *driver, FilterContext *fc, const char *source)
-{
-	GString *fsearch, *faction;
-	FilterFilter *rule = NULL;
-
-	if (TRUE /* perform_logging */) {
-		char *filename = g_strdup_printf("%s/evolution-filter-log", evolution_dir);
-		/* FIXME: This is a nasty little thing to stop leaking file handles.
-		   Needs to be setup elsewhere. */
-		static FILE *logfile = NULL;
-
-		if (logfile == NULL)
-			logfile = fopen(filename, "a+");
-		g_free(filename);
-		if (logfile)
-			camel_filter_driver_set_logfile(driver, logfile);
-	}
-
-	fsearch = g_string_new ("");
-	faction = g_string_new ("");
-	
-	while ((rule = (FilterFilter *)rule_context_next_rule((RuleContext *)fc, (FilterRule *)rule, source))) {
-		g_string_truncate (fsearch, 0);
-		g_string_truncate (faction, 0);
-		
-		filter_rule_build_code ((FilterRule *)rule, fsearch);
-		filter_filter_build_action (rule, faction);
-
-		camel_filter_driver_add_rule(driver, ((FilterRule *)rule)->name, fsearch->str, faction->str);
-	}
-	
-	g_string_free (fsearch, TRUE);
-	g_string_free (faction, TRUE);
-}
-
-static CamelFolder *
-filter_get_folder(CamelFilterDriver *d, const char *uri, void *data, CamelException *ex)
-{
-	CamelFolder *folder;
-
-	folder = mail_tool_uri_to_folder(uri, ex);
-
-	return folder;
-}
-
 /* used for both just filtering a folder + uid's, and for filtering a whole folder */
 /* used both for fetching mail, and for filtering mail */
 struct _filter_mail_msg {
@@ -116,6 +58,7 @@ struct _filter_mail_msg {
 
 	CamelFolder *source_folder; /* where they come from */
 	GPtrArray *source_uids;	/* uids to copy, or NULL == copy all */
+	CamelUIDCache *cache;  /* UID cache if we are to cache the uids, NULL otherwise */
 	CamelOperation *cancel;
 	CamelFilterDriver *driver;
 	int delete;		/* delete messages after filtering them? */
@@ -138,49 +81,49 @@ struct _fetch_mail_msg {
 /* filter a folder, or a subset thereof, uses source_folder/source_uids */
 /* this is shared with fetch_mail */
 static void
-filter_folder_filter(struct _mail_msg *mm)
+filter_folder_filter (struct _mail_msg *mm)
 {
 	struct _filter_mail_msg *m = (struct _filter_mail_msg *)mm;
 	CamelFolder *folder;
 	GPtrArray *uids, *folder_uids = NULL;
-
+	
 	if (m->cancel)
-		camel_operation_register(m->cancel);
-
+		camel_operation_register (m->cancel);
+	
 	folder = m->source_folder;
-
+	
 	if (folder == NULL || camel_folder_get_message_count (folder) == 0) {
 		if (m->cancel)
-			camel_operation_unregister(m->cancel);
+			camel_operation_unregister (m->cancel);
 		return;
 	}
-
+	
 	if (m->destination) {
-		camel_folder_freeze(m->destination);
-		camel_filter_driver_set_default_folder(m->driver, m->destination);
+		camel_folder_freeze (m->destination);
+		camel_filter_driver_set_default_folder (m->driver, m->destination);
 	}
-
-	camel_folder_freeze(folder);
-
+	
+	camel_folder_freeze (folder);
+	
 	if (m->source_uids)
 		uids = m->source_uids;
 	else
 		folder_uids = uids = camel_folder_get_uids (folder);
-
-	camel_filter_driver_filter_folder(m->driver, folder, uids, m->delete, &mm->ex);
-
+	
+	camel_filter_driver_filter_folder (m->driver, folder, m->cache, uids, m->delete, &mm->ex);
+	
 	if (folder_uids)
-		camel_folder_free_uids(folder, folder_uids);
-
+		camel_folder_free_uids (folder, folder_uids);
+	
 	/* sync and expunge */
-	camel_folder_sync (folder, TRUE, &mm->ex);
-	camel_folder_thaw(folder);
-
+	camel_folder_sync (folder, TRUE, camel_exception_is_set (&mm->ex) ? NULL : &mm->ex);
+	camel_folder_thaw (folder);
+	
 	if (m->destination)
-		camel_folder_thaw(m->destination);
-
+		camel_folder_thaw (m->destination);
+	
 	if (m->cancel)
-		camel_operation_unregister(m->cancel);
+		camel_operation_unregister (m->cancel);
 }
 
 static void
@@ -193,20 +136,25 @@ filter_folder_free(struct _mail_msg *mm)
 {
 	struct _filter_mail_msg *m = (struct _filter_mail_msg *)mm;
 	int i;
-
+	
 	if (m->source_folder)
-		camel_object_unref((CamelObject *)m->source_folder);
+		camel_object_unref (CAMEL_OBJECT (m->source_folder));
+	
 	if (m->source_uids) {
-		for (i=0;i<m->source_uids->len;i++)
-			g_free(m->source_uids->pdata[i]);
-		g_ptr_array_free(m->source_uids, TRUE);
+		for (i = 0; i < m->source_uids->len; i++)
+			g_free (m->source_uids->pdata[i]);
+		
+		g_ptr_array_free (m->source_uids, TRUE);
 	}
+	
 	if (m->cancel)
-		camel_operation_unref(m->cancel);
+		camel_operation_unref (m->cancel);
+	
 	if (m->destination)
-		camel_object_unref((CamelObject *)m->destination);
+		camel_object_unref (CAMEL_OBJECT (m->destination));
+	
 	if (m->driver)
-		camel_object_unref((CamelObject *)m->driver);
+		camel_object_unref (CAMEL_OBJECT (m->driver));
 }
 
 static struct _mail_msg_op filter_folder_op = {
@@ -217,36 +165,32 @@ static struct _mail_msg_op filter_folder_op = {
 };
 
 void
-mail_filter_folder(CamelFolder *source_folder, GPtrArray *uids,
-		   FilterContext *fc, const char *type,
-		   CamelOperation *cancel)
+mail_filter_folder (CamelFolder *source_folder, GPtrArray *uids,
+		    const char *type, CamelOperation *cancel)
 {
 	struct _filter_mail_msg *m;
-
-	m = mail_msg_new(&filter_folder_op, NULL, sizeof(*m));
+	
+	m = mail_msg_new (&filter_folder_op, NULL, sizeof (*m));
 	m->source_folder = source_folder;
-	camel_object_ref((CamelObject *)source_folder);
+	camel_object_ref (CAMEL_OBJECT (source_folder));
 	m->source_uids = uids;
+	m->cache = NULL;
 	m->delete = FALSE;
 	if (cancel) {
 		m->cancel = cancel;
-		camel_operation_ref(cancel);
+		camel_operation_ref (cancel);
 	}
-
-	m->driver = camel_filter_driver_new(filter_get_folder, NULL);
-	setup_filter_driver(m->driver, fc, type);
-
+	
+	m->driver = camel_session_get_filter_driver (session, type, NULL);
+	
 	e_thread_put(mail_thread_new, (EMsg *)m);
 }
 
 /* convenience function for it */
-void mail_filter_on_demand(CamelFolder *folder, GPtrArray *uids)
+void
+mail_filter_on_demand (CamelFolder *folder, GPtrArray *uids)
 {
-	FilterContext *fc;
-
-	fc = mail_load_filter_context();
-	mail_filter_folder(folder, uids, fc, FILTER_SOURCE_INCOMING, NULL);
-	gtk_object_unref((GtkObject *)fc);
+	mail_filter_folder (folder, uids, FILTER_SOURCE_INCOMING, NULL);
 }
 
 /* ********************************************************************** */
@@ -257,52 +201,52 @@ uid_cachename_hack (CamelStore *store)
 {
 	CamelURL *url = CAMEL_SERVICE (store)->url;
 	char *encoded_url, *filename;
-
+	
 	encoded_url = g_strdup_printf ("pop://%s%s%s@%s/", url->user,
 				       url->authmech ? ";auth=" : "",
 				       url->authmech ? url->authmech : "",
 				       url->host);
 	e_filename_make_safe (encoded_url);
-
+	
 	filename = g_strdup_printf ("%s/config/cache-%s", evolution_dir, encoded_url);
 	g_free (encoded_url);
-
+	
 	return filename;
 }
 
 static void
-fetch_mail_fetch(struct _mail_msg *mm)
+fetch_mail_fetch (struct _mail_msg *mm)
 {
 	struct _fetch_mail_msg *m = (struct _fetch_mail_msg *)mm;
 	struct _filter_mail_msg *fm = (struct _filter_mail_msg *)mm;
 	int i;
-
+	
 	if (m->cancel)
-		camel_operation_register(m->cancel);
-
-	if ( (fm->destination = mail_tool_get_local_inbox(&mm->ex)) == NULL) {
+		camel_operation_register (m->cancel);
+	
+	if ((fm->destination = mail_tool_get_local_inbox (&mm->ex)) == NULL) {
 		if (m->cancel)
-			camel_operation_unregister(m->cancel);
+			camel_operation_unregister (m->cancel);
 		return;
 	}
-
+	
 	/* FIXME: this should support keep_on_server too, which would then perform a spool
 	   access thingy, right?  problem is matching raw messages to uid's etc. */
 	if (!strncmp (m->source_uri, "mbox:", 5)) {
 		char *path = mail_tool_do_movemail (m->source_uri, &mm->ex);
 		
 		if (path && !camel_exception_is_set (&mm->ex)) {
-			camel_folder_freeze(fm->destination);
-			camel_filter_driver_set_default_folder(fm->driver, fm->destination);
-			camel_filter_driver_filter_mbox(fm->driver, path, &mm->ex);
-			camel_folder_thaw(fm->destination);
+			camel_folder_freeze (fm->destination);
+			camel_filter_driver_set_default_folder (fm->driver, fm->destination);
+			camel_filter_driver_filter_mbox (fm->driver, path, m->source_uri, &mm->ex);
+			camel_folder_thaw (fm->destination);
 			
 			if (!camel_exception_is_set (&mm->ex))
 				unlink (path);
 		}
 		g_free (path);
 	} else {
-		CamelFolder *folder = fm->source_folder = mail_tool_get_inbox(m->source_uri, &mm->ex);
+		CamelFolder *folder = fm->source_folder = mail_tool_get_inbox (m->source_uri, &mm->ex);
 		
 		if (folder) {
 			/* this handles 'keep on server' stuff, if we have any new uid's to copy
@@ -327,50 +271,66 @@ fetch_mail_fetch(struct _mail_msg *mm)
 						uids->pdata[i] = g_strdup (cache_uids->pdata[i]);
 					camel_uid_cache_free_uids (cache_uids);
 					
+					fm->cache = cache;
 					filter_folder_filter (mm);
 					
-					/* if we are not to delete the messages, save the UID cache */
-					if (!fm->delete && !camel_exception_is_set (&mm->ex))
+					/* if we are not to delete the messages or there was an
+					 * exception, save the UID cache */
+					if (!fm->delete || camel_exception_is_set (&mm->ex))
 						camel_uid_cache_save (cache);
 					
-					camel_uid_cache_destroy (cache);
+					/* if we are deleting off the server an no exception occured
+					 * then iterate through the folder uids and mark them all
+					 * for deletion. */
+					if (fm->delete && !camel_exception_is_set (&mm->ex)) {
+						camel_folder_freeze (folder);
+						
+						for (i = 0; i < folder_uids->len; i++)
+							camel_folder_delete_message (folder, folder_uids->pdata[i]);
+						
+						/* sync and expunge */
+						camel_folder_sync (folder, TRUE, &mm->ex);
+						
+						camel_folder_thaw (folder);
+					}
 				}
+				camel_uid_cache_destroy (cache);
 				camel_folder_free_uids (folder, folder_uids);
 			} else {
 				filter_folder_filter (mm);
 			}
 		}
 	}
-
+	
 	if (m->cancel)
-		camel_operation_unregister(m->cancel);
-
+		camel_operation_unregister (m->cancel);
+	
 	/* we unref this here as it may have more work to do (syncing
 	   folders and whatnot) before we are really done */
 	/* should this be cancellable too? (i.e. above unregister above) */
-	camel_object_unref((CamelObject *)m->fmsg.driver);
+	camel_object_unref ((CamelObject *)m->fmsg.driver);
 	m->fmsg.driver = NULL;
 }
 
 static void
-fetch_mail_fetched(struct _mail_msg *mm)
+fetch_mail_fetched (struct _mail_msg *mm)
 {
 	struct _fetch_mail_msg *m = (struct _fetch_mail_msg *)mm;
-
+	
 	if (m->done)
-		m->done(m->source_uri, m->data);
+		m->done (m->source_uri, m->data);
 }
 
 static void
-fetch_mail_free(struct _mail_msg *mm)
+fetch_mail_free (struct _mail_msg *mm)
 {
 	struct _fetch_mail_msg *m = (struct _fetch_mail_msg *)mm;
-
-	g_free(m->source_uri);
+	
+	g_free (m->source_uri);
 	if (m->cancel)
-		camel_operation_unref(m->cancel);
-
-	filter_folder_free(mm);
+		camel_operation_unref (m->cancel);
+	
+	filter_folder_free (mm);
 }
 
 static struct _mail_msg_op fetch_mail_op = {
@@ -382,31 +342,31 @@ static struct _mail_msg_op fetch_mail_op = {
 
 /* ouch, a 'do everything' interface ... */
 void mail_fetch_mail(const char *source, int keep,
-		     FilterContext *fc, const char *type,
-		     CamelOperation *cancel,
+		     const char *type, CamelOperation *cancel,
 		     CamelFilterGetFolderFunc get_folder, void *get_data,
 		     CamelFilterStatusFunc *status, void *status_data,
 		     void (*done)(char *source, void *data), void *data)
 {
 	struct _fetch_mail_msg *m;
 	struct _filter_mail_msg *fm;
-
-	m = mail_msg_new(&fetch_mail_op, NULL, sizeof(*m));
+	
+	m = mail_msg_new (&fetch_mail_op, NULL, sizeof (*m));
 	fm = (struct _filter_mail_msg *)m;
-	m->source_uri = g_strdup(source);
+	m->source_uri = g_strdup (source);
 	fm->delete = !keep;
+	fm->cache = NULL;
 	if (cancel) {
 		m->cancel = cancel;
-		camel_operation_ref(cancel);
+		camel_operation_ref (cancel);
 	}
 	m->done = done;
 	m->data = data;
-
-	fm->driver = camel_filter_driver_new(get_folder, get_data);
-	setup_filter_driver(fm->driver, fc, type);
+	
+	fm->driver = camel_session_get_filter_driver (session, type, NULL);
+	camel_filter_driver_set_folder_func (fm->driver, get_folder, get_data);
 	if (status)
-		camel_filter_driver_set_status_func(fm->driver, status, status_data);
-
+		camel_filter_driver_set_status_func (fm->driver, status, status_data);
+	
 	e_thread_put(mail_thread_new, (EMsg *)m);
 }
 
@@ -419,30 +379,33 @@ struct _update_info {
 	void *data;
 };
 
-static void do_update_subfolders_rec(CamelStore *store, CamelFolderInfo *info, EvolutionStorage *storage, const char *prefix)
+static void
+do_update_subfolders_rec (CamelStore *store, CamelFolderInfo *info, EvolutionStorage *storage, const char *prefix)
 {
-	char *path, *name;
+	char *path;
 	
+	if (info->url) {
+		/* info->url == URI??? */
+		mail_folder_cache_set_update_estorage (info->url, storage);
+		mail_folder_cache_note_folderinfo (info->url, info);
+	}
+
 	path = g_strdup_printf("%s/%s", prefix, info->name);
-	if (info->unread_message_count > 0)
-		name = g_strdup_printf("%s (%d)", info->name, info->unread_message_count);
-	else
-		name = g_strdup(info->name);
-	
-	evolution_storage_update_folder(storage, path, name, info->unread_message_count > 0);
-	g_free(name);
+
 	if (info->child)
 		do_update_subfolders_rec(store, info->child, storage, path);
 	if (info->sibling)
 		do_update_subfolders_rec(store, info->sibling, storage, prefix);
+
 	g_free(path);
 }
 
-static void do_update_subfolders(CamelStore *store, CamelFolderInfo *info, void *data)
+static void
+do_update_subfolders (CamelStore *store, CamelFolderInfo *info, void *data)
 {
 	struct _update_info *uinfo = data;
 	
-	if (uinfo) {
+	if (uinfo && info) {
 		do_update_subfolders_rec(store, info, uinfo->storage, "");
 	}
 
@@ -454,8 +417,9 @@ static void do_update_subfolders(CamelStore *store, CamelFolderInfo *info, void
 }
 
 /* this interface is a little icky */
-int mail_update_subfolders(CamelStore *store, EvolutionStorage *storage,
-			   void (*done)(CamelStore *, void *data), void *data)
+int
+mail_update_subfolders (CamelStore *store, EvolutionStorage *storage,
+			void (*done)(CamelStore *, void *data), void *data)
 {
 	struct _update_info *info;
 
@@ -484,25 +448,26 @@ mail_send_message(CamelMimeMessage *message, const char *destination, CamelFilte
 	CamelTransport *xport = NULL;
 	CamelFolder *folder;
 	const char *version, *header;
+	gchar *acct_header;
 	char *transport_url = NULL, *sent_folder_uri = NULL;
 	
 	if (SUB_VERSION[0] == '\0')
-		version = "Evolution/" VERSION " (Preview Release)";
+		version = "Evolution/" VERSION " (Beta Release)";
 	else
-		version = "Evolution/" VERSION SUB_VERSION " (Preview Release)";
+		version = "Evolution/" VERSION SUB_VERSION " (Beta Release)";
 	camel_medium_add_header (CAMEL_MEDIUM (message), "X-Mailer", version);
 	camel_mime_message_set_date (message, CAMEL_MESSAGE_DATE_CURRENT, 0);
 	
-	/* Remove the X-Evolution header so we don't send our flags too ;-) */
+	/* Remove the X-Evolution and X-Evolution-Source headers so we don't send our flags & other info too ;-) */
 	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution");
+	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution-Source");
 	
 	/* Get information about the account this was composed by. */
-	header = camel_medium_get_header (CAMEL_MEDIUM (message), "X-Evolution-Account");
-	if (header) {
+	acct_header = g_strdup (camel_medium_get_header (CAMEL_MEDIUM (message), "X-Evolution-Account"));
+	if (acct_header) {
 		const MailConfigAccount *account;
-
-		/* FIXME: this is broken, since we need to be threadsafe */
-		account = mail_config_get_account_by_name (header);
+		
+		account = mail_config_get_account_by_name (acct_header);
 		camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution-Account");
 		if (account) {
 			transport_url = g_strdup (account->transport->url);
@@ -546,20 +511,38 @@ mail_send_message(CamelMimeMessage *message, const char *destination, CamelFilte
 	info = camel_message_info_new ();
 	info->flags = CAMEL_MESSAGE_SEEN;
 	
-	if (driver)
-		camel_filter_driver_filter_message (driver, message, info,
-						    NULL, NULL, "", ex);
+	/* Re-attach account header */
+	if (acct_header) {
+		camel_medium_add_header (CAMEL_MEDIUM (message), "X-Evolution-Account", acct_header);
+		g_free (acct_header);
+	}
 	
-	if (sent_folder_uri)
-		folder = mail_tool_uri_to_folder (sent_folder_uri, ex);
-	else
+	if (driver) {
+		camel_filter_driver_filter_message (driver, message, info,
+						    NULL, NULL, NULL, "", ex);
+		
+		if (camel_exception_is_set (ex)) {
+			camel_message_info_free (info);
+			return;
+		}
+	}
+	
+	if (sent_folder_uri) {
+		folder = mail_tool_uri_to_folder (sent_folder_uri, NULL);
+		if (!folder) {
+			/* FIXME */
+			camel_object_ref (CAMEL_OBJECT (sent_folder));
+			folder = sent_folder;
+		}
+	} else {
+		camel_object_ref (CAMEL_OBJECT (sent_folder));
 		folder = sent_folder;
+	}
 	
 	if (folder) {
 		camel_folder_append_message (folder, message, info, ex);
-		camel_folder_sync (folder, FALSE, ex);
-		if (folder != sent_folder)
-			camel_object_unref (CAMEL_OBJECT (folder));
+		camel_folder_sync (folder, FALSE, NULL);
+		camel_object_unref (CAMEL_OBJECT (folder));
 	}
 	
 	camel_message_info_free (info);
@@ -578,48 +561,52 @@ struct _send_mail_msg {
 	void *data;
 };
 
-static char *send_mail_desc(struct _mail_msg *mm, int done)
+static char *
+send_mail_desc (struct _mail_msg *mm, int done)
 {
 	struct _send_mail_msg *m = (struct _send_mail_msg *)mm;
 	char *subject;
 	const char *subject_utf8;
 	
-	subject_utf8 = camel_mime_message_get_subject(m->message);
+	subject_utf8 = camel_mime_message_get_subject (m->message);
 	
 	if (subject_utf8) {
 		char *desc;
-
+		
 		subject = e_utf8_to_locale_string (subject_utf8);
 		desc = g_strdup_printf (_("Sending \"%s\""), subject);
 		g_free (subject);
 		return desc;
 	} else
-		return g_strdup(_("Sending message"));
+		return g_strdup (_("Sending message"));
 }
 
-static void send_mail_send(struct _mail_msg *mm)
+static void
+send_mail_send (struct _mail_msg *mm)
 {
 	struct _send_mail_msg *m = (struct _send_mail_msg *)mm;
-
-	camel_operation_register(mm->cancel);
-	mail_send_message(m->message, m->destination, m->driver, &mm->ex);
-	camel_operation_unregister(mm->cancel);
+	
+	camel_operation_register (mm->cancel);
+	mail_send_message (m->message, m->destination, m->driver, &mm->ex);
+	camel_operation_unregister (mm->cancel);
 }
 
-static void send_mail_sent(struct _mail_msg *mm)
+static void
+send_mail_sent (struct _mail_msg *mm)
 {
 	struct _send_mail_msg *m = (struct _send_mail_msg *)mm;
-
+	
 	if (m->done)
-		m->done(m->destination, m->message, !camel_exception_is_set(&mm->ex), m->data);
+		m->done (m->destination, m->message, !camel_exception_is_set (&mm->ex), m->data);
 }
 
-static void send_mail_free(struct _mail_msg *mm)
+static void
+send_mail_free (struct _mail_msg *mm)
 {
 	struct _send_mail_msg *m = (struct _send_mail_msg *)mm;
-
-	camel_object_unref((CamelObject *)m->message);
-	g_free(m->destination);
+	
+	camel_object_unref (CAMEL_OBJECT (m->message));
+	g_free (m->destination);
 }
 
 static struct _mail_msg_op send_mail_op = {
@@ -630,27 +617,25 @@ static struct _mail_msg_op send_mail_op = {
 };
 
 int
-mail_send_mail(const char *uri, CamelMimeMessage *message, void (*done) (char *uri, CamelMimeMessage *message, gboolean sent, void *data), void *data)
+mail_send_mail (const char *uri, CamelMimeMessage *message,
+		void (*done) (char *uri, CamelMimeMessage *message, gboolean sent, void *data),
+		void *data)
 {
 	struct _send_mail_msg *m;
 	int id;
-	FilterContext *fc;
-
-	m = mail_msg_new(&send_mail_op, NULL, sizeof(*m));
-	m->destination = g_strdup(uri);
+	
+	m = mail_msg_new (&send_mail_op, NULL, sizeof (*m));
+	m->destination = g_strdup (uri);
 	m->message = message;
-	camel_object_ref((CamelObject *)message);
+	camel_object_ref (CAMEL_OBJECT (message));
 	m->data = data;
 	m->done = done;
-
+	
 	id = m->msg.seq;
-
-	m->driver = camel_filter_driver_new(filter_get_folder, NULL);
-	fc = mail_load_filter_context();
-	setup_filter_driver(m->driver, fc, FILTER_SOURCE_OUTGOING);
-	gtk_object_unref((GtkObject *)fc);
-
-	e_thread_put(mail_thread_new, (EMsg *)m);
+	
+	m->driver = camel_session_get_filter_driver (session, FILTER_SOURCE_OUTGOING, NULL);
+	
+	e_thread_put (mail_thread_new, (EMsg *)m);
 	return id;
 }
 
@@ -709,7 +694,7 @@ send_queue_send(struct _mail_msg *mm)
 		CamelMessageInfo *info;
 		int pc = (100 * i) / uids->len;
 		
-		report_status (m, CAMEL_FILTER_STATUS_START, pc, "Sending message %d of %d", i+1, uids->len);
+		report_status (m, CAMEL_FILTER_STATUS_START, pc, _("Sending message %d of %d"), i+1, uids->len);
 		
 		info = camel_folder_get_message_info (m->queue, uids->pdata[i]);
 		if (info && info->flags & CAMEL_MESSAGE_DELETED)
@@ -728,9 +713,9 @@ send_queue_send(struct _mail_msg *mm)
 	}
 	
 	if (camel_exception_is_set (&mm->ex))
-		report_status (m, CAMEL_FILTER_STATUS_END, 100, "Failed on message %d of %d", i+1, uids->len);
+		report_status (m, CAMEL_FILTER_STATUS_END, 100, _("Failed on message %d of %d"), i+1, uids->len);
 	else
-		report_status (m, CAMEL_FILTER_STATUS_END, 100, "Complete.");
+		report_status (m, CAMEL_FILTER_STATUS_END, 100, _("Complete."));
 	
 	camel_folder_free_uids (m->queue, uids);
 	
@@ -738,7 +723,7 @@ send_queue_send(struct _mail_msg *mm)
 		camel_folder_expunge (m->queue, &mm->ex);
 	
 	if (sent_folder)
-		camel_folder_sync (sent_folder, FALSE, &mm->ex);
+		camel_folder_sync (sent_folder, FALSE, NULL);
 	
 	if (m->cancel)
 		camel_operation_unregister (m->cancel);
@@ -774,8 +759,7 @@ static struct _mail_msg_op send_queue_op = {
 /* same interface as fetch_mail, just 'cause i'm lazy today (and we need to run it from the same spot?) */
 void
 mail_send_queue(CamelFolder *queue, const char *destination,
-		FilterContext *fc, const char *type,
-		CamelOperation *cancel,
+		const char *type, CamelOperation *cancel,
 		CamelFilterGetFolderFunc get_folder, void *get_data,
 		CamelFilterStatusFunc *status, void *status_data,
 		void (*done)(char *destination, void *data), void *data)
@@ -795,8 +779,8 @@ mail_send_queue(CamelFolder *queue, const char *destination,
 	m->done = done;
 	m->data = data;
 
-	m->driver = camel_filter_driver_new(get_folder, get_data);
-	setup_filter_driver(m->driver, fc, type);
+	m->driver = camel_session_get_filter_driver (session, type, NULL);
+	camel_filter_driver_set_folder_func (m->driver, get_folder, get_data);
 
 	e_thread_put(mail_thread_new, (EMsg *)m);
 }
@@ -814,12 +798,14 @@ struct _append_msg {
 	void *data;
 };
 
-static char *append_mail_desc(struct _mail_msg *mm, int done)
+static char *
+append_mail_desc (struct _mail_msg *mm, int done)
 {
-	return g_strdup(_("Saving message to folder"));
+	return g_strdup (_("Saving message to folder"));
 }
 
-static void append_mail_append(struct _mail_msg *mm)
+static void
+append_mail_append (struct _mail_msg *mm)
 {
 	struct _append_msg *m = (struct _append_msg *)mm;
 
@@ -827,7 +813,8 @@ static void append_mail_append(struct _mail_msg *mm)
 	camel_folder_append_message(m->folder, m->message, m->info, &mm->ex);
 }
 
-static void append_mail_appended(struct _mail_msg *mm)
+static void
+append_mail_appended (struct _mail_msg *mm)
 {
 	struct _append_msg *m = (struct _append_msg *)mm;
 
@@ -835,7 +822,8 @@ static void append_mail_appended(struct _mail_msg *mm)
 		m->done(m->folder, m->message, m->info, !camel_exception_is_set(&mm->ex), m->data);
 }
 
-static void append_mail_free(struct _mail_msg *mm)
+static void
+append_mail_free (struct _mail_msg *mm)
 {
 	struct _append_msg *m = (struct _append_msg *)mm;
 
@@ -851,27 +839,26 @@ static struct _mail_msg_op append_mail_op = {
 };
 
 void
-mail_append_mail (CamelFolder *folder,
-		  CamelMimeMessage *message,
-		  CamelMessageInfo *info,
-		  void (*done)(CamelFolder *folder, CamelMimeMessage *msg, CamelMessageInfo *info, int ok, void *data), void *data)
+mail_append_mail (CamelFolder *folder, CamelMimeMessage *message, CamelMessageInfo *info,
+		  void (*done)(CamelFolder *folder, CamelMimeMessage *msg, CamelMessageInfo *info, int ok, void *data),
+		  void *data)
 {
 	struct _append_msg *m;
-
+	
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 	g_return_if_fail (CAMEL_IS_MIME_MESSAGE (message));
-
-	m = mail_msg_new(&append_mail_op, NULL, sizeof(*m));
+	
+	m = mail_msg_new (&append_mail_op, NULL, sizeof (*m));
 	m->folder = folder;
-	camel_object_ref((CamelObject *)folder);
+	camel_object_ref (CAMEL_OBJECT (folder));
 	m->message = message;
-	camel_object_ref((CamelObject *)message);
+	camel_object_ref (CAMEL_OBJECT (message));
 	m->info = info;
-
+	
 	m->done = done;
 	m->data = data;
-
-	e_thread_put(mail_thread_new, (EMsg *)m);
+	
+	e_thread_put (mail_thread_new, (EMsg *)m);
 }
 
 /* ** TRANSFER MESSAGES **************************************************** */
@@ -883,9 +870,13 @@ struct _transfer_msg {
 	GPtrArray *uids;
 	gboolean delete;
 	char *dest_uri;
+	
+	void (*done)(gboolean ok, void *data);
+	void *data;
 };
 
-static char *transfer_messages_desc(struct _mail_msg *mm, int done)
+static char *
+transfer_messages_desc (struct _mail_msg *mm, int done)
 {
 	struct _transfer_msg *m = (struct _transfer_msg *)mm;
 
@@ -894,7 +885,8 @@ static char *transfer_messages_desc(struct _mail_msg *mm, int done)
 			       
 }
 
-static void transfer_messages_transfer(struct _mail_msg *mm)
+static void
+transfer_messages_transfer (struct _mail_msg *mm)
 {
 	struct _transfer_msg *m = (struct _transfer_msg *)mm;
 	CamelFolder *dest;
@@ -910,59 +902,84 @@ static void transfer_messages_transfer(struct _mail_msg *mm)
 		func = camel_folder_copy_messages_to;
 		desc = _("Copying");
 	}
-
+	
 	dest = mail_tool_uri_to_folder (m->dest_uri, &mm->ex);
 	if (camel_exception_is_set (&mm->ex))
 		return;
-
+	
 	camel_folder_freeze (m->source);
 	camel_folder_freeze (dest);
 	
-	(func) (m->source, m->uids, dest, &mm->ex);
+	if (CAMEL_IS_VTRASH_FOLDER (dest)) {
+		if (m->delete) {
+			int i;
+			
+			for (i = 0; i < m->uids->len; i++)
+				camel_folder_delete_message (m->source, m->uids->pdata[i]);
+		} else {
+			/* no-op - can't copy messages to*/
+		}
+	} else
+		(func) (m->source, m->uids, dest, &mm->ex);
 	
-	camel_folder_thaw(m->source);
-	camel_folder_thaw(dest);
-	camel_object_unref((CamelObject *)dest);
+	camel_folder_thaw (m->source);
+	camel_folder_thaw (dest);
+	camel_folder_sync (dest, FALSE, NULL);
+	camel_object_unref (CAMEL_OBJECT (dest));
 }
 
-static void transfer_messages_free(struct _mail_msg *mm)
+static void
+transfer_messages_transferred (struct _mail_msg *mm)
+{
+	struct _transfer_msg *m = (struct _transfer_msg *)mm;
+	
+	if (m->done)
+		m->done (!camel_exception_is_set (&mm->ex), m->data);
+}
+
+static void
+transfer_messages_free (struct _mail_msg *mm)
 {
 	struct _transfer_msg *m = (struct _transfer_msg *)mm;
 	int i;
 
-	camel_object_unref((CamelObject *)m->source);
-	g_free(m->dest_uri);
-	for (i=0;i<m->uids->len;i++)
-		g_free(m->uids->pdata[i]);
-	g_ptr_array_free(m->uids, TRUE);
+	camel_object_unref (CAMEL_OBJECT (m->source));
+	g_free (m->dest_uri);
+	for (i = 0; i < m->uids->len; i++)
+		g_free (m->uids->pdata[i]);
+	g_ptr_array_free (m->uids, TRUE);
 
 }
 
 static struct _mail_msg_op transfer_messages_op = {
 	transfer_messages_desc,
 	transfer_messages_transfer,
-	NULL,
+	transfer_messages_transferred,
 	transfer_messages_free,
 };
 
 void
-mail_do_transfer_messages (CamelFolder *source, GPtrArray *uids,
-			   gboolean delete_from_source,
-			   gchar *dest_uri)
+mail_transfer_messages (CamelFolder *source, GPtrArray *uids,
+			gboolean delete_from_source,
+			const char *dest_uri,
+			void (*done) (gboolean ok, void *data),
+			void *data)
 {
 	struct _transfer_msg *m;
-
+	
 	g_return_if_fail (CAMEL_IS_FOLDER (source));
 	g_return_if_fail (uids != NULL);
 	g_return_if_fail (dest_uri != NULL);
-
+	
 	m = mail_msg_new(&transfer_messages_op, NULL, sizeof(*m));
 	m->source = source;
-	camel_object_ref((CamelObject *)source);
+	camel_object_ref (CAMEL_OBJECT (source));
 	m->uids = uids;
 	m->delete = delete_from_source;
 	m->dest_uri = g_strdup (dest_uri);
-
+	m->done = done;
+	m->data = data;
+	
 	e_thread_put(mail_thread_queued, (EMsg *)m);
 }
 
@@ -977,7 +994,8 @@ struct _get_folderinfo_msg {
 	void *data;
 };
 
-static char *get_folderinfo_desc(struct _mail_msg *mm, int done)
+static char *
+get_folderinfo_desc (struct _mail_msg *mm, int done)
 {
 	struct _get_folderinfo_msg *m = (struct _get_folderinfo_msg *)mm;
 	char *ret, *name;
@@ -998,14 +1016,14 @@ add_vtrash_info (CamelFolderInfo *info)
 	g_return_if_fail (info != NULL);
 	
 	for (fi = info; fi->sibling; fi = fi->sibling) {
-		if (!strcmp (fi->name, _("Trash")))
+		if (!strcmp (fi->name, CAMEL_VTRASH_NAME))
 			break;
 	}
 	
 	/* create our vTrash URL */
 	url = camel_url_new (info->url, NULL);
 	g_free (url->path);
-	url->path = g_strdup_printf ("/%s", _("Trash"));
+	url->path = g_strdup_printf ("/%s", CAMEL_VTRASH_NAME);
 	uri = camel_url_to_string (url, 0);
 	camel_url_free (url);
 	
@@ -1030,26 +1048,40 @@ add_vtrash_info (CamelFolderInfo *info)
 	g_free (uri);
 }
 
-static void get_folderinfo_get(struct _mail_msg *mm)
+static void
+get_folderinfo_get (struct _mail_msg *mm)
 {
 	struct _get_folderinfo_msg *m = (struct _get_folderinfo_msg *)mm;
-	
+	guint32 flags = CAMEL_STORE_FOLDER_INFO_RECURSIVE;
+
+	if (camel_store_supports_subscriptions (m->store))
+		flags |= CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
+
 	camel_operation_register(mm->cancel);
-	m->info = camel_store_get_folder_info(m->store, NULL, FALSE, TRUE, camel_store_supports_subscriptions(m->store), &mm->ex);
+	m->info = camel_store_get_folder_info (m->store, NULL, flags, &mm->ex);
 	if (m->info && m->info->url)
 		add_vtrash_info (m->info);
 	camel_operation_unregister(mm->cancel);
 }
 
-static void get_folderinfo_got(struct _mail_msg *mm)
+static void
+get_folderinfo_got (struct _mail_msg *mm)
 {
 	struct _get_folderinfo_msg *m = (struct _get_folderinfo_msg *)mm;
 
+	if (camel_exception_is_set (&(mm->ex)))
+		g_warning ("Error getting folder info from store at %s: %s",
+			   camel_service_get_url (CAMEL_SERVICE (m->store)),
+			   camel_exception_get_description (&(mm->ex)));
+
+	/* 'done' is probably guaranteed to fail, but... */
+
 	if (m->done)
 		m->done(m->store, m->info, m->data);
 }
 
-static void get_folderinfo_free(struct _mail_msg *mm)
+static void
+get_folderinfo_free (struct _mail_msg *mm)
 {
 	struct _get_folderinfo_msg *m = (struct _get_folderinfo_msg *)mm;
 
@@ -1065,7 +1097,8 @@ static struct _mail_msg_op get_folderinfo_op = {
 	get_folderinfo_free,
 };
 
-int mail_get_folderinfo(CamelStore *store, void (*done)(CamelStore *store, CamelFolderInfo *info, void *data), void *data)
+int
+mail_get_folderinfo (CamelStore *store, void (*done)(CamelStore *store, CamelFolderInfo *info, void *data), void *data)
 {
 	struct _get_folderinfo_msg *m;
 	int id;
@@ -1372,6 +1405,90 @@ mail_create_folder(const char *uri, void (*done) (char *uri, CamelFolder *folder
 	e_thread_put(mail_thread_new, (EMsg *)m);
 }
 
+/* ** REMOVE FOLDER ******************************************************* */
+
+struct _remove_folder_msg {
+	struct _mail_msg msg;
+
+	char *uri;
+	gboolean removed;
+	void (*done) (char *uri, gboolean removed, void *data);
+	void *data;
+};
+
+static char *
+remove_folder_desc (struct _mail_msg *mm, int done)
+{
+	struct _remove_folder_msg *m = (struct _remove_folder_msg *)mm;
+	
+	return g_strdup_printf (_("Removing folder %s"), m->uri);
+}
+
+static void
+remove_folder_get (struct _mail_msg *mm)
+{
+	struct _remove_folder_msg *m = (struct _remove_folder_msg *)mm;
+	CamelStore *store;
+	CamelFolder *folder;
+	
+	m->removed = FALSE;
+	
+	camel_operation_register (mm->cancel);
+
+	folder = mail_tool_uri_to_folder (m->uri, &mm->ex);
+
+	store = camel_folder_get_parent_store (folder);
+	if (!store)
+		goto done;
+
+	camel_store_delete_folder (store, camel_folder_get_full_name (folder), &mm->ex);
+	m->removed = !camel_exception_is_set (&mm->ex);
+	camel_object_unref (CAMEL_OBJECT (store));
+	
+ done:
+	if (store)
+		camel_object_unref (CAMEL_OBJECT (store));
+
+	camel_operation_unregister (mm->cancel);
+}
+
+static void
+remove_folder_got (struct _mail_msg *mm)
+{
+	struct _remove_folder_msg *m = (struct _remove_folder_msg *)mm;
+	
+	if (m->done)
+		m->done (m->uri, m->removed, m->data);
+}
+
+static void
+remove_folder_free (struct _mail_msg *mm)
+{
+	struct _remove_folder_msg *m = (struct _remove_folder_msg *)mm;
+	
+	g_free (m->uri);
+}
+
+static struct _mail_msg_op remove_folder_op = {
+	remove_folder_desc,
+	remove_folder_get,
+	remove_folder_got,
+	remove_folder_free,
+};
+
+void
+mail_remove_folder (const char *uri, void (*done) (char *uri, gboolean removed, void *data), void *data)
+{
+	struct _remove_folder_msg *m;
+	
+	m = mail_msg_new (&remove_folder_op, NULL, sizeof (*m));
+	m->uri = g_strdup (uri);
+	m->data = data;
+	m->done = done;
+	
+	e_thread_put (mail_thread_new, (EMsg *)m);
+}
+
 /* ** SYNC FOLDER ********************************************************* */
 
 struct _sync_folder_msg {
@@ -1434,6 +1551,42 @@ mail_sync_folder(CamelFolder *folder, void (*done) (CamelFolder *folder, void *d
 
 /* ******************************************************************************** */
 
+static char *refresh_folder_desc(struct _mail_msg *mm, int done)
+{
+	return g_strdup(_("Refreshing folder"));
+}
+
+static void refresh_folder_refresh(struct _mail_msg *mm)
+{
+	struct _sync_folder_msg *m = (struct _sync_folder_msg *)mm;
+
+	camel_folder_refresh_info(m->folder, &mm->ex);
+}
+
+/* we just use the sync stuff where we can, since it would be the same */
+static struct _mail_msg_op refresh_folder_op = {
+	refresh_folder_desc,
+	refresh_folder_refresh,
+	sync_folder_synced,
+	sync_folder_free,
+};
+
+void
+mail_refresh_folder(CamelFolder *folder, void (*done) (CamelFolder *folder, void *data), void *data)
+{
+	struct _sync_folder_msg *m;
+
+	m = mail_msg_new(&refresh_folder_op, NULL, sizeof(*m));
+	m->folder = folder;
+	camel_object_ref((CamelObject *)folder);
+	m->data = data;
+	m->done = done;
+
+	e_thread_put(mail_thread_new, (EMsg *)m);
+}
+
+/* ******************************************************************************** */
+
 static char *expunge_folder_desc(struct _mail_msg *mm, int done)
 {
 	return g_strdup(_("Expunging folder"));
@@ -1907,3 +2060,99 @@ mail_save_part(CamelMimePart *part, const char *path,
 
 	return id;
 }
+
+
+/* ** GO OFFLINE ***************************************************** */
+
+struct _set_offline_msg {
+	struct _mail_msg msg;
+
+	CamelStore *store;
+	gboolean offline;
+	void (*done)(CamelStore *store, void *data);
+	void *data;
+};
+
+static char *set_offline_desc(struct _mail_msg *mm, int done)
+{
+	struct _set_offline_msg *m = (struct _set_offline_msg *)mm;
+	char *service_name = camel_service_get_name (CAMEL_SERVICE (m->store), TRUE);
+	char *msg;
+
+	msg = g_strdup_printf (_("Disconnecting from %s"), service_name);
+	g_free (service_name);
+	return msg;
+}
+
+static void set_offline_do(struct _mail_msg *mm)
+{
+	struct _set_offline_msg *m = (struct _set_offline_msg *)mm;
+
+	if (CAMEL_IS_DISCO_STORE (m->store) &&
+	    camel_disco_store_can_work_offline (CAMEL_DISCO_STORE (m->store))) {
+		if (m->offline) {
+			CamelFolder *inbox;
+
+			/* FIXME. Something more generic here... */
+			inbox = camel_store_get_inbox (m->store, NULL);
+			if (inbox) {
+				camel_disco_folder_prepare_for_offline (
+					CAMEL_DISCO_FOLDER (inbox),
+					"(match-all (not (system-flag \"Seen\")))",
+					&mm->ex);
+				camel_folder_sync (inbox, FALSE, NULL);
+				camel_object_unref (CAMEL_OBJECT (inbox));
+				if (camel_exception_is_set (&mm->ex))
+					return;
+			}
+		}
+
+		camel_disco_store_set_status (CAMEL_DISCO_STORE (m->store),
+					      m->offline ? CAMEL_DISCO_STORE_OFFLINE :
+					      CAMEL_DISCO_STORE_ONLINE, &mm->ex);
+	} else {
+		if (m->offline) {
+			camel_service_disconnect (CAMEL_SERVICE (m->store),
+						  TRUE, &mm->ex);
+		}
+	}
+}
+
+static void set_offline_done(struct _mail_msg *mm)
+{
+	struct _set_offline_msg *m = (struct _set_offline_msg *)mm;
+
+	if (m->done)
+		m->done(m->store, m->data);
+}
+
+static void set_offline_free(struct _mail_msg *mm)
+{
+	struct _set_offline_msg *m = (struct _set_offline_msg *)mm;
+
+	camel_object_unref((CamelObject *)m->store);
+}
+
+static struct _mail_msg_op set_offline_op = {
+	set_offline_desc,
+	set_offline_do,
+	set_offline_done,
+	set_offline_free,
+};
+
+void
+mail_store_set_offline (CamelStore *store, gboolean offline,
+			void (*done)(CamelStore *, void *data),
+			void *data)
+{
+	struct _set_offline_msg *m;
+
+	m = mail_msg_new(&set_offline_op, NULL, sizeof(*m));
+	m->store = store;
+	camel_object_ref((CamelObject *)store);
+	m->offline = offline;
+	m->data = data;
+	m->done = done;
+
+	e_thread_put(mail_thread_queued, (EMsg *)m);
+}
