@@ -3,11 +3,11 @@
 
 /* 
  * Authors: 
- *  Dan Winship <danw@helixcode.com>
- *  Peter Williams <peterw@helixcode.com>
- *  Jeffrey Stedfast <fejj@helixcode.com>
+ *  Dan Winship <danw@ximian.com>
+ *  Peter Williams <peterw@ximian.com>
+ *  Jeffrey Stedfast <fejj@ximian.com>
  *
- * Copyright 2000 Helix Code, Inc. (http://www.helixcode.com)
+ * Copyright 2000 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,7 +25,12 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include <unistd.h>
+#include <pthread.h>
 #include <ctype.h>
 #include <errno.h>
 #include "camel/camel.h"
@@ -38,6 +43,8 @@
 #include "mail.h" /*session*/
 #include "mail-tools.h"
 #include "mail-local.h"
+#include "mail-mt.h"
+#include "mail-folder-cache.h"
 #include "e-util/e-html-utils.h"
 
 /* **************************************** */
@@ -59,29 +66,43 @@ mail_tool_get_folder_from_urlname (const gchar *url, const gchar *name,
 	return folder;
 }
 
-const gchar *
+char *
 mail_tool_get_folder_name (CamelFolder *folder)
 {
 	const char *name = camel_folder_get_full_name (folder);
-	char *path;
-
+	char *path, *pend;
+	
 	/* This is a kludge. */
-
-	if (strcmp (name, "//mbox") && strcmp (name, "//mh"))
-		return name;
-
+	if (strcmp (name, "mbox") && strcmp (name, "mh") && strcmp (name, "maildir"))
+		return g_strdup (name);
+	
 	/* For mbox/mh, return the parent store's final path component. */
-	path = CAMEL_SERVICE (folder->parent_store)->url->path;
-	if (strchr (path, '/'))
-		return strrchr (path, '/') + 1;
-	else
-		return path;
+	path = g_strdup (CAMEL_SERVICE (folder->parent_store)->url->path);
+	pend = path + strlen (path) - 1;
+	if (*pend == '/')
+		*pend = '\0';
+	
+	pend = path;
+	path = g_strdup (g_basename (path));
+	g_free (pend);
+	
+	return path;
 }
 
 gchar *
 mail_tool_get_local_movemail_path (void)
 {
-	return g_strdup_printf ("%s/local/Inbox/movemail", evolution_dir);
+	static gint count = 0;
+	static pthread_mutex_t movemail_path_lock = PTHREAD_MUTEX_INITIALIZER;
+	gint my_count;
+
+	/* Ah, the joys of being multi-threaded... */
+	pthread_mutex_lock (&movemail_path_lock);
+	my_count = count;
+	++count;
+	pthread_mutex_unlock (&movemail_path_lock);
+
+	return g_strdup_printf ("%s/local/Inbox/movemail.%d", evolution_dir, my_count);
 }
 
 CamelFolder *
@@ -155,6 +176,7 @@ mail_tool_do_movemail (const gchar *source_url, CamelException *ex)
 	camel_movemail (source, dest_path, ex);
 
 	if (stat (dest_path, &sb) < 0 || sb.st_size == 0) {
+		unlink (dest_path); /* Clean up the movemail.foo file. */
 		g_free (dest_path);
 		return NULL;
 	}
@@ -231,16 +253,23 @@ mail_tool_uri_to_folder (const char *uri, CamelException *ex)
 	CamelStore *store = NULL;
 	CamelFolder *folder = NULL;
 	int offset = 0;
-	
+
 	g_return_val_if_fail (uri != NULL, NULL);
-	
+
+	folder = mail_folder_cache_try_folder (uri);
+	if (folder) {
+		camel_object_ref (CAMEL_OBJECT (folder));
+		return folder;
+	}
+
 	if (!strncmp (uri, "vtrash:", 7))
 		offset = 7;
-	
+
 	url = camel_url_new (uri + offset, ex);
-	if (!url)
+	if (!url) {
 		return NULL;
-	
+	}
+
 	if (!strcmp (url->protocol, "vfolder")) {
 		folder = vfolder_uri_to_folder (uri, ex);
 	} else {
@@ -248,16 +277,23 @@ mail_tool_uri_to_folder (const char *uri, CamelException *ex)
 		if (store) {
 			char *name;
 			
-			if (url->path && *url->path)
-				name = url->path + 1;
-			else
-				name = "";
+			/* if we have a fragment, then the path is actually used by the store,
+			   so the fragment is the path to the folder instead */
+			if (url->fragment) {
+				name = url->fragment;
+			} else {
+				if (url->path && *url->path)
+					name = url->path + 1;
+				else
+					name = "";
+			}
 
 			if (offset)
 				folder = camel_store_get_trash (store, ex);
 			else
 				folder = camel_store_get_folder (store, name,
 								 CAMEL_STORE_FOLDER_CREATE, ex);
+			camel_object_unref (CAMEL_OBJECT (store));
 		}
 	}
 
@@ -266,9 +302,9 @@ mail_tool_uri_to_folder (const char *uri, CamelException *ex)
 			camel_object_unref (CAMEL_OBJECT (folder));
 			folder = NULL;
 		}
-	}
-	if (store)
-		camel_object_unref (CAMEL_OBJECT (store));
+	} else
+		mail_folder_cache_note_folder (uri, folder);
+
 	camel_url_free (url);
 
 	return folder;
@@ -307,15 +343,15 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 		}
 		
 		if (is_html) {
-			ret_text = g_strdup_printf ("<blockquote><i><font color=\"%06x\">\n%s%s%s\n"
-						    "</font></i></blockquote>\n",
-						    mail_config_get_citation_color (),
+			ret_text = g_strdup_printf ("%s<!--+GtkHTML:<DATA class=\"ClueFlow\" key=\"orig\" value=\"1\">-->"
+						    "<blockquote><i><font color=\"%06x\">\n%s\n"
+						    "</font></i></blockquote>"
+						    "<!--+GtkHTML:<DATA class=\"ClueFlow\" clear=\"orig\">-->",
 						    credits ? credits : "",
-						    credits ? "\n" : "",
-						    text);
+						    mail_config_get_citation_color (), text);
 		} else {
-			gchar *s, *d, *quoted_text;
-			gint lines, len, offset = 0;
+			gchar *s, *d, *quoted_text, *orig_text;
+			gint lines, len;
 			
 			/* Count the number of lines in the body. If
 			 * the text ends with a \n, this will be one
@@ -325,21 +361,15 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 			for (s = text, lines = 0; s; s = strchr (s + 1, '\n'))
 				lines++;
 			
-			offset = credits ? strlen (credits) : 0;
-
 			/* offset is the size of the credits, strlen (text)
 			 * covers the body, lines * 2 does the "> "s, and
 			 * the last +2 covers the final "\0", plus an extra
 			 * "\n" in case text doesn't end with one.
 			 */
-			quoted_text = g_malloc (offset + strlen (text) +
-						lines * 2 + 2);
-			
-			if (credits)
-				memcpy (quoted_text, credits, offset);
+			quoted_text = g_malloc (strlen (text) + lines * 2 + 2);
 			
 			s = text;
-			d = quoted_text + offset;
+			d = quoted_text;
 			
 			/* Copy text to quoted_text line by line,
 			 * prepending "> ".
@@ -357,14 +387,21 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 			*d = '\0';
 			
 			/* Now convert that to HTML. */
-			ret_text = e_text_to_html_full (quoted_text, E_TEXT_TO_HTML_PRE
-							| (mail_config_get_citation_highlight ()
-							   ? E_TEXT_TO_HTML_MARK_CITATION : 0),
-							mail_config_get_citation_color ());
+			orig_text = e_text_to_html_full (quoted_text, E_TEXT_TO_HTML_PRE
+							 | (mail_config_get_citation_highlight ()
+							    ? E_TEXT_TO_HTML_MARK_CITATION : 0),
+							 mail_config_get_citation_color ());
 			g_free (quoted_text);
+			ret_text = g_strdup_printf ("%s<!--+GtkHTML:<DATA class=\"ClueFlow\" key=\"orig\" value=\"1\">-->"
+						    "%s"
+						    "<!--+GtkHTML:<DATA class=\"ClueFlow\" clear=\"orig\">-->",
+						    credits ? credits : "",
+						    orig_text);
+			g_free (orig_text);
 		}
 		
 		g_free (text);
+		printf ("ret: %s\n", ret_text);
 		return ret_text;
 	}
 	
@@ -392,27 +429,36 @@ mail_tool_forward_message (CamelMimeMessage *message)
 	if (text) {
 		gchar *ret_text, *credits = NULL;
 		const CamelInternetAddress *cia;
-		const char *subject;
-		char *buf, *from, *to;
+		char *buf, *from, *to, *subject;
 		
 		/* create credits */
 		cia = camel_mime_message_get_from (message);
 		buf = camel_address_format (CAMEL_ADDRESS (cia));
-		from = e_text_to_html (buf, E_TEXT_TO_HTML_CONVERT_NL | E_TEXT_TO_HTML_CONVERT_URLS);
-		g_free (buf);
+		if (buf) {
+			from = e_text_to_html (buf, E_TEXT_TO_HTML_CONVERT_NL | E_TEXT_TO_HTML_CONVERT_URLS);
+			g_free (buf);
+		} else
+			from = NULL;
 		
 		cia = camel_mime_message_get_recipients (message, CAMEL_RECIPIENT_TYPE_TO);
 		buf = camel_address_format (CAMEL_ADDRESS (cia));
-		to = e_text_to_html (buf, E_TEXT_TO_HTML_CONVERT_NL | E_TEXT_TO_HTML_CONVERT_URLS);
-		g_free (buf);
+		if (buf) {
+			to = e_text_to_html (buf, E_TEXT_TO_HTML_CONVERT_NL | E_TEXT_TO_HTML_CONVERT_URLS);
+			g_free (buf);
+		} else
+			to = NULL;
 		
-		subject = camel_mime_message_get_subject (message);
+		buf = (char *) camel_mime_message_get_subject (message);
+		if (buf)
+			subject = e_text_to_html (buf, E_TEXT_TO_HTML_CONVERT_NL | E_TEXT_TO_HTML_CONVERT_URLS);
+		else
+			subject = "";
 		
 		credits = g_strdup_printf (_("-----Forwarded Message-----<br>"
 					     "<b>From:</b> %s<br>"
 					     "<b>To:</b> %s<br>"
 					     "<b>Subject:</b> %s<br>"),
-					   from, to, subject);
+					   from ? from : "", to ? to : "", subject);
 		g_free (from);
 		g_free (to);
 		
