@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-component-registry.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,19 +25,18 @@
 #include <config.h>
 #endif
 
-#include <gtk/gtkobject.h>
+#include "e-component-registry.h"
+
+#include <glib.h>
 #include <gtk/gtktypeutils.h>
-#include <bonobo.h>
 
 #include <gal/util/e-util.h>
-#include "e-shell-utils.h"
 
 #include "Evolution.h"
 
+#include "e-shell-utils.h"
 #include "evolution-shell-component-client.h"
 
-#include "e-component-registry.h"
-
 
 #define PARENT_TYPE GTK_TYPE_OBJECT
 static GtkObjectClass *parent_class = NULL;
@@ -143,6 +142,7 @@ register_component (EComponentRegistry *component_registry,
 	GNOME_Evolution_ShellComponent component_corba_interface;
 	GNOME_Evolution_Shell shell_corba_interface;
 	GNOME_Evolution_FolderTypeList *supported_types;
+	GNOME_Evolution_URISchemaList *supported_schemas;
 	Component *component;
 	EvolutionShellComponentClient *client;
 	CORBA_Environment ev;
@@ -167,6 +167,8 @@ register_component (EComponentRegistry *component_registry,
 	component_corba_interface = bonobo_object_corba_objref (BONOBO_OBJECT (client));
 	shell_corba_interface = bonobo_object_corba_objref (BONOBO_OBJECT (priv->shell));
 
+	/* Register the supported folder types.  */
+
 	supported_types = GNOME_Evolution_ShellComponent__get_supported_types (component_corba_interface, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION || supported_types->_length == 0) {
 		bonobo_object_unref (BONOBO_OBJECT (client));
@@ -178,6 +180,7 @@ register_component (EComponentRegistry *component_registry,
 
 	component = component_new (id, client);
 	g_hash_table_insert (priv->component_id_to_component, component->id, component);
+	bonobo_object_unref (BONOBO_OBJECT (client));
 
 	for (i = 0; i < supported_types->_length; i++) {
 		const GNOME_Evolution_FolderType *type;
@@ -201,6 +204,27 @@ register_component (EComponentRegistry *component_registry,
 
 	CORBA_free (supported_types);
 
+	/* Register the supported external URI schemas.  */
+
+	supported_schemas = GNOME_Evolution_ShellComponent__get_external_uri_schemas (component_corba_interface, &ev);
+	if (ev._major == CORBA_NO_EXCEPTION) {
+		EUriSchemaRegistry *uri_schema_registry;
+
+		uri_schema_registry = e_shell_get_uri_schema_registry (priv->shell); 
+
+		for (i = 0; i < supported_schemas->_length; i++) {
+			const CORBA_char *schema;
+
+			schema = supported_schemas->_buffer[i];
+			if (! e_uri_schema_registry_set_handler_for_schema (uri_schema_registry, schema, component->client))
+				g_warning ("Cannot register schema `%s' for component %s", schema, component->id);
+			else
+				g_print ("Registered handler for schema `%s' -- %s\n", schema, component->id);
+		}
+
+		CORBA_free (supported_schemas);
+	}
+
 	return TRUE;
 }
 
@@ -227,8 +251,6 @@ destroy (GtkObject *object)
 	component_registry = E_COMPONENT_REGISTRY (object);
 	priv = component_registry->priv;
 
-	bonobo_object_unref (BONOBO_OBJECT (priv->shell));
-
 	g_hash_table_foreach (priv->component_id_to_component, component_id_foreach_free, NULL);
 	g_hash_table_destroy (priv->component_id_to_component);
 
@@ -273,8 +295,6 @@ e_component_registry_construct (EComponentRegistry *component_registry,
 	g_return_if_fail (E_IS_SHELL (shell));
 
 	priv = component_registry->priv;
-
-	bonobo_object_ref (BONOBO_OBJECT (shell));
 	priv->shell = shell;
 }
 
