@@ -25,14 +25,14 @@
 #include <config.h>
 #endif
 
-#include <bonobo.h>
+#include "e-corba-storage.h"
 
+#include <glib.h>
+#include <bonobo/bonobo-main.h>
 #include <gal/util/e-util.h>
 
 #include "Evolution.h"
 
-#include "e-corba-storage.h"
-
 
 #define PARENT_TYPE E_TYPE_STORAGE
 static EStorageClass *parent_class = NULL;
@@ -393,7 +393,8 @@ async_remove_folder (EStorage *storage, const char *path,
 
 	CORBA_exception_init (&ev);
 	GNOME_Evolution_Storage_asyncRemoveFolder (priv->storage_interface,
-						   path, e_folder_get_physical_uri (folder),
+						   path,
+						   e_folder_get_physical_uri (folder),
 						   corba_listener, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
