@@ -45,11 +45,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <gnome.h>
-
-#include <bonobo.h>
-
+#include <gtk/gtksignal.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-util.h>
 #include <gal/util/e-util.h>
+#include "e-util/e-path.h"
 #include "e-local-folder.h"
 
 #include "evolution-local-storage.h"
@@ -60,9 +60,6 @@
 #define PARENT_TYPE E_TYPE_STORAGE
 static EStorageClass *parent_class = NULL;
 
-#define SUBFOLDER_DIR_NAME     "subfolders"
-#define SUBFOLDER_DIR_NAME_LEN 10
-
 struct _ELocalStoragePrivate {
 	EFolderTypeRegistry *folder_type_registry;
 	char *base_path;
@@ -131,88 +128,6 @@ shell_component_result_to_storage_result (EvolutionShellComponentResult result)
 
 /* Utility functions.  */
 
-/* Translate a storage path into a physical path on the file system.  */
-static char *
-get_physical_path (ELocalStorage *local_storage,
-		   const char *path)
-{
-	EStorage *storage;
-	ELocalStoragePrivate *priv;
-	const char *p, *newp;
-	char *dp;
-	char *real_path;
-	int real_path_len;
-	int base_path_len;
-
-	storage = E_STORAGE (local_storage);
-	priv = local_storage->priv;
-
-	/* @path is always absolute, so it starts with a slash.  The base class should
-           make sure this is the case; if not, it's broken.  */
-	g_assert (*path == G_DIR_SEPARATOR);
-	path++;
-
-	/* Calculate the length of the real path. */
-
-	real_path_len = strlen (path);
-	real_path_len++;	/* For the ending zero.  */
-
-	base_path_len = strlen (priv->base_path);
-	real_path_len += base_path_len;
-	real_path_len++;	/* For the separating slash.  */
-
-	/* Take account for the fact that we need to translate every separator into
-           `subfolders/'. */
-	p = path;
-	while (1) {
-		newp = strchr (p, G_DIR_SEPARATOR);
-		if (newp == NULL)
-			break;
-
-		real_path_len += SUBFOLDER_DIR_NAME_LEN;
-		real_path_len++; /* For the separating slash.  */
-
-		/* Skip consecutive slashes.  */
-		while (*newp == G_DIR_SEPARATOR)
-			newp++;
-
-		p = newp;
-	};
-
-	real_path = g_malloc (real_path_len);
-	dp = real_path;
-
-	memcpy (dp, priv->base_path, base_path_len);
-	dp += base_path_len;
-	*(dp++) = G_DIR_SEPARATOR;
-
-	/* Copy the mangled path.  */
-	p = path;
- 	while (1) {
-		newp = strchr (p, G_DIR_SEPARATOR);
-		if (newp == NULL) {
-			strcpy (dp, p);
-			break;
-		}
-
-		memcpy (dp, p, newp - p + 1); /* `+ 1' to copy the slash too.  */
-		dp += newp - p + 1;
-
-		memcpy (dp, SUBFOLDER_DIR_NAME, SUBFOLDER_DIR_NAME_LEN);
-		dp += SUBFOLDER_DIR_NAME_LEN;
-
-		*(dp++) = G_DIR_SEPARATOR;
-
-		/* Skip consecutive slashes.  */
-		while (*newp == G_DIR_SEPARATOR)
-			newp++;
-
-		p = newp;
-	}
-
-	return real_path;
-}
-
 static void
 new_folder (ELocalStorage *local_storage,
 	    const char *path,
@@ -234,79 +149,18 @@ new_folder (ELocalStorage *local_storage,
 }
 
 static gboolean
-load_folders (ELocalStorage *local_storage,
-	      const char *parent_path,
-	      const char *path,
-	      const char *physical_path)
+load_folder (const char *physical_path, const char *path, gpointer data)
 {
-	DIR *dir;
-	char *subfolder_directory_path;
+	ELocalStorage *local_storage = data;
+	EFolder *folder;
 
-	if (parent_path == NULL) {
-		/* On the top level, we don't have any folders and, consequently, no
-		   subfolder directory.  */
-
-		subfolder_directory_path = g_strdup (physical_path);
-	} else {
-		EFolder *folder;
-
-		/*  Otherwise, we have to load the corresponding folder.  */
-
-		folder = e_local_folder_new_from_path (physical_path);
-		if (folder == NULL)
-			return FALSE;
-
-		new_folder (local_storage, path, folder);
-
-		subfolder_directory_path = g_concat_dir_and_file (physical_path, SUBFOLDER_DIR_NAME);
+	folder = e_local_folder_new_from_path (physical_path);
+	if (folder == NULL) {
+		g_warning ("No folder metadata in %s... ignoring", physical_path);
+		return TRUE;
 	}
 
-	/* Now scan the subfolders and load them.  The subfolders are represented by
-           directories under the "SUBFOLDER_DIR_NAME" directory.  */
-
-	dir = opendir (subfolder_directory_path);
-
-	if (dir == NULL) {
-		g_free (subfolder_directory_path);
-		return FALSE;
-	}
-
-	while (1) {
-		struct stat file_stat;
-		struct dirent *dirent;
-		char *file_path;
-		char *new_path;
-
-		dirent = readdir (dir);
-		if (dirent == NULL)
-			break;
-
-		if (strcmp (dirent->d_name, ".") == 0 || strcmp (dirent->d_name, "..") == 0)
-			continue;
-
-		file_path = g_concat_dir_and_file (subfolder_directory_path,
-						   dirent->d_name);
-
-		if (stat (file_path, &file_stat) < 0) {
-			g_free (file_path);
-			continue;
-		}
-		if (! S_ISDIR (file_stat.st_mode)) {
-			g_free (file_path);
-			continue;
-		}
-
-		new_path = g_concat_dir_and_file (path, dirent->d_name);
-
-		load_folders (local_storage, path, new_path, file_path);
-
-		g_free (file_path);
-		g_free (new_path);
-	}
-
-	closedir (dir);
-	g_free (subfolder_directory_path);
-
+	new_folder (local_storage, path, folder);
 	return TRUE;
 }
 
@@ -317,27 +171,7 @@ load_all_folders (ELocalStorage *local_storage)
 
 	base_path = e_local_storage_get_base_path (local_storage);
 
-	return load_folders (local_storage, NULL, G_DIR_SEPARATOR_S, base_path);
-}
-
-static void
-notify_bonobo_listener (const Bonobo_Listener listener,
-			EStorageResult result,
-			const char *physical_path)
-{
-	CORBA_any any;
-	GNOME_Evolution_Storage_FolderResult folder_result;
-	CORBA_Environment ev;
-
-	folder_result.result = result;
-	folder_result.path = CORBA_string_dup (physical_path ? physical_path : "");
-	any._type = TC_GNOME_Evolution_Storage_FolderResult;
-	any._value = &folder_result;
-
-	CORBA_exception_init (&ev);
-	Bonobo_Listener_event (listener, "evolution-shell:folder_created", 
-			       &any, &ev);
-	CORBA_exception_free (&ev);
+	return e_path_find_folders (base_path, load_folder, local_storage);
 }
 
 
@@ -345,7 +179,6 @@ notify_bonobo_listener (const Bonobo_Listener listener,
 
 struct _AsyncCreateFolderCallbackData {
 	EStorage *storage;
-	Bonobo_Listener listener;
 
 	char *path;
 	char *display_name;
@@ -395,10 +228,6 @@ component_async_create_folder_callback (EvolutionShellComponentClient *shell_com
 
 	bonobo_object_unref (BONOBO_OBJECT (shell_component_client));
 
-	if (callback_data->listener != CORBA_OBJECT_NIL)
-		notify_bonobo_listener (callback_data->listener, storage_result,
-					callback_data->physical_path);
-
 	if (callback_data->callback != NULL)
 		(* callback_data->callback) (callback_data->storage,
 					     storage_result,
@@ -427,7 +256,6 @@ create_folder_directory (ELocalStorage *local_storage,
 	ELocalStoragePrivate *priv;
 	const char *folder_name;
 	char *physical_path;
-	char *parent_path;
 	
 	storage = E_STORAGE (local_storage);
 	priv = local_storage->priv;
@@ -437,50 +265,39 @@ create_folder_directory (ELocalStorage *local_storage,
 	
 	folder_name = g_basename (path);
 
-	if (folder_name == path + 1) {
-		/* We want a direct child of the root, so we don't need to create a
-		   `subfolders' directory.  */
-		physical_path = get_physical_path (local_storage, path);
-		parent_path = g_strdup (G_DIR_SEPARATOR_S);
-	} else {
-		char *parent_physical_path;
+	if (folder_name != path + 1) {
 		char *subfolders_directory_physical_path;
+		char *parent_path;
 		
 		/* Create the `subfolders' subdirectory under the parent.  */
 		
-		parent_path = g_strndup (path, folder_name - path - 1);
-		parent_physical_path = get_physical_path (local_storage, parent_path);
-		subfolders_directory_physical_path = g_concat_dir_and_file (parent_physical_path,
-									    SUBFOLDER_DIR_NAME);
+		parent_path = g_strndup (path, folder_name - path);
+		subfolders_directory_physical_path = e_path_to_physical (priv->base_path, parent_path);
+		g_free (parent_path);
 
 		if (! g_file_exists (subfolders_directory_physical_path)) {
 			if (mkdir (subfolders_directory_physical_path, 0700) == -1) {
 				g_free (subfolders_directory_physical_path);
-				g_free (parent_physical_path);
 				return errno_to_storage_result ();
 			}
 		}
-
-		physical_path = g_concat_dir_and_file (subfolders_directory_physical_path,
-						       folder_name);
 		g_free (subfolders_directory_physical_path);
-		g_free (parent_physical_path);
 	}
-	
+
+	physical_path = e_path_to_physical (priv->base_path, path);
+
 	/* Create the directory that holds the folder.  */
 	
+	*physical_path_return = physical_path;
 	if (mkdir (physical_path, 0700) == -1) {
-		g_free (physical_path);
 		return errno_to_storage_result ();
 	}
 
-	*physical_path_return = physical_path;
 	return E_STORAGE_OK;
 }
 
-static void
+static EStorageResult
 create_folder (ELocalStorage *local_storage,
-	       Bonobo_Listener listener,
 	       const char *path,
 	       const char *type,
 	       const char *description,
@@ -503,20 +320,18 @@ create_folder (ELocalStorage *local_storage,
 	if (component_client == NULL) {
 		if (callback != NULL)
 			(* callback) (storage, E_STORAGE_INVALIDTYPE, data);
-		if (listener != CORBA_OBJECT_NIL)
-			notify_bonobo_listener (listener, E_STORAGE_INVALIDTYPE, NULL);
-		return;
+		return E_STORAGE_INVALIDTYPE;
 	}
 	
 	g_assert (g_path_is_absolute (path));
 
 	result = create_folder_directory (local_storage, path, type, description, &physical_path);
 	if (result != E_STORAGE_OK) {
+		g_warning ("physical_path: %s", physical_path);
 		if (callback != NULL)
 			(* callback) (storage, result, data);
-		if (listener != CORBA_OBJECT_NIL)
-			notify_bonobo_listener (listener, result, NULL);
-		return;
+		g_free (physical_path);
+		return result;
 	}
 	
 	folder_name = g_basename (path);
@@ -536,50 +351,162 @@ create_folder (ELocalStorage *local_storage,
 	callback_data->description   = g_strdup (description);
 	callback_data->physical_uri  = physical_uri;
 	callback_data->physical_path = physical_path;
-	callback_data->listener      = listener;
 	callback_data->callback      = callback;
 	callback_data->callback_data = data;
-	
+
 	bonobo_object_ref (BONOBO_OBJECT (component_client));
-	
+
 	evolution_shell_component_client_async_create_folder (component_client,
 							      physical_uri,
 							      type,
 							      component_async_create_folder_callback,
 							      callback_data);
+
+	return result;
 }
 
+struct _AsyncRemoveFolderCallbackData {
+	EStorage *storage;
+
+	char *path;
+	char *physical_path;
+};
+typedef struct _AsyncRemoveFolderCallbackData AsyncRemoveFolderCallbackData;
+
+static void
+component_async_remove_folder_callback (EvolutionShellComponentClient *shell_component_client,
+					EvolutionShellComponentResult result,
+					void *data)
+{
+	AsyncRemoveFolderCallbackData *callback_data;
+	EStorageResult storage_result;
+
+	callback_data = (AsyncRemoveFolderCallbackData *) data;
+
+	storage_result = shell_component_result_to_storage_result (result);
+
+	/* If result == HASSUBFOLDERS then recurse delete the subfolders dir? */
+
+	/* FIXME: Handle errors */
+	if (result == EVOLUTION_SHELL_COMPONENT_OK) {
+		ELocalStoragePrivate *priv;
+
+		priv = E_LOCAL_STORAGE (callback_data->storage)->priv;
+
+		e_storage_removed_folder (E_STORAGE (callback_data->storage),
+					  callback_data->path);
+
+		evolution_storage_removed_folder (EVOLUTION_STORAGE (priv->bonobo_interface),
+						  callback_data->path);
+	}
+
+	bonobo_object_unref (BONOBO_OBJECT (shell_component_client));
+
+	g_free (callback_data->path);
+	g_free (callback_data->physical_path);
+	g_free (callback_data);
+}
+				
 static EStorageResult
 remove_folder_directory (ELocalStorage *local_storage,
 			 const char *path)
 {
-	char *physical_path;
+	EStorage *storage;
+	ELocalStoragePrivate *priv;
+	EFolder *folder;
+	const char *folder_name;
 	char *file_name;
+	char *physical_path;
 
-	physical_path = get_physical_path (local_storage, path);
+	priv = local_storage->priv;
 
-	/* 1. Delete the subfolder directory.  If this fails, it means that we
-	   have subfolders.  */
-	file_name = g_concat_dir_and_file (physical_path, SUBFOLDER_DIR_NAME);
-	if (g_file_exists (file_name) && rmdir (file_name) == -1)
-		return E_STORAGE_NOTEMPTY; /* FIXME? */
-	g_free (file_name);
+	storage = E_STORAGE (local_storage);
+	folder = e_storage_get_folder (storage, path);
+	folder_name = g_basename (path);
 
-	/* 2. Delete the metadata file associated with this folder.  */
+	/* Delete the metadata file associated with this folder.  */
+	physical_path = e_path_to_physical (priv->base_path, path);
 	file_name = g_concat_dir_and_file (physical_path, E_LOCAL_FOLDER_METADATA_FILE_NAME);
 	unlink (file_name);
 	g_free (file_name);
 
-	/* 3. Delete the physical directory.  */
+	/* Delete the physical directory.  */
 	if (rmdir (physical_path) == -1) {
 		g_free (physical_path);
 		return E_STORAGE_GENERICERROR;
 	}
 
 	g_free (physical_path);
+
+	/* Delete the 'subfolders' directory that this folder lies in */
+	if (folder_name != path + 1) {
+		char *subfolders_directory_physical_path;
+		char *parent_path;
+		
+		parent_path = g_strndup (path, folder_name - path);
+		subfolders_directory_physical_path = e_path_to_physical (priv->base_path, parent_path);
+		g_free (parent_path);
+
+		rmdir (subfolders_directory_physical_path);
+		g_free (subfolders_directory_physical_path);
+	}
+
 	return E_STORAGE_OK;
 }
 
+static EStorageResult
+remove_folder (ELocalStorage *local_storage,
+	       const char *path,
+	       const char *physical_uri)
+{
+	ELocalStoragePrivate *priv;
+	EStorage *storage;
+	AsyncRemoveFolderCallbackData *callback_data;
+	EvolutionShellComponentClient *component_client;
+	EStorageResult result;
+	EFolder *folder;
+	char *physical_path;
+	GList *subfolder_paths;
+	GList *p;
+
+	priv = local_storage->priv;
+
+	storage = E_STORAGE (local_storage);
+	folder = e_storage_get_folder (storage, path);
+
+	component_client = e_folder_type_registry_get_handler_for_type (priv->folder_type_registry,
+									e_folder_get_type_string (folder));
+	if (component_client == NULL)
+		return E_STORAGE_INVALIDTYPE;
+
+	physical_path = e_path_to_physical (priv->base_path, path);
+
+	if (!physical_uri)
+		physical_uri = g_strconcat ("file://", physical_path, NULL);
+
+	/* Recursively remove the subfolders */
+	subfolder_paths = e_storage_get_subfolder_paths (storage, path);
+
+	for (p = subfolder_paths; p; p = p->next)
+		remove_folder (local_storage, p->data, NULL);
+
+	callback_data = g_new (AsyncRemoveFolderCallbackData, 1);
+	callback_data->storage       = E_STORAGE (local_storage);
+	callback_data->path          = g_strdup (path);
+	callback_data->physical_path = physical_path;
+
+	bonobo_object_ref (BONOBO_OBJECT (component_client));
+
+	evolution_shell_component_client_async_remove_folder (component_client,
+							      physical_uri,
+							      component_async_remove_folder_callback,
+							      callback_data);
+
+	result = remove_folder_directory (E_LOCAL_STORAGE (local_storage), path);
+
+	return result;
+}
+
 
 /* GtkObject methods.  */
 
@@ -634,7 +561,7 @@ impl_async_create_folder (EStorage *storage,
 
 	local_storage = E_LOCAL_STORAGE (storage);
 
-	create_folder (local_storage, NULL, path, type, description, callback, data);
+	create_folder (local_storage, path, type, description, callback, data);
 }
 
 
@@ -647,8 +574,14 @@ impl_async_remove_folder (EStorage *storage,
 			  void *data)
 {
 	ELocalStorage *local_storage;
+	EStorageResult result;
 
 	local_storage = E_LOCAL_STORAGE (storage);
+
+	result = remove_folder (local_storage, path, NULL);
+
+	if (callback != NULL)
+		(* callback) (E_STORAGE (local_storage), result, data);
 }
 
 
@@ -901,9 +834,8 @@ impl_async_xfer_folder (EStorage *storage,
 
 
 /* Callbacks for the `Evolution::LocalStorage' interface we are exposing to the outside world.  */
-static void
+static int
 bonobo_interface_create_folder_cb (EvolutionStorage *storage,
-				   const Bonobo_Listener listener,
 				   const char *path,
 				   const char *type,
 				   const char *description,
@@ -914,7 +846,20 @@ bonobo_interface_create_folder_cb (EvolutionStorage *storage,
 
 	local_storage = E_LOCAL_STORAGE (data);
 
-	create_folder (local_storage, listener, path, type, description, NULL, NULL);
+	return create_folder (local_storage, path, type, description, NULL, NULL);
+}
+
+static int
+bonobo_interface_remove_folder_cb (EvolutionStorage *storage,
+				   const char *path,
+				   const char *physical_uri,
+				   void *data)
+{
+	ELocalStorage *local_storage;
+
+	local_storage = E_LOCAL_STORAGE (data);
+
+	return remove_folder (local_storage, path, physical_uri);
 }
 
 static void
@@ -935,6 +880,8 @@ bonobo_interface_update_folder_cb (EvolutionLocalStorage *bonobo_local_storage,
 
 	e_folder_set_name (folder, display_name);
 	e_folder_set_highlighted (folder, highlighted);
+
+	return;
 }
 
 
@@ -1004,6 +951,9 @@ construct (ELocalStorage *local_storage,
 	gtk_signal_connect (GTK_OBJECT (priv->bonobo_interface), "create_folder",
 			    GTK_SIGNAL_FUNC (bonobo_interface_create_folder_cb), 
 			    local_storage);
+	gtk_signal_connect (GTK_OBJECT (priv->bonobo_interface), "remove_folder",
+			    GTK_SIGNAL_FUNC (bonobo_interface_remove_folder_cb),
+			    local_storage);
 	gtk_signal_connect (GTK_OBJECT (priv->bonobo_interface), "update_folder",
 			    GTK_SIGNAL_FUNC (bonobo_interface_update_folder_cb),
 			    local_storage);
