@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* main.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,12 +23,19 @@
 
 #include <config.h>
 #include <fcntl.h>
-#include <gnome.h>
-#include <bonobo.h>
+#include <glib.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtksignal.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-init.h>
+#include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <bonobo/bonobo-main.h>
 #include <glade/glade.h>
 #include <liboaf/liboaf.h>
-#include <unicode.h>
 
 #include <gal/widgets/e-gui-utils.h>
 #include <gal/widgets/e-cursors.h>
@@ -49,7 +56,14 @@ extern char *evolution_debug_log;
 static void
 no_views_left_cb (EShell *shell, gpointer data)
 {
-	e_shell_quit (shell);
+	/* FIXME: This is wrong.  We should exit only when the shell is
+	   destroyed.  But refcounting is broken at present, so this is a
+	   reasonable workaround for now.  */
+
+	e_shell_unregister_all (shell);
+
+	bonobo_object_unref (BONOBO_OBJECT (shell));
+
 	gtk_main_quit ();
 }
 
@@ -72,20 +86,16 @@ development_warning (void)
 		NULL);
 	label = gtk_label_new (
 		/* xgettext:no-c-format */
-		_("Hi.  Thanks for taking the time to download this PREVIEW RELEASE\n"
+		_("Hi.  Thanks for taking the time to download this preview release\n"
 		  "of the Evolution groupware suite.\n"
 		  "\n"
 		  "Evolution is not yet complete. It's getting close, but there are\n"
-		  "places where features are either missing or only half working. If\n"
-		  "you can't figure out how to do something, it's probably because\n"
-		  "there's no way to do it yet! :-)\n"
-		  "\n"
-		  "We hope Evolution will be usable for you, but we still feel the\n"
-		  "need to warn you that it may: crash, lose your mail, leave stray\n"
-		  "processes running, consume 100% of your CPU, send non-\n"
-		  "compliant commands to your servers, and generally embarass you\n"
-		  "in front of your friends and co-workers. Use only as directed.\n"
+		  "places where features are either missing or only half working. \n"
 		  "\n"
+		  "If you find bugs, please report them to us at bugzilla.ximian.com.\n"
+                  "This product comes with no warranty and is not intended for\n"
+		  "individuals prone to violent fits of anger.\n"
+                  "\n"
 		  "We hope that you enjoy the results of our hard work, and we\n"
 		  "eagerly await your contributions!\n"
 		  ));
@@ -115,57 +125,78 @@ development_warning (void)
 
 /* This is for doing stuff that requires the GTK+ loop to be running already.  */
 
-static void
-new_view_on_running_shell (void)
+static gint
+idle_cb (void *data)
 {
-	CORBA_Object corba_object;
-	GNOME_Evolution_ShellView shell_view;
+	GSList *uri_list;
+	GNOME_Evolution_Shell corba_shell;
 	CORBA_Environment ev;
+	gboolean restored;
 
 	CORBA_exception_init (&ev);
 
-	corba_object = oaf_activate_from_id (E_SHELL_OAFIID, 0, NULL, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION
-	    || CORBA_Object_is_nil (corba_object, &ev)) {
-		e_notice (NULL, GNOME_MESSAGE_BOX_ERROR,
-			  _("Cannot initialize the Evolution shell."));
-		return;
-	}
-
-	shell_view = GNOME_Evolution_Shell_createNewView ((GNOME_Evolution_Shell) corba_object, STARTUP_URI, &ev);
-	if (ev._major == CORBA_NO_EXCEPTION) {
-		Bonobo_Unknown_unref ((Bonobo_Unknown) shell_view, &ev);
-		CORBA_Object_release ((CORBA_Object) shell_view, &ev);
-	}
-
-	CORBA_exception_free (&ev);
-}
-
-static gint
-idle_cb (gpointer data)
-{
-	EShellView *view;
+	uri_list = (GSList *) data;
 
 	shell = e_shell_new (evolution_directory, ! no_splash);
 	g_free (evolution_directory);
 
 	if (shell == NULL) {
-		/* A new shell cannot be created, so try to get a new view from
-                   an already running one.  */
-		new_view_on_running_shell ();
-		exit (1);
+		corba_shell = oaf_activate_from_id (E_SHELL_OAFIID, 0, NULL, &ev);
+
+		if (ev._major != CORBA_NO_EXCEPTION || corba_shell == CORBA_OBJECT_NIL) {
+			e_notice (NULL, GNOME_MESSAGE_BOX_ERROR,
+				  _("Cannot access the Evolution shell."));
+			CORBA_exception_free (&ev);
+			gtk_main_quit ();
+			return FALSE;
+		}
+
+		restored = FALSE;
+	} else {
+		gtk_signal_connect (GTK_OBJECT (shell), "no_views_left",
+				    GTK_SIGNAL_FUNC (no_views_left_cb), NULL);
+		gtk_signal_connect (GTK_OBJECT (shell), "destroy",
+				    GTK_SIGNAL_FUNC (destroy_cb), NULL);
+
+		if (uri_list == NULL)
+			restored = e_shell_restore_from_settings (shell);
+		else
+			restored = FALSE;
+
+		if (!getenv ("EVOLVE_ME_HARDER"))
+			development_warning ();
+
+		corba_shell = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
+		corba_shell = CORBA_Object_duplicate (corba_shell, &ev);
+		Bonobo_Unknown_ref (corba_shell, &ev);
 	}
 
-	gtk_signal_connect (GTK_OBJECT (shell), "no_views_left",
-			    GTK_SIGNAL_FUNC (no_views_left_cb), NULL);
-	gtk_signal_connect (GTK_OBJECT (shell), "destroy",
-			    GTK_SIGNAL_FUNC (destroy_cb), NULL);
+	if (! restored && uri_list == NULL) {
+		const char *uri = "evolution:/local/Inbox";
 
-	if (! e_shell_restore_from_settings (shell))
-		view = e_shell_new_view (shell, STARTUP_URI);
+		GNOME_Evolution_Shell_handleURI (corba_shell, uri, &ev);
+		if (ev._major != CORBA_NO_EXCEPTION)
+			g_warning ("CORBA exception %s when requesting URI -- %s", ev._repo_id, uri);
+	} else {
+		GSList *p;
 
-	if (!getenv ("EVOLVE_ME_HARDER"))
-		development_warning ();
+		for (p = uri_list; p != NULL; p = p->next) {
+			char *uri;
+
+			uri = (char *) p->data;
+
+			GNOME_Evolution_Shell_handleURI (corba_shell, uri, &ev);
+			if (ev._major != CORBA_NO_EXCEPTION)
+				g_warning ("CORBA exception %s when requesting URI -- %s", ev._repo_id, uri);
+		}
+
+		g_slist_free (uri_list);
+	}
+
+	CORBA_exception_free (&ev);
+
+	if (shell == NULL)
+		gtk_main_quit ();
 
 	return FALSE;
 }
@@ -180,11 +211,14 @@ main (int argc, char **argv)
 		POPT_AUTOHELP
 		{ NULL, '\0', 0, NULL, 0, NULL, NULL }
 	};
+	GSList *uri_list;
+	const char **args;
+	poptContext popt_context;
 
 	bindtextdomain (PACKAGE, EVOLUTION_LOCALEDIR);
 	textdomain (PACKAGE);
 
-	gnome_init_with_popt_table ("Evolution", VERSION, argc, argv, options, 0, NULL);
+	gnome_init_with_popt_table ("Evolution", VERSION, argc, argv, options, 0, &popt_context);
 
 	if (evolution_debug_log) {
 		int fd;
@@ -201,7 +235,6 @@ main (int argc, char **argv)
 	oaf_init (argc, argv);
 
 	glade_gnome_init ();
-	unicode_init ();
 	e_cursors_init ();
 
 	gnome_window_icon_set_default_from_file (EVOLUTION_IMAGES "/evolution-inbox.png");
@@ -215,12 +248,20 @@ main (int argc, char **argv)
 	/* FIXME */
 	evolution_directory = g_concat_dir_and_file (g_get_home_dir (), "evolution");
 
-	if (! e_setup (evolution_directory)) {
-		g_free (evolution_directory);
+	if (! e_setup (evolution_directory))
 		exit (1);
+
+	uri_list = NULL;
+
+	args = poptGetArgs (popt_context);
+	if (args != NULL) {
+		const char **p;
+
+		for (p = args; *p != NULL; p++)
+			uri_list = g_slist_prepend (uri_list, (char *) *p);
 	}
 
-	gtk_idle_add (idle_cb, evolution_directory);
+	gtk_idle_add (idle_cb, uri_list);
 
 	bonobo_main ();
 
